///////////////////////////////////////////////////////////////////////////////
//
//  ATTACH.PRG
//
//  Xb2.NET sample showing how to send large attachments to an HTTP server.
//  This program assumes that you are running the sample Xb2.NET webserver
//  (WEBSERVE.EXE) on the same local machine.
//
//  Copyright (c) 2006-2009, Xb2.NET inc.
//  All rights reserved.
//
//  Author:   Boris Borzic
//  Email:    support@xb2.net
//  Web:      http://www.xb2.net
//
///////////////////////////////////////////////////////////////////////////////


#include "xb2net.ch"
#include "common.ch"
#include "directry.ch"

#pragma  library ("xb2net.lib")

#xtranslate NTrim(<n>) =>  LTrim(Str(<n>))

// uncomment next line to send attachments using DIME (Direct Internet Message Encapsulation)
//#define DIME

//-----------------------------------------------------------------------------
procedure Main()
   Local i, imax, oReq, oResp, aFiles

   oReq := xbHTTPClient():new()
   oReq:SetCallBack({|a,b| DisplayStatus(a,b)})
   #ifdef DIME
      oReq:Payload:SetFormat(PAYLOAD_DIME)
   #else
      oReq:Payload:SetFormat(PAYLOAD_MP_FORMDATA)
   #endif

   aFiles := Directory(".\*.exe")
   imax   := min(len(aFiles), 5)

   // some form vars:
   oReq:Payload:Add( "Test Application", PL_DATA_STRING, nil, nil, "UserName" )
   oReq:Payload:Add( "test@xb2.net", PL_DATA_STRING, nil, nil, "UserEmail" )

   ?? "Sending the following files as attachments:"
   for i := 1 to imax
      ? Str(i,4) + " -", aFiles[i,F_NAME]
      #ifdef DIME
         // when using DIME, we can provide the remote server with a suggested filename
         // by using a "content-disposition" header
         oReq:Payload:Add( aFiles[i,F_NAME], PL_DATA_FILE, nil, nil, "UserFile"+NTrim(i), ;
            'content-disposition: attachment; filename="' + aFiles[i,F_NAME] + '"' )
      #else
         oReq:Payload:Add( aFiles[i,F_NAME], PL_DATA_FILE, nil, nil, "UserFile"+NTrim(i) )
      #endif
   next

   oResp := oReq:Execute("http://localhost/upload?")
   if oResp == NIL
      MsgBox("An error occurred trying to send request to HTTP server!" + chr(10) +;
             "Error code: " + NTrim(oReq:ErrorCode) + chr(10) + oReq:ErrorMessage )
      Return
   endif

   ?
   ? "Done; this is the HTTP response..."
   SetColor("w/b")
   MemoEdit(oResp:AsString(), Row()+1, 0, MaxRow(), MaxCol())

   Return

//-----------------------------------------------------------------------------
procedure DisplayStatus( nStatus, x )
   Static oXbp
   Local oWindow

   if oXbp == nil
      oXbp := XbpCrt():new(,,{372,165},15,33,"STATUS")
      oXbp:autoFocus := .F.
      oXbp:create()
   endif

   oWindow := SetAppWindow(oXbp)

   if valtype(x) == "O" .and. x:isDerivedFrom("xbSocket")
      ? nStatus, x:SendCount, x:RecvCount
   else
      ? nStatus, x
   endif

   DispOutAt(0,0,"StatusCode  SendCount  RecvCount", "gr+,w")
   SetAppWindow(oWindow)
   Return

