///////////////////////////////////////////////////////////////////////////////
//
//  FTPTEST.PRG
//
//  Xb2.NET xbFTPClient test program
//
//  Copyright (c) 2004-2009, Xb2.NET inc.
//  All rights reserved.
//
//  Author:   Boris Borzic
//  Email:    support@xb2.net
//  Web:      http://www.xb2.net
//
///////////////////////////////////////////////////////////////////////////////



#include "xb2net.ch"


procedure Main( cServer, cUserID, cPassword, cPath, cFlag, cFlag2, cFlag3 )
   Local oFtp, bCallback

   if PCount() == 0 .or. "/?" $ cServer
      ?
      ? "USAGE: " + Lower(AppName()) + " URL [uid] [pwd] [path] [/passive] [/securedata] [/show]"
      ?
      ? "  URL         - FTP/FTPS server address or domain name"
      ? "  uid         - login userid, default is 'anonymous'"
      ? "  pwd         - login password, default is a null string"
      ? "  path        - switch to a specific directory"
      ? "  /passive    - use passive mode if connecting through a firewall"
      ? "  /securedata - use secure data channel (when using FTPS)"
      ? "  /show       - display detailed progress information"
      ?
      ? "EXAMPLE:"
      ? Lower(AppName()) + " ftps://TheBatCave.com batman 1o7T&x0 batfiles /passive/securedata"
      ?
      inkey(0)
      quit
   endif

   // make the console window bigger
   SetAppWindow():Configure( NIL, NIL, {0,23}, 45, 80, AppName())

   cFlag := lower(var2char(cFlag) + var2char(cFlag2) + var2char(cFlag3))

   oFtp := xbFTPClient():new()

   oFtp:PassiveMode      := "/passive" $ cFlag
   oFtp:SecureDataChannel:= "/securedata" $ cFlag
   oFtp:RecvTimeout(10000)
   oFtp:SendTimeout(10000)

   if "/show" $ cFlag
      // set callback to see what's going on
      // this codeblock will display status/progress info in terminal window:
      SetColor("w+/n")
      oFtp:SetCallBack({|a,b| DisplayStatus(a,b)})
   endif

   // if the FTP server needs a certificate from the client do this:
   //
   // (1) create xbSSLContext object:
   //
   //    oSSL := xbSSLContext():new(TLS_client_method)
   //
   // (2a) if the certificate is in PKCS12 (.PFX) format do this:
   //
   //    if ! oSSL:UsePKCS12File(".\MyCert.pfx", "MyPassword")
   //       ? "Certificate Error:", xbSSLGetErrorString()
   //    endif
   //
   // (2b) if the certificate is in PEM (base64 encoded) format do this:
   //
   //    oSSL:UseCertificateFile(".\MyCert.pem")
   //    oSSL:UsePrivateKeyFile(".\MyCert.pem")
   //
   // (3) attach oSSL to oFtp:
   //
   //    oFtp:SetSSLContext(oSSL)
   //


   // connect to FTP server
   if ! oFtp:Connect( cServer )
      Terminate( "*** ERROR: Failed to connect to FTP server: " + cServer )
   endif

   // do login
   if oFtp:Login(cUserID, cPassword)
      ? "Connected to ", cServer
   else
      Terminate( "*** ERROR: Login failed!" )
   endif

   // get current directory
   ? "current dir =", oFtp:GetCurrentDirectory()

   if !empty(cPath)
      // change directory
      if ! oFtp:SetCurrentDirectory(cPath)
         Terminate( "*** ERROR: Unable to change path to: " + cPath )
      endif

      ? "path changed, current dir =", oFtp:GetCurrentDirectory()
   endif

   ShowDir( oFtp )

   ? "Create directory 'TEST'"
   if !oFtp:CreateDirectory("TEST")
      Terminate( "*** ERROR: Failed to create directory 'TEST'", .F.)
   endif

   ? "Change path to 'TEST'"
   if ! oFtp:SetCurrentDirectory("TEST")
      Terminate( "*** ERROR: Unable to change path to: 'TEST'", .F. )
   endif

   ? "path changed, current dir =", oFtp:GetCurrentDirectory()

   ShowDir( oFtp )

   ? "Upload file: " + AppName(.t.)
   if oFtp:PutFile(AppName(.T.), "FTPTest.exe")
      ? "Transmitted:", oFtp:BytesTransferred, "bytes in", oFtp:ElapsedTime, "seconds"
   else
      Terminate( "*** ERROR: Unable to upload file!")
   endif

   ShowDir( oFtp )

   ? "Rename file FTPTest.exe -> FTPTEST-1.EXE"
   if ! oFtp:RenameFile("FTPTest.exe", "FTPTEST-1.EXE")
      Terminate( "*** ERROR: Unable to rename file!")
   endif

   ShowDir( oFtp )

   ? "Download file: FTPTEST-1.EXE"
   if oFtp:GetFile("FTPTEST-1.EXE")
      ? "Received:", oFtp:BytesTransferred, "bytes in", oFtp:ElapsedTime, "seconds"
   else
      Terminate( "*** ERROR: Unable to download file!")
   endif

   ?
   ? "Delete file FTPTEST-1.EXE"
   if ! oFtp:DeleteFile("FTPTEST-1.EXE")
      Terminate( "*** ERROR: Unable to delete file!")
   endif

   ShowDir( oFtp )

   ? "Change to parent directory and delete directory 'TEST'"
   oFtp:SetCurrentDirectory()
   if ! oFtp:DeleteDirectory("TEST")
      Terminate( "*** ERROR: Unable to delete directory!", .F.)
   endif

   ShowDir( oFtp )

   oFtp:Destroy()

   Terminate("end of test...")
   Return


procedure DisplayStatus( nStatus, cMessage )
   Local cColor := SetColor(iif(isFTP_ST(nStatus), "n+/n", "r/n"))
   ? nStatus, cMessage
   SetColor(cColor)
   Return

procedure Terminate( cMessage, lQuit )
   Local cColor

   if cMessage != NIL
      cColor := SetColor("gr+/n")
      ? cMessage
      inkey(0)
      SetColor(cColor)
   endif
   if lQuit == NIL .or. lQuit
      quit
   endif
   Return

Procedure ShowDir( oFtp )
   Local aDir, cColor

   cColor := SetColor("g/n")
   ?
   ? "Push a key to get directory listing..."
   SetColor(cColor)
   inkey(0)
   aDir := oFtp:Directory()
   if empty(aDir)
      ? "dir: <empty>"
   else
      ? "dir: "
      AEval(aDir, {|x| QOut(x)})
   endif
   ?
   Return

