///////////////////////////////////////////////////////////////////////////////
//

//  GETWEB.PRG
//
//  Xb2.NET sample showing how to connect to a web server and retrieve a file
//
//  Copyright (c) 2002-2016, Xb2.NET inc.
//  All rights reserved.
//
//  Author:   Boris Borzic
//  Email:    support@xb2.net
//  Web:      http://www.xb2.net
//
//  You can specify a URL on the command line as follows:
//
//  GETWEB "http://www.xb2.net:80/index.htm"
//  GETWEB localhost
//  GETWEB 127.0.0.1
//
///////////////////////////////////////////////////////////////////////////////

* GETWEB vlastne sli na zskanie sboru z WEBSERVERA

#include "xb2net.ch"
#include "common.ch"

#pragma  library ("xb2net.lib")

#xtranslate NTrim(<n>) =>  LTrim(Str(<n>))


* tu sa d otestova akou metdou, Method1 alebo Method2
* otestovan, asi Method2 je low level dao...
#define USE_HTTPCLIENT // comment out to try Method2


//-----------------------------------------------------------------------------
function Main( cURL )

* nastavenie zkladej adresy URL
   DEFAULT cURL TO "http://www.sqlexpress.net"

   // make sure that the URL is an absolute reference
   cURL := AllTrim(cURL)
   if ! Lower(Left(cURL, 7)) == "http://"  .and. ! Lower(Left(cURL, 7)) == "http:\\" .and. ;
      ! Lower(Left(cURL, 8)) == "https://" .and. ! Lower(Left(cURL, 8)) == "https:\\"
      cURL := "http://" + cURL
   endif

   SetAppWindow():SetTitle( cURL )

   #ifdef USE_HTTPCLIENT
      Return Method1( cURL )
   #else
      Return Method2( cURL )
   #endif

//-----------------------------------------------------------------------------
static function Method1( cURL )
   Local oHttp, oResponse, cColor

   cColor := SetColor("r+/n")
   ?? "Method 1: using higher level xbHTTPClient class"
   SetColor(cColor)

   // create an HTTP client instance
   oHttp := xbHTTPClient():new()

   // Comment out next line to use the Windows WinINet library instead of raw sockets.
   // Note that WinINet may be slower, but it does provide automatic support for the
   // Windows credential cache, security zones, cookies, connection settings & socks proxy
   //
   // oHttp:Transport := VIA_WININET

   ? "executing HTTP request..."	// tu asi iba vykon o je na cURL a ulo do objektu oResponse ?
   oResponse := oHttp:Execute( cURL )

   if oResponse == NIL
      MsgBox("Error:" + str(oHttp:ErrorCode) + chr(10) + oHttp:ErrorMessage + chr(10) + oHttp:ErrorSource)
      Return .f.
   endif

* a tu zobraz o natal v memoedit	
   DispResponse( oResponse )

   Return .t.

//-----------------------------------------------------------------------------
static function Method2( cURL )
   Local i, cHost, nPort, cPath, cQuery, oURI, oSock, oRequest, oResponse, oSSLContext, cColor, isSecure := .f.

   cColor := SetColor("r+/n")
   ?? "Method 2: using low level socket and HTTP request/response objects"
   SetColor(cColor)

   // if a URL string is supplied, then parse it
   if !Empty( cURL )
      oURI  := xbURI():new( cURL )
      cHost := oURI:Authority()
      cPath := oURI:Path()
      cQuery:= oURI:Query()
      isSecure := Lower(oURI:Scheme) == "https"

      if (i := At(":", cHost)) > 0
         nPort := Val(SubStr(cHost,i+1))
         cHost := Left(cHost,i-1)
      endif
   endif

   if Empty(cHost)
      cHost := "www.sqlexpress.net"
   endif

   if Empty(nPort)
      // default port for HTTP / HTTPS server
      nPort := iif(isSecure, 443, 80)
   endif

   if Empty(cPath)
      cPath := "/"
   endif


   ? "Socket DLL:", xbSocket():DllDescription, ;
     ", version:", xbSocket():DllVersion, ;
     ", status:", xbSocket():DllStatus

   if isSecure
      oSSLContext := xbSSLContext():new( TLS_client_method )
      ? "Using", oSSLContext:Version[SSLEAY_VERSION]
   endif

   // create a socket
   oSock := xbSocket():New(AF_INET, SOCK_STREAM, IPPROTO_IP, oSSLContext)

   if oSock:ErrorCode > 0
      DispError("Unable to create a socket!", oSock, oSSLContext )
      Return .f.
   endif

   // and connect to the host
   if ! oSock:Connect(cHost, nPort)
      DispError("Unable to connect to: " + cHost, oSock, oSSLContext)
      Return .f.
   endif

   ? "connected to: ", oSock:RemoteName, ", address:", oSock:InetNtoA(oSock:RemoteAddr), ", port:", NTrim(oSock:RemotePort)


   // since we are going to connect to a web server, let's create an xbHTTPRequest
   // and an xbHTTPResponse object for our socket to help us format HTTP requests
   // and parse responses from the web server

   oRequest  := xbHTTPRequest():new( oSock )
   oResponse := xbHTTPResponse():new( oSock )

   // now we need to format the HTTP request

   oRequest:Command := "GET"
   oRequest:KeepAlive(.f.)                   // tell server to close the connection after replying
   oRequest:Host(cHost + ":" + NTrim(nPort)) // eg. "www.sqlexpress.net:80"
   oRequest:Path(cPath)                      // eg: "/index.htm")
   oRequest:Query(cQuery)

   // now send the request
   ?
   ? "sending HTTP request..."

   if oRequest:send() == SOCKET_ERROR
      DispError("An error occurred trying to send request to HTTP server!", oSock, oSSLContext)
      Return .f.
   endif

   ? "sent: " + NTrim(oSock:BytesSent) + " bytes"
   ?
   ? "waiting for reply..."

   if ! oResponse:Recv()
      DispError("No reply from HTTP server, connection has been dropped!", oSock, oSSLContext)
      Return .f.
   endif

   DispResponse( oResponse )

   // DON'T FORGET TO DESTROY THE OBJECT IN ORDER TO RELEASE SYSTEM HANDLES!!
   FreeResources( oSock, oSSLContext )
   Return .t.

//-----------------------------------------------------------------------------
static procedure DispError(cMsg, oSock, oSSLContext)
   MsgBox(cMsg + chr(10) + "Error code: " + NTrim(oSock:ErrorCode) + " (" + oSock:ErrorMessage + ")")
   FreeResources( oSock, oSSLContext )
   Return

//-----------------------------------------------------------------------------
static procedure FreeResources( oSock, oSSLContext )
   oSock:destroy()
   if oSSLContext != NIL
      oSSLContext:destroy()
   endif
   Return

* tu zobraz o sa mu podarilo zska asi z webservera, tj. poet bytov at, a samotn obsah 
//-----------------------------------------------------------------------------
static procedure DispResponse( oResponse )
   ? "received      :", NTrim(oResponse:BytesReceived), "bytes"
   ? "start line    :", oResponse:StartLine
   ? "content type  :", oResponse:ContentType()
   ? "content length:", oResponse:ContentLength()
   ? "last modified :", oResponse:LastModified()
   ?
   ? "This is the response content..."
   SetColor("w/b")
   MemoEdit(oResponse:Content, Row()+1, 0, MaxRow(), MaxCol())
   Return


