///////////////////////////////////////////////////////////////////////////////
//
//  SOAPEXEC.PRG
//
//  Xb2.NET sample showing how to execute remote SOAP methods
//
//  Copyright (c) 2002-2014, Xb2.NET inc.
//  All rights reserved.
//
//  Author:        Boris Borzic
//  Email:         support@xb2.net
//  Web:           http://www.xb2.net
//
///////////////////////////////////////////////////////////////////////////////


#include "xb2net.ch"
#include "common.ch"
#include "inkey.ch"
#include "achoice.ch"

#pragma  library ("xb2net.lib")

#define CRLF              (Chr(13)+Chr(10))
#xtranslate NTrim(<n>) =>  LTrim(Str(<n>))

static GetList[0]
static scHost          := "http://localhost:80                          "
static scProxyHost     := "                                             "
static snProxyPort     := 0
static scProxyUserID   := "                    "
static scProxyPassword := "                    "

static saMenu := {{"--- require internet connection --------------------------", nil             },;
                  {"Get your external IP address"                              , "GetMyIP"       },;
                  {"Get country name from IP address"                          , "IP2Country"    },;
               ;//{"GetPlaceList from TerraService"                            , "GetPlaceList"  },; // this service is no longer active
                  {"Check if a European VAT number is valid"                   , "CheckVAT"      },;
                  {"Get currency conversion rate"                              , "ConversionRate"},;
               ;//{"Send and Receive MTOM attachment"                          , "MTOMTest"      },; // this service is no longer active
                  {""                                                          , nil             },;
                  {"--- require WEBSERVE.EXE to be running -------------------", nil             },;
                  {"Get current date and time of Xb2.NET server"               , "GetDateTime"   },;
                  {"Save a message on Xb2.NET server"                          , "SaveMessage"   },;
                  {"Request Xb2.NET server to echo back all parameters"        , "EchoAll"       },;
                  {"Send and receive DIME attachments"                         , "DIMETest"      },;
                  {""                                                          , nil             },;
                  {"----------------------------------------------------------", nil             },;
                  {"Proxy setup  (* required when connecting via proxy *)"     , "ProxySetup"    }}

PROCEDURE DbeSys(); Return

//-----------------------------------------------------------------------------
PROCEDURE Main()
   Local nItem, oResult, cColor1 := SetColor(), cColor2

   set scoreboard off
   set cursor on

   while (nItem := SelectItem()) > 0
      // execute the selected function
      SetColor(cColor1); CLS
      cls
      oResult := &( saMenu[nItem,2]+"()" )

      if oResult == Nil
         // canceled
      else
         cColor2 := iif(Empty(oResult:Action) .or. Upper(oResult:Action) == "FAULT", "w/r", "w/b")
         ? "Result.......: "
         SetColor(cColor2)
         ?? oResult:Action
         SetColor(cColor1)
         ? "Returned data:"
         SetColor(cColor2)

         if oResult:Payload:isEmpty()
            MemoEdit(ExpandVars(oResult), Row()+1, 0, MaxRow(), MaxCol())
         else
            MemoEdit(ExpandVars(oResult) + CRLF + "ATTACHMENTS:" + CRLF + var2char(oResult:Payload:List), Row()+1, 0, MaxRow(), MaxCol())
         endif

         SetColor(cColor1)
      endif
      clear typeahead
   end

   Return

//-----------------------------------------------------------------------------
STATIC FUNCTION ExpandVars(oData, nIndent)
   Local i, xVal
   Local aVarList := oData:GetVar()
   Local imax := len(aVarList)
   Local cRet := ""

   DEFAULT nIndent TO 0

   for i := 1 to imax
      xVal := aVarList[i,VAR_VALUE]
      if ValType(xVal) == "O" .and. xVal:isDerivedFrom("xbComplexType")
         cRet += space(nIndent) + aVarList[i,VAR_NAME] + CRLF + ExpandVars(xVal,nIndent+2) + CRLF
      else
         cRet += space(nIndent) + var2char(aVarList[i]) + CRLF
      endif
   next
   Return cRet

//-----------------------------------------------------------------------------
STATIC FUNCTION SelectItem()
   SetColor( "w/n,w+/b,,,n+/n" ); CLS
   @ 1,0 TO 1,MaxCol()
   @ 1,2 SAY " Xb2.NET SOAP client samples "
   Return AChoice(3,3,len(saMenu)+3,60, GetMenu(), GetMenuSelect(), {|a,b,c|MenuUdf(a,b,c)})

//-----------------------------------------------------------------------------
STATIC FUNCTION GetMenu()
   Local aRet := Array(Len(saMenu))
   AEval(saMenu, {|a,i|aRet[i] := a[1]})
   Return aRet

//-----------------------------------------------------------------------------
STATIC FUNCTION GetMenuSelect()
   Local aRet := Array(Len(saMenu))
   AEval(saMenu, {|a,i|aRet[i] := a[2]!=NIL})
   Return aRet

//-----------------------------------------------------------------------------
STATIC FUNCTION MenuUdf( nMode, nElement, nRow )
   Local nReturn := AC_CONT        // default: continues
   Local nKey    := LastKey()

   if nMode == AC_EXCEPT
      do case
      case nKey == K_RETURN
         nReturn := AC_SELECT      // select element
      case nKey == K_ESC
         nReturn := AC_ABORT       // terminate AChoice()
      case nKey != K_LEFT .and. nKey != K_RIGHT
         nReturn := AC_GOTO        // jump to element
      endcase
   endif

   Return nReturn

//-----------------------------------------------------------------------------
FUNCTION ProxySetup()

   @ 0,0 SAY "PROXY CONNECTION SETUP" COLOR "g+/n"

   @ 2,0 SAY "The following info must be supplied when connecting through a proxy:"
   @ 4,0 SAY "Proxy host name or IP address:" GET scProxyHost
   @ 5,0 SAY "            Proxy port number:" GET snProxyPort
   @ 6,0 SAY "                    User Name:" GET scProxyUserID
   @ 7,0 SAY "                     Password:" GET scProxyPassword
   READ

   Return Nil

//-----------------------------------------------------------------------------
FUNCTION GetMyIP()
   Local oSoap, oResult, cIP := space(15)

   @ 0,0 SAY "Returns your external IP address" COLOR "g+/n"
   oSoap := xbSOAPEnvelope():new()
   oSoap:SetStyle( SOAP_STYLE_DOCUMENT )
   oSoap:SetProxy(scProxyHost, snProxyPort, scProxyUserID, scProxyPassword)

   @ 2,0 SAY "Executing request..."

   oResult := oSoap:Execute("http://ws.xb2.net", "GetMyIP" )
   @ 2,0 SAY Space(80)

   Return oResult

//-----------------------------------------------------------------------------
FUNCTION IP2Country()
   Local oSoap, oResult, cIP := space(15)

   @ 0,0 SAY "Get country name from IP address" COLOR "g+/n"
   @ 1,0 SAY "(if no IP address is provided, method returns your country name)" COLOR "g+/n"
   @ 3,0 SAY "Enter IP Address:" GET cIP PICTURE "999.999.999.999"
   READ

   if LastKey() != K_ESC
      oSoap := xbSOAPEnvelope():new()
      oSoap:SetStyle( SOAP_STYLE_DOCUMENT )
      oSoap:SetProxy(scProxyHost, snProxyPort, scProxyUserID, scProxyPassword)

      oSoap:SetVar("IP", strtran(cIP, " ", ""))

      @ 4,0 SAY "Executing request..."

      oResult := oSoap:Execute("http://ws.xb2.net", "IP2Country" )

      @ 4,0 SAY Space(80)
   endif

   Return oResult

//-----------------------------------------------------------------------------
FUNCTION GetPlaceList()
   Local oSoap, oResult
   Local cPlaceName := "Chicago           "
   Local nItems     := 10

   @ 0,0 SAY "Fetch place list given a name" COLOR "g+/n"
   @ 2,0 SAY "Enter Place Name  :" GET cPlaceName
   @ 3,0 SAY "Max Items Returned:" GET nItems
   READ

   if LastKey() != K_ESC
      oSoap := xbSOAPEnvelope():new()
      oSoap:SetStyle( SOAP_STYLE_DOCUMENT )
      oSoap:SetProxy(scProxyHost, snProxyPort, scProxyUserID, scProxyPassword)
      oSoap:SendTimeout(8000)
      oSoap:RecvTimeout(8000)

      oSoap:NameSpace := "http://terraserver-usa.com/terraserver/"
      oSoap:SetVar("placeName", AllTrim(cPlaceName))
      oSoap:SetVar("MaxItems", NTrim(nItems))
      oSoap:SetVar("imagePresence", .t.)

      @ 4,0 SAY "Executing request..."

      oResult := oSoap:Execute("http://terraservice.net/terraService.asmx", "GetPlaceList", "http://terraserver-usa.com/terraserver/GetPlaceList")

      // this is an alternate server that may be used if the above server is unavailable:
      //oSoap:NameSpace := "http://terraservice-usa.com/"
      //oResult := oSoap:Execute("http://65.54.135.103/TerraService2.asmx", "GetPlaceList", "http://terraservice-usa.com/GetPlaceList")

      @ 4,0 SAY Space(80)
   endif

   Return oResult

//-----------------------------------------------------------------------------
FUNCTION CheckVAT()
   Local oSoap, oResult
   Local cCode  := "  "
   Local cVat   := space(12)

   @ 0,0 SAY "Check if a European VAT # is valid" COLOR "g+/n"
   @ 2,0 SAY "Country Code: " GET cCode PICTURE "@!"
   @ 3,0 SAY "VAT Number  : " GET cVat  PICTURE "@!"
   READ

   if LastKey() != K_ESC
      oSoap := xbSOAPEnvelope():new()
      oSoap:SetStyle( SOAP_STYLE_DOCUMENT )
      oSoap:SetProxy(scProxyHost, snProxyPort, scProxyUserID, scProxyPassword)
      oSoap:SendTimeout(8000)
      oSoap:RecvTimeout(8000)

      oSoap:NameSpace := "urn:ec.europa.eu:taxud:vies:services:checkVat:types"
      oSoap:SetVar("countryCode", upper(trim(cCode)))
      oSoap:SetVar("vatNumber"  , upper(trim(cVat)))

      @ 4,0 SAY "Executing request..."

      oResult := oSoap:Execute("http://ec.europa.eu/taxation_customs/vies/services/checkVatService", "checkVat")

      @ 4,0 SAY Space(80)
   endif

   Return oResult

//-----------------------------------------------------------------------------
FUNCTION ConversionRate ()
   Local oSoap, oResult
   Local cFrom := "EUR"
   Local cTo   := "USD"

   @ 0,0 SAY "Get conversion rate from one currency to another" COLOR "g+/n"
   @ 2,0 SAY "From Currency: " GET cFrom PICTURE "@!"
   @ 3,0 SAY "To Currency  : " GET cTo   PICTURE "@!"
   READ

   if LastKey() != K_ESC
      oSoap := xbSOAPEnvelope():new()
      oSoap:Version := "1.2"
      oSoap:SetStyle( SOAP_STYLE_DOCUMENT )
      oSoap:SetProxy(scProxyHost, snProxyPort, scProxyUserID, scProxyPassword)
      oSoap:SendTimeout(8000)
      oSoap:RecvTimeout(8000)

      oSoap:NameSpace := "http://www.webserviceX.NET/"
      oSoap:SetVar("FromCurrency", upper(trim(cFrom)))
      oSoap:SetVar("ToCurrency"  , upper(trim(cTo)))

      @ 4,0 SAY "Executing request..."

      oResult := oSoap:Execute("http://www.webservicex.net/CurrencyConvertor.asmx", "ConversionRate")

      @ 4,0 SAY Space(80)
   endif

   Return oResult

//-----------------------------------------------------------------------------
FUNCTION GetDateTime()
   Local oSoap, oResult

   @ 0,0 SAY "This method will get the current date and time of the Xb2.NET HTTP server" COLOR "g+/n"
   @ 2,0 SAY "Server name or IP address:" GET scHost
   READ

   if LastKey() != K_ESC
      oSoap := xbSOAPEnvelope():new()
      oSoap:SetProxy(scProxyHost, snProxyPort, scProxyUserID, scProxyPassword)

      @ 3,0 SAY "Executing request..."

      oResult := oSoap:Execute( Trim(scHost), "GetDateTime" )

      @ 3,0 SAY Space(80)
   endif

   Return oResult

//-----------------------------------------------------------------------------
FUNCTION SaveMessage()
   Local oSoap, oResult
   Local cName    := space(40)
   Local cMessage := space(40)

   @ 0,0 SAY "This method will save a message on the Xb2.NET HTTP server" COLOR "g+/n"
   @ 2,0 SAY "Server name or IP address:" GET scHost
   @ 4,0 SAY "                Your name:" GET cName
   @ 5,0 SAY "     Text message to save:" GET cMessage
   READ

   if LastKey() != K_ESC
      oSoap := xbSOAPEnvelope():new()
      oSoap:SetProxy(scProxyHost, snProxyPort, scProxyUserID, scProxyPassword)

      oSoap:SetVar("UserName", cName)
      oSoap:SetVar("Message" , cMessage)

      @ 6,0 SAY "Executing request..."

      oResult := oSoap:Execute( Trim(scHost), "SaveMessage" )

      @ 6,0 SAY Space(80)
   endif

   Return oResult

//-----------------------------------------------------------------------------
FUNCTION EchoAll()
   Local oSoap, oResult
   Local cChar    := "This is a sample character string."
   Local dDate    := date()
   Local lLogical := .t.
   Local nInteger := 12345
   Local nDecimal := 1234.567
   Local aArray   := {{"This is subarray [1,1]", 10, .t.}, {"This is subarray [2,1]", 20.2, .f., date()}}
   Local cMessage := "Eureka!" + chr(10) + "I can pass a codeblock using SOAP!"
   Local bBlock   := {||MsgBox(cMessage)}
   Local cError   := "Just testing how to pass an object using SOAP"
   Local cFile    := PadR(AppName(), 30)
   Local oObject


   @ 0,0 SAY "This method requests the Xb2.NET HTTP server to echo back all parameters" COLOR "g+/n"
   @ 2,0 SAY "Server name or IP address:" GET scHost
   @ 4,0 SAY "PARAMETERS:      MyString:" GET cChar
   @ 5,0 SAY "                   MyDate:" GET dDate
   @ 6,0 SAY "                MyLogical:" GET lLogical
   @ 7,0 SAY "                MyInteger:" GET nInteger
   @ 8,0 SAY "                MyDecimal:" GET nDecimal
   @ 9,0 SAY "      MyCodeBlock message:" GET cMessage
   @10,0 SAY "     MyObject description:" GET cError
   @11,0 SAY "       My file attachment:" GET cFile VALID Empty(cFile) .or. FExists(cFile)

   READ

   if LastKey() != K_ESC
      oSoap := xbSOAPEnvelope():new()
      oSoap:SetProxy(scProxyHost, snProxyPort, scProxyUserID, scProxyPassword)

      oSoap:SetVar("MyString"   , cChar   )
      oSoap:SetVar("MyDate"     , dDate   )
      oSoap:SetVar("MyLogical"  , lLogical)
      oSoap:SetVar("MyInteger"  , nInteger)
      oSoap:SetVar("MyDecimal"  , nDecimal)
      oSoap:SetVar("MyArray"    , aArray  )
      oSoap:SetVar("MyCodeBlock", bBlock  )

      // create an object and save it in the SOAP envelope
      oObject := Error():New()
      oObject:subSystem   := "SOAP-TEST"
      oObject:subCode     := 999
      oObject:description := cError
      oObject:operation   := ":SetVar"
      oObject:severity    := 0

      oSoap:SetVar("MyObject"   , oObject )

      // add file attachment
      if !empty(cFile)
         oSoap:Payload:Add(RTrim(cFile), PL_DATA_FILE)
      endif

      @ 12,0 SAY "Executing request..."

      oResult := oSoap:Execute( Trim(scHost), "EchoAll" )

      @ 12,0 SAY Space(80)

      if oResult:Action == "EchoAllResponse"
         ? "MyObject:description:", oResult:GetVar("MyObject"):description
         ?
         Eval(oResult:GetVar("MyCodeBlock"))

         oResult:Payload:Erase() // delete temp files from disk
      endif
   endif

   Return oResult

//-----------------------------------------------------------------------------
FUNCTION DIMETest()
   Local i, oSoap, oResult, aFile[3], aPayload

   AFill(aFile, space(45))
   aFile[1] := PadR(".\SOAPEXEC.EXE", 45)

   @ 0,0 SAY "This method will send one or more files to an Xb2.NET SOAP server for" COLOR "g+/n"
   @ 1,0 SAY "base64 encoding. The files are sent and received as DIME attachments." COLOR "g+/n"

   @ 3,0 SAY "Server name or IP address:" GET scHost
   for i := 1 to len(aFile)
      @ 4+i,0 SAY PadL("File Name " + NTrim(i) + ":",26)  GET aFile[i]
   next
   READ

   if LastKey() != K_ESC
      oSoap := xbSOAPEnvelope():new()
      oSoap:SetProxy(scProxyHost, snProxyPort, scProxyUserID, scProxyPassword)

      for i := 1 to len(aFile)
         if !empty(aFile[i])
            oSoap:SetVar("File" + NTrim(i), RTrim(aFile[i]))
            oSoap:Payload:Add(aFile[i], PL_DATA_FILE, "", PL_TYPE_UNKNOWN, RTrim(aFile[i]))
         endif
      next

      @ 5+Len(aFile),0 SAY "Executing request..."
      oResult := oSoap:Execute( Trim(scHost), "B64Encode_Payload" )

      // process the returned payload
      aPayload := oResult:Payload:List
      for i := 1 to len(aPayload)
         if aPayload[i, PL_DATA_T] == PL_DATA_FILE
            FErase( oResult:GetVar("File"+NTrim(i)) + ".b64")
            FRename( aPayload[i,PL_DATA], oResult:GetVar("File"+NTrim(i)) + ".b64")
         else
            xbSaveToFile( aPayload[i,PL_DATA], oResult:GetVar("File"+NTrim(i)) + ".b64", .F.)
         endif
      next
      oResult:Payload:Erase() // delete temp files from disk

      @ 5+Len(aFile),0 SAY Space(80)
   endif

   Return oResult

//-----------------------------------------------------------------------------
FUNCTION MTOMTest()
   Local oSoap, oReq, oResult, aPayload, xFileData
   Local cFileName := AppName()
   Local cURL := "http://acccertificaten.vecozo.nl/Webservices/MTOMTest/MTomDummy.asmx"
   Local cSOAPAction := '"http://www.org.vecozo.nl/MTOMTest/v1/AddFile"'

   set cursor off
   cls
   ? "Sending file " + cFileName + " as MTOM attachment..."
   ?

   oSoap := xbSOAPEnvelope():new()
   oSoap:Version := "1.2"
   oSoap:SetStyle( SOAP_STYLE_DOCUMENT )
   oSoap:SetProxy(scProxyHost, snProxyPort, scProxyUserID, scProxyPassword)
   oSoap:SendTimeout(8000)
   oSoap:RecvTimeout(8000)
   oSoap:Payload:SetFormat( PAYLOAD_MTOM )
   oSoap:NameSpace := "http://www.org.vecozo.nl/MTOMTest/v1"

   // callback to display progress
   oSoap:SetCallBack({|a,b| DisplayStatus(a,b)})

   // create "request" structure as required by server
   oReq := xbComplexType():New()
   oReq:Payload := oSoap:Payload
   oReq:AddVar("EmailAddress", nil)
   oReq:AddVar("Filename"    , cFileName)
   oReq:AddPayload("FileData", cFileName, PL_DATA_FILE )

   // add "request" structure to SOAP envelope
   oSoap:AddVar("request", oReq)

   oResult := oSoap:Execute(cURL, "AddFile", cSOAPAction)

   if oResult == NIL
      MsgBox("An error occurred trying to send request to SOAP server!" + chr(10) +;
             "Error code: " + NTrim(oSoap:ErrorCode) + chr(10) + oSoap:ErrorMessage)
      Return oResult
   endif

   ?
   xFileData := oResult:GetVar({"AddFileResult","FileData"})

   if ValType(xFileData) == "C"
      // we have the entire file data contents in memory
      ? "Saving returned file data to: ~" + cFileName
      xbSaveToFile(xFileData, "~" + cFileName, .F.)
      ?
      wait
   elseif ValType(xFileData) == "O" .and. xFileData:isDerivedFrom("xbStream")
      // the returned file data is saved in a temp file on disk
      ? "Moving returned file data to: ~" + cFileName
      FErase("~" + cFileName)
      FRename(xFileData:Name, "~" + cFileName)
      ?
      wait
   endif
   oResult:Payload:Erase() // delete temp files from disk

   set cursor on
   cls

   Return oResult

//-----------------------------------------------------------------------------
STATIC PROCEDURE DisplayStatus( nStatus, x )
   if valtype(x) == "O" .and. x:isDerivedFrom("xbSocket")
      ? SOCK_ST_text(nStatus), "SendCount=", x:SendCount, "RecvCount=", x:RecvCount
   else
      ? SOCK_ST_text(nStatus), x
   endif
   Return

//-----------------------------------------------------------------------------
STATIC FUNCTION SOCK_ST_text( nCode )
   Local i
   Static aCodes := {{SOCK_ST_CONNECT   , "SOCK_ST_CONNECT   "},;
                     {SOCK_ST_SSLCONNECT, "SOCK_ST_SSLCONNECT"},;
                     {SOCK_ST_ACCEPT    , "SOCK_ST_ACCEPT    "},;
                     {SOCK_ST_CLOSE     , "SOCK_ST_CLOSE     "},;
                     {SOCK_ST_RECV      , "SOCK_ST_RECV      "},;
                     {SOCK_ST_SEND      , "SOCK_ST_SEND      "}}

   if (i := AScan(aCodes, nCode)) > 0
      Return aCodes[i,2]
   endif

   Return ""


