///////////////////////////////////////////////////////////////////////////////
//
//  TIMECLOCK.PRG
//
//  Sample TimeClock web application demonstrates the following:
//    - use of session management without client-side cookies
//    - how to handle web subdirectories
//    - how to obfuscate function parameters from users
//    - use of frames to create simple menu
//    - generic function to generate DBF browse screens
//    - how to post data from a browser
//
//  Copyright (c) 2005-2009, Xb2.NET inc.
//  All rights reserved.
//
//  Author:   Boris Borzic
//  Email:    support@xb2.net
//  Web:      http://www.xb2.net
//
///////////////////////////////////////////////////////////////////////////////

* Pozn. alie funkcie pre pravu RAUKN s v TimeClock_Raukn.PRG

#include "xb2net.ch"
#include "common.ch"
#pragma  library ("xb2net.lib")

#xtranslate NTrim(<n>)        =>  LTrim(Str(<n>))

//#xtranslate Debug([<p,...>])  =>  QOut(<p>)
#xtranslate Debug([<p,...>])  =>

// security levels:
#define SEC_NONE          0
#define SEC_EMPLOYEE      1
#define SEC_SUPERVISOR    2
#define SEC_ADMIN         3


#xtranslate GETVAR <(name)> AS <var>     => Local <var> := oClient:GetVar(<(name)>)
#xtranslate GETCARGO <(name)> AS <var>   => Local <var> := oClient:GetCargo(<(name)>)
#xtranslate GETCOOKIE <(name)> AS <var>  => Local <var> := oClient:GetCookie(<(name)>)

#command WEBPROCEDURE <name>([<param,...>])         ;
         [VAR <(fname)> AS <fvar>]                  ;
         [COOKIE <(cname)> AS <cvar>]               ;
         [CARGO <(sname)> AS <svar>]                ;
            => PROCEDURE WEB_<name>(<param>)        ;
               ; Local oClient := ThreadObject()    ;
               [; GETVAR <(fname)> AS <fvar>]       ;
               [; GETCOOKIE <(cname)> AS <cvar>]    ;
               [; GETCARGO <(sname)> AS <svar>]

#command WEBPROCEDURE <name>([<param,...>])         ;
         PATH <path>                                ;
         [VAR <(fname)> AS <fvar>]                  ;
         [COOKIE <(cname)> AS <cvar>]               ;
         [CARGO <(sname)> AS <svar>]                ;
            => PROCEDURE WEB_<path>_<name>(<param>) ;
               ; Local oClient := ThreadObject()    ;
               [; GETVAR <(fname)> AS <fvar>]       ;
               [; GETCOOKIE <(cname)> AS <cvar>]    ;
               [; GETCARGO <(sname)> AS <svar>]

#command CHECKLOGIN ;
         => if ! isLoggedIn() ;
         ;     return ;
         ;  endif

#command CHECKLOGIN SECURITY [<nSecLevel>] ;
         => if ! isLoggedIn() .or. ! HaveSecurityPrivilege(<nSecLevel>) ;
         ;     return ;
         ;  endif

/*************************************************************
 *
 * // using the above pre-processor commands, this statement:
 *
 * WEBPROC MyFunc( cP1, nP2, lP3 ) PATH TEST ;
 *     VAR    V1 AS cVar1 ;
 *     VAR    V2 AS cVar2 ;
 *     COOKIE C1 AS cCookie1 ;
 *     CARGO  S1 AS cCargo1 ;
 *
 *     // your code goes here...
 *
 * RETURN
 *
 *
 * // will be converted into the following Xbase++ code:
 * // note that a local var "oClient" is automatically created
 *
 * PROCEDURE WEB_TEST_MyFunc( cP1, nP2, lP3 )
 *    Local oClient  := ThreadObject()
 *    Local cVar1    := oClient:GetVar("V1")
 *    Local cVar2    := oClient:GetVar("V2")
 *    Local cCookie1 := oClient:GetCookie("C1")
 *    Local cCargo1  := oClient:GetCargo("S1")
 *
 *    // your code goes here...
 *
 * RETURN
 *
 *************************************************************/

//-----------------------------------------------------------------------------
// This function is the entry point into the TimeClock app
//
WEBPROC Login() PATH TC
   GETVAR UID AS cUID
   GETVAR PWD AS cPWD
   Local lUpdate  := .t.
   Local cMessage := ""
   Local oServer  := oClient:Parent  // this is the xbHTTPServer object
   Local nID

   oClient:CloseSession()

   debug( "TC_Login 1: SID", oClient:GetSessionHandle(), "# Sess", xbSession():ActiveSessions(), "CARGO", oClient:GetCargo())

   if Empty(cUID)
      // no user ID specified - redisplay login screen
      oClient:SendFile( oServer:RootDir + "\TC\Login.htm" )
      Return
   endif

   OpenDBF( "EMPLOYEE", "EMP", "EMPLOYEE")

   nID := Val(cUID)

   if !(DbSeek(nID, .f.) .and. Upper(Trim(emp->password)) == Upper(Trim(cPWD)))
      // invalid userid/password - go back to login screen
      dbCLoseAll()
      oClient:SendFile( oServer:RootDir + "\TC\Login.htm" )
      Return
   endif

   // we have a valid userid/password

   // make sure any previous session is properly closed
   oClient:CloseSession()

   // the GetCargo/SetCargo methods will automatically open a new session to keep track of this guy

   if oClient:GetCargo("UID") != NIL
      if oClient:GetCargo("UID") == nID
         // same guy, don't update login count in password table
         lUpdate := .f.
      else
         // this client was previously logged in under a different userid
         // close the old session
         oClient:CloseSession()
      endif
   endif

   oClient:SetCargo("UID"       , nID)
   oClient:SetCargo("NAME"      , AllTrim(emp->MName + " " + Trim(emp->FName) + " " + emp->LName))
   oClient:SetCargo("SECURITY"  , emp->security)  // save the security level
   oClient:SetCargo("EXCEPTIONS", emp->(GetExceptions()))

   if emp->DATE_IN == date()
      oClient:SetCargo("CLOCKIN" , emp->time_in)
   endif

   if emp->DATE_OUT == date() .and. emp->time_out >= emp->time_in
      oClient:SetCargo("CLOCKOUT", emp->time_out)
   endif

   // update the login count if user was not already logged in
   if lUpdate .and. dbRLock()
      emp->nLogins   := emp->nLogins + 1
      emp->LogInDate := date()
      emp->LogInTime := time()
   endif

   dbCLoseAll()

   debug( "TC_Login 2: SID", oClient:GetSessionHandle(), "# Sess", xbSession():ActiveSessions(), "CARGO", oClient:GetCargo())

   // attach the response string to the HTTP response object
   oClient:HTTPResponse:Content := DrawOuterFrame( oClient:GetSessionHandle() )

   Return

//-----------------------------------------------------------------------------
// Close session and exit TimeClock app. Client browser is taken back to Login screen.
//
WEBPROC Logout() PATH TC
   GETVAR UID AS nID

   oClient:CloseSession()

   OpenDBF( "EMPLOYEE", "EMP", "EMPLOYEE")

   if DbSeek(nID) .and. dbRLock()
      emp->LogOutDate := date()
      emp->LogOutTime := time()
   endif

   dbCLoseAll()

*  FlashMessage( "You are now logged out...", "/", 2)
   FlashMessage( "Odhlsili ste sa ...", "/", 2)

   debug( "TC_Logout: SID", oClient:GetSessionHandle(), "# Sess", xbSession():ActiveSessions(), "CARGO", oClient:GetCargo())

   Return

//-----------------------------------------------------------------------------
// After a user successfully logs in, this function will draw the outer frameset
// for the TimeClock app. This frameset contains a small 'header' frame along the
// top for common hyperlink shortcuts and a 'main' frame where the TimeClock app
// will be drawn.
//
// Although we can use client-side "cookies" to perform session management automatically.
// This will fail when a client has cookie support turned off. To make it work for all
// cases we need to pass the unique session handle within each URL hyperlink pointing
// back to the server.
//

* FUNKCIA PRE VYTVORENIE ZKLADNHO FRAME
*****************************************
STATIC FUNCTION DrawOuterFrame( cSessionHandle )
   Static cTemplate

*qqq 8.11.2018 tu nastavujem frameset na 16, lebo maxrows je 15, potom treba upravi

   * asi ak ete nie je vytvoren template	
   if cTemplate == NIL

      TEXT INTO cTemplate TRIMMED
      <!doctype html public "-//w3c//dtd html 4.0 frameset//en">
      <html>

      <head>
        <title>TimeClock</title>
        <meta name=viewport content="width=device-width, initial-scale=1">
        <link rel="stylesheet" media="screen" type="text/css" href="/style.css">
        <style type="text/css">
        body {margin:0;}

        #red {position:absolute;
              top:0; left:0;
              width:100%;
              background-color:#f30;}
        </style>
        </head>

      <frameset rows="16,*" frameborder=no border=0>
        <frame name=header src=Header?<%sid%> marginwidth=0 marginheight=0 frameborder=no scrolling=no noresize>
        <frame name=main   src=TimeClock?<%sid%> marginwidth=0 marginheight=0 frameborder=no scrolling=auto>
        </frameset>

      <noframes>
        <body>
        <div id="red"></div>
        <center>
        <br><br><br>
        This page uses frames.<br>
        Your browser either does not support frames or you have turned frame support off.<p>
        <b><a href="TimeClock?" target="_top">click here to continue without frames</a></b>
        </center>
        </body>
        </noframes>

      </html>
      ENDTEXT
   endif

   Return StrTran(cTemplate, "<%sid%>", "_SID=" + cSessionHandle)

//-----------------------------------------------------------------------------
WEBPROC Header() PATH TC
   GETCARGO SECURITY AS nSecurity
   Static cTemplate
   Local cSID := oClient:GetSessionHandle()
   Local cLinks

   CHECKLOGIN

   if Empty(nSecurity)
      oClient:SendFile( oClient:Parent:RootDir + "\TC\Login.htm" )
      Return
   endif

   debug( "TC_Header: SID", oClient:GetSessionHandle(), "# Sess", xbSession():ActiveSessions(), "CARGO", oClient:GetCargo())

   if cTemplate == NIL
      TEXT INTO cTemplate TRIMMED
      <html>
      <head>
       <title>TimeClock</title>
       <link rel="stylesheet" media="screen" type="text/css" href="/style.css">
       <style type="text/css">

       body {margin:0; background-color: yellow;}

       a.cold {text-decoration:none; color: navy;}
       a:visited {text-decoration:none; color: navy;}
       a.cold:hover {text-decoration:underline; color:red;}

       table {cell-padding:0;margin-top:0;padding:0;}
       td		{background-color: yellow; vertical-align:center;}
       td.link {padding-left:8; font-weight:bold; text-decoration:none; font-family:verdana,arial;}
       td.name {text-align:right; padding-left:5; padding-top:4; font-weight:bold; text-decoration:none; color:black; font-family:verdana,arial;}

       </style>
       </head>
      <body>

      <table width=99%>
        <tr>
         <td>
            <table width=380px>
              <tr>
              <%links%>
              </tr>
            </table>
         </td>
         <td class=name>State&nbsp;Building&nbsp;Company</td>
        </tr>
      </table>

      </body>
      </html>
      ENDTEXT
   endif

   cLinks := HeaderLink( "CLOCK&nbsp;IN/OUT", "TimeClock", cSID )

   if nSecurity >= SEC_ADMIN
      cLinks += HeaderLink( "REPORTS", "Reports", cSID )
   endif

   cLinks += HeaderLink( "HELP"  , "UnderConstruction('HELP')", cSID )
   cLinks += HeaderLink( "LOGOUT", "Logout", cSID )

   oClient:HTTPResponse:Content := StrTran(cTemplate, "<%links%>", cLinks, 1, 1)

   Return

//-----------------------------------------------------------------------------
STATIC FUNCTION HeaderLink( cDisplay, cFunction, cSID )
   Return '<td class=link><img src=/4dot.gif> <a href=' + cFunction +;
          '?_SID=' + cSID + ' class="cold" title="' + cDisplay +;
          '" target=main>' + cDisplay + '</a></td>'

//-----------------------------------------------------------------------------
WEBPROC UnderConstruction( cName ) PATH TC
   CHECKLOGIN
   DEFAULT cName TO "This"
   FlashError( cName + " function is not implemented yet...", "TimeClock?_SID=" + ThreadObject():GetSessionHandle() )
   Return

//-----------------------------------------------------------------------------
WEBPROC Dir() PATH TC
   Local cTempFile

   CHECKLOGIN SECURITY SEC_ADMIN

   // create a unique temp file name to redirect output of DOS commands to
   cTempFile := "~" + NTrim(oClient:Handle) + ".TMP"

   RunShell( "/C DIR > " + cTempFile,,.f.,.t. )
   // read contents from temp file so we can send it to the client
   oClient:HTTPResponse:ContentType := "text/plain"
   oClient:HTTPResponse:Content     := MemoRead(cTempFile)
   FErase(cTempFile)

   Return

//-----------------------------------------------------------------------------
WEBPROC View( cFile ) PATH TC

   CHECKLOGIN SECURITY SEC_ADMIN

   if Empty(cFile)
      FlashError('No file specified...')
   elseif FExists(cFile)
      oClient:HTTPResponse:ContentType := "text/plain"
      oClient:HTTPResponse:Content     := MemoRead(cFile)
   else
      oClient:NotFound()
   endif

   Return

//-----------------------------------------------------------------------------
WEBPROC ServerStats() PATH TC
   Local oServer := oClient:Parent  // this is the xbHTTPServer object
   Local cHtml, aCargo, i

   CHECKLOGIN SECURITY SEC_ADMIN

   TEXT INTO cHtml TRIMMED
      <html><head>
      <title>Xb2.NET Server Stats</title>
      <link rel="stylesheet" type="text/css" href="/Style.css"></head>
      <body bgcolor=black style=margin:0;>
      <table width=80% align=center>
      <tr><th colspan=2><br>Server statistics:</th></tr>
   ENDTEXT

   cHtml += "<tr><td>Current server date/time</td><td>" + xbGMTDate() + "</td></tr>" +;
            "<tr><td>Server uptime</td><td>" + UpTime(oServer:UpTime()) + "</td></tr>" +;
            "<tr><td>Number of open connections</td><td>" + NTrim(oServer:ActiveConnections()) + "</td></tr>" +;
            "<tr><td>Total connect count</td><td>" + NTrim(oServer:ConnectCount) + "</td></tr>" +;
            "<tr><td>Total number of open sessions</td><td>" + NTrim(xbSession():ActiveSessions()) + "</td></tr>" +;
            "<tr><th colspan=2><br>Your session data:</th></tr>"

   aCargo := oClient:GetCargo()

*qqq ??? neviem nao toto je rozdiel r0 a r1

   for i := 1 to len(aCargo)
      if (i % 2) == 0
         cHtml += "<tr class=r0>"
      else
         cHtml += "<tr class=r1>"
      endif

      cHtml += "<td>" + aCargo[i,VAR_NAME] + "</td><td>" + Var2Char(aCargo[i,VAR_VALUE]) + "</td></tr>"
   next

   cHtml += "</table></body></html>"

   oClient:HTTPResponse:Content := cHtml

   oClient:HTTPResponse:Refresh("15")

   Return

//-----------------------------------------------------------------------------
STATIC FUNCTION UpTime( aTime )
   Local cTime := ""

   if aTime[UPTIME_DAY] != 0; cTime += NTrim(aTime[UPTIME_DAY]) + " d "; endif
   if aTime[UPTIME_HRS] != 0; cTime += NTrim(aTime[UPTIME_HRS]) + " h "; endif
   if aTime[UPTIME_MIN] != 0; cTime += NTrim(aTime[UPTIME_MIN]) + " m "; endif
   if aTime[UPTIME_SEC] != 0; cTime += NTrim(aTime[UPTIME_SEC]) + " s "; endif

   Return cTime

//-----------------------------------------------------------------------------
WEBPROC Reports() PATH TC
   Local cHtml, aParam

   CHECKLOGIN SECURITY SEC_ADMIN

   // This is the array of params that will be passed to the WEB_BrowseTable function.
   // However we do not want users to see the data that we are passing so we will
   // obfuscate it by first converting it to a string using var2char, then Base64
   // encoding it

   aParam := {"employee", ;
              nil,;
              nil,;
              nil,;
              "Employee Listing", ;
              GetFieldList("EMPLOYEE", {"PASSWORD"}, .T.), ;
              nil }

   TEXT INTO cHtml TRIMMED
      <html><head>
      <link rel="stylesheet" type="text/css" href="/Style.css"></head>
      <body bgcolor=black style=margin:0;>
      <table width=90% align=center>
        <tr><th colspan=2>Reports:<br>&nbsp;</th></tr>
        <tr><td colspan=2><br><a href=<%browse%>?<%sid%>>Browse employee table</a></td>
        <tr><td colspan=2><br><a href=ServerStats?<%sid%>>View current server stats</a></td>
        <tr><td colspan=2><br><a href=dir?<%sid%>>Server file listing</a></td>
        <tr><td colspan=2><br><a href="view([<%file%>])?<%sid%>">Error.log</a></td>
        </table></body></html>
   ENDTEXT


   cHtml := StrTran(cHtml, "<%file%>"  , xbEscape(xbSocket():ErrorLogFile))
   cHtml := StrTran(cHtml, "<%sid%>"   , "_SID=" + oClient:GetSessionHandle())
   cHtml := StrTran(cHtml, "<%browse%>", "BrowseTable('" + xbBase64Encode(var2char(aParam)) + "')" )

   oClient:HTTPResponse:Content := cHtml

   Return

//-----------------------------------------------------------------------------
STATIC FUNCTION GetFieldList( cTable, aFieldNames, lExclude )
   Local i, imax, aFields[0]

   use (cTable) shared readonly

   if ! NetErr()
      if lExclude
         imax := fcount()
         aFields := Array(imax)
         for i := 1 to imax
            aFields[i] := i
         next
         imax := len(aFieldNames)
         for i := 1 to imax
            aFieldNames[i] := FieldPos(aFieldNames[i])
         next
         ASort(aFieldNames)
         for i := imax to 1 step -1
            if aFieldNames[i] > 0
               ARemove(aFields, aFieldNames[i])
            endif
         next
      else
         aFields := Array(Len(aFieldNames))
         AEval(aFieldNames, {|cName,i| aFields[i] := FieldPos(cName)})
      endif
   endif

   dbCloseArea()
   Return aFields

//-----------------------------------------------------------------------------
STATIC FUNCTION GetExceptions()

   if field->date_in < date() .and. field->date_out < field->date_in
      Return "<b>* NOTE *</b><p>You forgot to clock out on " + dtoc(field->date_in)
   endif

   if field->date_out < date() .and. field->date_out > field->date_in
      Return "<b>* NOTE *</b><p>You forgot to clock in on " + dtoc(field->date_out)
   endif

   if field->date_out = date() .and. field->date_in < date()
      Return "<b>* NOTE *</b><p>You forgot to clock in today!"
   endif

   Return ""

//-----------------------------------------------------------------------------
WEBPROC TimeClock() PATH TC
   GETCARGO UID    AS nID
   GETVAR   ACTION AS cAction
   Local cTime, cFilter, cDate, aParam

   debug( "TimeClock: SID", oClient:GetSessionHandle(), "# Sess", xbSession():ActiveSessions(), "CARGO", oClient:GetCargo())

   CHECKLOGIN

   Debug("Action:", cAction, oClient:GetCargo())

   DEFAULT cAction TO ""

   cAction := Upper(cAction)

   do case
   case cAction == "CLOCK IN"
      cTime := SaveTimeClock( nID, 1 )
      oClient:SetCargo("CLOCKIN", cTime)

   case cAction == "CLOCK OUT"
      cTime := SaveTimeClock( nID, 2 )
      oClient:SetCargo("CLOCKOUT", cTime)
      // display timeclock page for 5 seconds, then logout user
      oClient:HTTPResponse:Refresh("5; URL=Logout?_SID=" + oClient:GetSessionHandle())

   case cAction == "CANCEL LAST CLOCK IN"
      cTime := SaveTimeClock( nID, 3 )
      oClient:SetCargo("CLOCKIN", "")

   case cAction == "CANCEL LAST CLOCK OUT"
      cTime := SaveTimeClock( nID, 4 )
      oClient:SetCargo("CLOCKOUT", "")

   case cAction == "CHANGE PASSWORD"
      WEB_TC_ChangePasswordForm()
      Return

   case cAction == "VIEW WORK WEEK"
      cDate := dtos(Date() - DoW(Date()) + 1)  // date of start of week
      cFilter := "{||field->ID=" + NTrim(nID) + ".and.(dtos(field->date_in)>='" + cDate + "'.or.dtos(field->date_out)>='" + cDate +"')}"
      //aParam := {"TCLOCK", "ID", Str()"Work Week", nil, &(cFilter) }
      aParam := {"TCLOCK", "ID", Str(nID,10), Str(nID,10), "Work Week for: " + oClient:GetCargo("NAME"), nil, &(cFilter)}
      WEB_TC_BrowseTable( xbBase64Encode(var2char(aParam)) )
      Return

   case cAction == "EXIT"
      WEB_TC_Logout()
      Return

   endcase

   oClient:HTTPResponse:Content := BuildTimeClockPage(;
                                       oClient:GetCargo("NAME"), ;
                                       oClient:GetCargo("CLOCKIN"), ;
                                       oClient:GetCargo("CLOCKOUT"), ;
                                       oClient:GetCargo("EXCEPTIONS"))
   Return

//-----------------------------------------------------------------------------
STATIC FUNCTION SaveTimeClock( nID, nAction )
   Local cTime := Left(Time(),5)

   OpenDBF( "EMPLOYEE", "EMP", "EMPLOYEE")
   OpenDBF( "TCLOCK"  , "TC" , "TCLOCK"  )

   if emp->(DbSeek(nID)) .and. emp->(LockRecord())
      do case
      case nAction == 1                // clock in
         if tc->(DbSeek(str(0,10))) .and. tc->(dbRLock())
            // recycle deleted record
         else
            tc->(dbAppend())
         endif
         tc->ID      := nID
         tc->DATE_IN := emp->DATE_IN := Date()
         tc->TIME_IN := emp->TIME_IN := cTime

      case nAction == 2                // clock out
         if tc->(DbSeek( STR(nID,10) + DTOS(emp->DATE_IN) + emp->TIME_IN ))
            if tc->(LockRecord())
               tc->DATE_OUT := emp->DATE_OUT := Date()
               tc->TIME_OUT := emp->TIME_OUT := cTime
            endif
         else
            tc->(dbAppend())
            tc->ID       := nID
            tc->DATE_IN  := emp->DATE_IN
            tc->TIME_IN  := emp->TIME_IN
            tc->DATE_OUT := emp->DATE_OUT := Date()
            tc->TIME_OUT := emp->TIME_OUT := cTime
         endif

      case nAction == 3                // cancel clock in
         if tc->(DbSeek( STR(nID,10) + DTOS(emp->DATE_IN) + emp->TIME_IN )) .and. tc->(LockRecord())
            tc->ID      := 0
            tc->DATE_IN := tc->DATE_OUT := ctod("")
            tc->TIME_IN := tc->TIME_OUT := ""
         endif
         emp->DATE_IN := ctod("")
         emp->TIME_IN := ""

      case nAction == 4                // cancel clock out
         if tc->(DbSeek( STR(nID,10) + DTOS(emp->DATE_IN) + emp->TIME_IN )) .and. tc->(LockRecord())
            tc->DATE_OUT := ctod("")
            tc->TIME_OUT := ""
         endif
         emp->DATE_OUT := ctod("")
         emp->TIME_OUT := ""

      endcase
   endif

   dbCLoseAll()

   Return cTime


//-----------------------------------------------------------------------------
STATIC FUNCTION BuildTimeClockPage( cName, cTimeIn, cTimeOut, cExceptions )
   Static cTemplate
   Local cHtml

   DEFAULT cTimeIn     TO ""
   DEFAULT cTimeOut    TO ""
   DEFAULT cExceptions TO ""

   if cTemplate == NIL
      TEXT INTO cTemplate TRIMMED
      <html>
      <head>
        <title>TimeClock</title>
        <style>
         td {font:normal 16px verdana; color:white; text-align:left;}
         td.head {font: bold 18px verdana; color:magenta; align:center; }
         td.foot {padding:20; color:yellow; align:center; }
         input {width:220px; font:bold 16px verdana; color:navy; text-align:left;}
         input.disabled {color:gray;}
        </style>
      </head>

      <body bgcolor=black>
        <table width=100% height=100%>
          <tr><td align=center>

          <table width=400 height=380 align=center style='background-color:#036;'>
            <tr><td valign=center>

            <table align=center>
              <tr><td class=head><%name%></td></tr>
              <tr><td><%date%><br><br></td></tr>

               <form name="tc" method="get" action="TimeClock">
                 <input type=hidden name="_SID" value="<%sid%>">
                 <tr><td><input <%disabled%> type=submit name="action" value="Clock In" accesskey="I"></td><td>&nbsp;<%timein%></td></tr>
                 <tr><td><input type=submit name="action" value="Clock Out" accesskey="O"></td><td>&nbsp;<%timeout%></td></tr>
                 <tr><td><input type=submit name="action" value="Cancel Last Clock <%inout%>" accesskey="C"></td></tr>
                 <tr><td><input type=submit name="action" value="Change Password" accesskey="P"></td></tr>
                 <tr><td><input type=submit name="action" value="View Work Week" accesskey="V"></td></tr>
                 <tr><td><input type=submit name="action" value="Exit" accesskey="X"></td></tr>
               </form>

            </table>
            </td></tr>

            <tr><td class=foot><%exceptions%></td></tr>

          </table>
          </td></tr>
        </table>
      </body>
      </html>
      ENDTEXT
   endif

   cHtml := StrTran(cTemplate, "<%sid%>"   , ThreadObject():GetSessionHandle(), 1, 1)
   cHtml := StrTran(cHtml, "<%name%>"      , cName, 1, 1)
   cHtml := StrTran(cHtml, "<%date%>"      , dtoc(date()), 1, 1)
   cHtml := StrTran(cHtml, "<%date%>"      , dtoc(date()), 1, 1)
   cHtml := StrTran(cHtml, "<%timein%>"    , cTimeIn, 1, 1)
   cHtml := StrTran(cHtml, "<%timeout%>"   , cTimeOut, 1, 1)
   cHtml := StrTran(cHtml, "<%exceptions%>", cExceptions, 1, 1)

   if empty(cTimeIn) .or. (!empty(cTimeOut) .and. cTimeOut > cTimeIn)
      cHtml := StrTran(cHtml, "<%disabled%>", "", 1, 1)
      cHtml := StrTran(cHtml, "<%inout%>", "Out", 1, 1)
   else
      cHtml := StrTran(cHtml, "<%disabled%>", "class=disabled disabled", 1, 1)
      cHtml := StrTran(cHtml, "<%inout%>", "In", 1, 1)
   endif

   Return cHtml

//-----------------------------------------------------------------------------
WEBPROC ChangePasswordForm() PATH TC
   Static cTemplate

   if cTemplate == NIL
      TEXT INTO cTemplate TRIMMED
      <html>
      <head>
        <title>ChangePassword</title>
        <style>
         td {font:normal 16px verdana; color:yellow; text-align:center;}
         td.head   {font: bold 18px verdana; color:magenta; text-align:center; align:center; }
         td.prompt {font: bold 12px verdana; color:yellow; text-align:right; }
         td.field  {font: normal 14px verdana; color:white; text-align:left; text-transform:uppercase; padding:0; background-color:#036;}
         td.footer {text-align:right; padding:0; background-color:#036; width:230px;}
        </style>
      </head>

      <body bgcolor=black>
        <table width=100% height=100%>
          <tr><td align=center>

          <table width=400 height=380 align=center style='background-color:#036;'>
            <tr><td valign=center>

            <table align=center>
              <tr><td class=head>Change Password<p><br><br></td></tr>

                <table cellspacing=1 cellpadding=0 align=center>
                  <form name='pwd' method='post' action='ChangePassword'>
                    <input type=hidden name="_SID" value="<%sid%>">
                    <tr><td class=prompt>Old Password:&nbsp;</td><td class=field><input type=password name='OLD'  maxlength=15 size=16></td></tr>
                    <tr><td class=prompt>New Password:&nbsp;</td><td class=field><input type=password name='NEW1' maxlength=15 size=16></td></tr>
                    <tr><td class=prompt>Confirm:&nbsp;</td><td class=field><input type=password name='NEW2' maxlength=15 size=16></td></tr>
                    <tr><td colspan=2><hr></td></tr>
                    <tr><td colspan=2 class=footer><input type=submit value='Save' width=100px style='width:80px; font:bold 12px verdana;'></td></tr>
                  </form>
                </table>
            </table>
            </td></tr>

          </table>
          </td></tr>
        </table>
        <script language='javascript'>document.pwd.OLD.focus();</script>
      </body>
      </html>
      ENDTEXT
   endif

   oClient:HTTPResponse:Content := StrTran(cTemplate, "<%sid%>", oClient:GetSessionHandle(), 1, 1)

   Return

//-----------------------------------------------------------------------------
WEBPROC ChangePassword() PATH TC
   GETCARGO UID  AS nID
   GETVAR   OLD  AS cPWD
   GETVAR   NEW1 AS cNew1
   GETVAR   NEW2 AS cNew2

   CHECKLOGIN

   if ! cNew1 == cNew2
      FlashError( "The new password and confirmation do not match!" )
      Return
   endif

   USE employee ALIAS emp INDEX employee SHARED NEW

   if !(DbSeek(nID, .f.) .and. Upper(Trim(emp->password)) == Upper(Trim(cPWD)))
      FlashError( "Invalid password!" )
   elseif dbRLock()
      emp->password := cNew1
      FlashMessage( "Password changed...", "TimeClock?_SID=" + oClient:GetSessionHandle())
   else
      FlashError( "ERROR: Unable to obtain database record lock.<br>Try again..." )
   endif

   dbCLoseAll()
   Return

* Tu zata browser tabuky, parametre ako nzov tabuky, at dostal z funkcie TimeClock_Raukn
//--------------------------------------------------------------------------------------------
********************************************
WEBPROC BrowseTable( cBase64String ) PATH TC
********************************************
   Local cHtml

*qqq 12.11.2018 tu skam ako zisti UID , d sa, iba pokusne som skal 
*Local ccUID:=oClient:GetCargo("UID")

   CHECKLOGIN		// kontrola i uvate prihlsen, iba tak povol otvorenie 

*qqq 6.11.2018 skam upravi poda seba aby vyfarbovalo riadky a diakritika doplnenie kdovania
* + pridvam aj java script setrow pre vber riadku ale typ radio pre pokus
/*
   TEXT INTO cHtml TRIMMED
      <html><head>
      <link rel="stylesheet" type="text/css" href="/Style.css"></head>
      <body bgcolor=black style=margin:0;>
   ENDTEXT
*/

* TU JE ZAIATOK HTML STRLNY, HLAVIKA, NASTAVENIA ...
*******************************************************
TEXT INTO cHtml TRIMMED
<html>
<head>
  <title>Vber katastrlneho zemia</title>
  <meta charset="Windows-1250">
  <link rel="stylesheet" type="text/css" href="/Style.css">
</head>

<body bgcolor=black style=margin:0;>
  <style>
    table tr:hover td {background:#EDB01C;color:#191970}
  </style>

<!-- skripty pre ovldanie browsera -->

<!-- skript pre vyfarbovanie riadkov ke po nich prechdzam mykou, funguje OK -->
<script type="text/javascript">
function setrow(row) {
	// Clear all rows each time - vymae zakadm vetky predtm oznaen riadky
	for (i=1; i<26; i++) {
		var elem = 'r' + i;
		document.getElementById(elem).style.background = "#fff";
	}
	// set desired row - vyfarb zvolen riadok
	document.getElementById(row).style.background = "#aaa";
	return true;
}
</script>

<!-- skript pre zobrazenie radio buttonov a ich ovldania, tj. zaklikvania, funguje OK -->
<!-- vracia hodnotu bu radioCollection[j].value alebo false  -->
<script type="text/javascript">
    function radioValue(radioCollection) {
       for (var j = radioCollection.length - 1; j >= 0; j--) {
        if (radioCollection[j].checked) {
          return radioCollection[j].value;
        }
      }
      return false;
    }    
</script>

<!--skript pre zistenie hodnoty kliknutho riadku resp. jeho porad.indexu, dvam -1 lebo zapotava aj header-->
<script type="text/javascript">
    function myFunction(x) {
	px=x.rowIndex-1;
	px2=radioValue(document.getElementsByName('rctrl'));
	alert("myFunction Klikol si na riadok : "+
		px+", CISKU="+px2);
	return px;
      }
</script>

<!--skript pre zistenie hodnoty kliknutho riadku resp. jeho porad.indexu, dvam -1 lebo zapotava aj header bez alertu , iba test-->
<script type="text/javascript">
    function myFunction1(x) {
	px=x.rowIndex-1;
	px2="000000";
	cCISKU="123456";
	return cCISKU;
      }
</script>

<!--skript pre vyskanie ak kliknem, napr. pre zistenie hodnoty kliknutho riadku -->
<script type="text/javascript">
    function myFunction2(x) {
	px2:="222222"
	alert("myFunction2 Klikol si na tlatko 2: "+
		", CISKU="+px2);
	return px2;
      }
</script>

<!-- 1.rad horn tlatka samostatn, napr. pre vyuitie na funkcie, typ onclick aj onmouseover s aj in typy -->
<p>
    <button onclick="alert(radioValue(document.getElementsByName('radio')))">
      ONCLICK RADIO radiovalue
    </button>
<!-- 
    <button onmouseover="alert(radioValue(document.getElementsByName('radio')))">
      ONMOUSEOVER RADIO radiovalue
    </button>
-->
    <button onclick="alert(radioValue(document.getElementsByName('rctrl')))">
      ONCLICK RCTRL radiovalue
    </button>
<!-- 
    <button onmouseover="alert(radioValue(document.getElementsByName('rctrl')))">
      ONMOUSEOVER RCTRL radiovalue
    </button>
-->
</p>

<!--druh rad horn tlatko samostatn , ktorm by som mohol zisti ak vybran k.u.je-->
<p>
    <button onclick="myFunction2(this)">
      ONCLICK myFunction2
    </button>
</p>

<!--toto skam zobrazi hodnoty pol ak kliknem na riadok ale zatial nefunguje-->
<script type="text/javascript">
var table=document.getElementById('my-table-id');
                for(var k = 1; k < table.rows.length; k++)
                {
                    table.rows[k].onclick = function()
                    {
                         //rIndex = this.rowIndex;
                         document.getElementById("okres").value = this.cells[0].innerHTML;
                         document.getElementById("cisku").value = this.cells[1].innerHTML;
                         document.getElementById("vymera").value = this.cells[2].innerHTML;
                    };
                }
</script>

<!-- ak toto dm, potom schov radio a d sa dvojklikom vybra pole, ale funguje len ak je to nastaven aj pri zobrazen polka 
 vi kde je 
<style type="text/css">
    input[type="radio"] { opacity: 0; position: absolute; }
    input:checked + .sc-tooltip { background: red; color: white; }
    input:focus + .sc-tooltip { box-shadow: 0 0 3px 1px blue; }
</style>
-->

<!-- toto by malo riei vyznaovanie checkboxu ak kliknem hocikde na riadku, vi odvolva sa na to tr, tj. tr onclick="HandleRadioRow()" -->
<!-- FUNGUJE -->
<script type="text/javascript">
    function handleRadioRow() {
        var t = event.target;
       if (t.tagName.toUpperCase() == "INPUT")
           return;
       while (t.tagName.toUpperCase() != "TR")
           t = t.parentElement;
       var r = t.getElementsByTagName("input")[0];
       r.click();
    }
</script>

<!-- iba test, i sa podar tu nejak uloi hodnotu rctrl, tu sa cez parameter do alertu prenesie hodnota rctrl pri volan tlatka 
ale neviem u i riadok document.forms... nieo sprav, zatia iba na otestovanie e aj tak sa d ale potrebujem alej vedie o s tm -->
<script type="text/javascript">
var rctrl;
	function zistirctrl(pomrctrl) {
	alert(pomrctrl) ;
<!-- toto blokujem, aj tak neviem i je to dobre, poda rady Rudolfa document.forms[0].rctrl.value=pomrctrl; -->
    }
</script>

ENDTEXT
* koniec vygenerovanej strnky, alej u nasleduj daje , ako hlavika pol a polia a alie ovldanie

* pozn, vo funkcii setrow bolo pvodne i<5 ale to iba v prklade boli iba 4 riadky, ja mm nastaven na 25 preto 26
* ale zatia neviem, ako vyui to , e som zaklikol riadok
* tj. neviem ako zisti na ktorom som riadku

/* toto vyfarbuje polka v tabuke ak prechdzam mykou
<style>
table td:hover { background-color: red; }
</style>
*/
/* toto vyfarbuje cel riadok ale na ierno
<style>
table tr:hover td {background:#000}
</style>
*/
/* toto vyfarbuje na oranovo, tj. u celkom OK
<style>
table tr:hover td {background:#EDB01C}
</style>
*/
/* toto je u ok, aj psmo sa zmenilo na biele ak oznen ak dm color 191970 tak tmavomodr
<style>
table tr:hover td {background:#EDB01C;color:#FFF}
</style>
*/

* volanie funkcie pre samotn zobrazenie databzovej tabuky, priom DrawTableGrid vygeneruje HTML kd 
* ale toto spracuje, ale alej u kontrolovan z HTML strnky, teda na Return sa asi nevrti 

   ThreadObject():HTTPResponse:Content := cHtml + DrawTableGrid( cBase64String ) + "</body></html>"

* teda tu u nepjde, lebo kontrolu asi preber web strnk

   Return

//-----------------------------------------------------------------------------
STATIC FUNCTION DrawTableGrid( cBase64String, nMaxRows )

   Local i, cHtml, cPrev, cNext, n
   Local cTable, xTag, xTopScope, xBotScope, cHeading, aFields, bFilter, nFields, aParam
   Local nRows    := 0
   Local oClient  := ThreadObject()
   Local oURL     := xbURI():New()
   Local nDir     := iif(oClient:GetVar("DIR")==NIL, 1, Val(oClient:GetVar("DIR")))
   Local nFromRec := iif(oClient:GetVar("POS")==NIL, 0, Val(oClient:GetVar("POS")))

Local pocetku:=0

*qqq 8.11.2018 iba pokusne preddefinujem zvolen slo k.u., ale toto mus by univerzlne, take peci browser musm funkciu vyrobi
PUBLIC cCISKU:="000000"

* menm aby som lepie videl sprvanie sa browsera
*   DEFAULT nMaxRows TO 25
   DEFAULT nMaxRows TO 15

   // decode the params from the base64 string
   if !Empty( cBase64String )
      aParam := &(xbBase64Decode( cBase64String ))
   endif

   // this would be so much simpler with SQL
   cTable   := aParam[1]	// nzov databzy
   xTag     := aParam[2]	// order
   xTopScope:= aParam[3]	// od zznamu poda aktulneho scope ! tj. orderu
   xBotScope:= aParam[4]	// po zznam  poda aktulneho scope ! tj. orderu
   cHeading := aParam[5]	// nzov tabuky, tj. titulok
   aFields  := aParam[6]	// zoznam pol
   bFilter  := aParam[7]	// code block filtra dajov

   USE (cTable) INDEX (cTable) SHARED NEW
   SET DELETED ON

   if ! Empty(xTag)
      OrdSetFocus(xTag)
   endif

   if xTopScope != NIL
      dbSetScope(SCOPE_TOP, xTopScope)
   endif

   if xBotScope != NIL
      dbSetScope(SCOPE_BOTTOM, xBotScope)
   endif

   if ValType(bFilter) == "B"
      dbSetFilter( bFilter )
   endif

   if Empty(aFields)
      nFields := FCount()
      aFields := Array(nFields)
      for i := 1 to nFields
         aFields[i] := i
      next
   else
      nFields := Len(aFields)
   endif

   oURL:Path("BrowseTable('" + cBase64String + "')")
   oURL:SetVar(oClient:Parent:SessionTag, oClient:GetSessionHandle())

*qqq 7.11.2018 a ete tu sksim iba na pokus doplni zobrazenie hodnt vybranch pol tabuky, vi ku skriptu
cHtml1:="OKRES:<input type='text' name='okres' id='okres'>"+;
        "CISKU:<input type='text' name='cisku' id='cisku'>"+;
        "RCTRL:<input type='text' name='rctrl' id='rctrl'><br>"		// toto iba pokusne i zoberie ak potvrdm z formulra

*tu zana zobrazenie tabuky browsera vrtane nastavenia nzvu id , titulku, headers
*qqq 6.11.2018 tu sksim prida id tabuky poda jednho z prkladov ktor vyuva onclick javascript
*  cHtml := "<table width=90% align=center><tr>"

*qqq tu skam prida border, hodnoty 0,1,2,.. s iba varianty ormovania, alej sa d vea rznych parametrov nastavova
*  cHtml := "<table id='my-table-id' width=90% align=center><tr>"
cHtml := cHtml1+"<table border=2 id='my-table-id' width=90% align=center>"+;	// zkladn nastavenia tabuky
	 "<CAPTION>Zoznam katastrlnych zem</CAPTION>"+;			// nzov, titulok nad tabukou
	 "<tr>"									// u zana prv riadok ale ide najprv hlavika

* zobrazenie hlaviky browsera tj. titulok , nFields zvujem + 1 lebo pridvam radio button
* ale nemus tu by takto, lebo mem ma v CAPTION ako je vyie
   if ! Empty(cHeading)
      cHtml += "<th colspan=" + NTrim(nFields+1) + ">" + cHeading + "<br>&nbsp;</th></tr></tr>"
   endif

*qqq 6.11.2018 tu sksim prida stpec pre vber cez RADIO button ete pred pridanm hlaviiek pol databzy
      cHtml += "<th>" + "Vber" + "</th>"

* zobrazenie hlaviky stpcov tabuky
   // draw column headings
   for i := 1 to nFields
      cHtml += "<th>" + FieldName(aFields[i]) + "</th>"
   next

   * koniec riadku hlaviky 
   cHtml += "</tr>"


****
*qqq 12.11.2018 tu sksim prida tento riadok nzov formulra a spsob sprstupnenia dajov formulra cez post, ale to iba test lebo toto je v Login.htm
*    a tu skam prida akciu ak kliknem pri vbere kat.zemia na tlatko Vybra
*    tj. po odoslan,potvrden tlatkom ktor m submit sa vykon funkcia ktor je v action
****
*cHtml += "<form name='Logout_Raukn_ku' method='post' action='Logout'>"	// ete by som mohol da aj action ale neviem ako spravi
*cHtml += "<form name='Logout_Raukn_ku' method='post' action='Logout_Raukn'>"	// ete by som mohol da aj action ale neviem ako spravi
cHtml += "<form name='Logout_Raukn_ku' method='post' action='Logout_Raukn_ku'>"	// ete by som mohol da aj action ale neviem ako spravi
*

* toto je nejak oetrenie na zznamy, tj. i m skoi na zaiatok alebo od uritho riadku 
   if nFromRec == 0
      dbGoTop()
   else
      dbGoTo(nFromRec)
      if nDir < 0
         // skip back nMaxRows
         dbSkip(-nMaxRows)
      endif
      // skip back to see if we're at the top
      // no sense of displaying "Prev Page" link if we're already at the top
      dbSkip(-1)
      if !Bof()
         dbSkip()
         oURL:SetVar("DIR", "-1")
         oURL:SetVar("POS", NTrim(RecNo()))
         cPrev := oURL:AsString()
      endif
   endif

* vpis riadkov tabuky	
   // draw the table contents
   n := 0	// vynulovanie potadla riadkov

*zistenie potu zatia iba pokusne takto cez count
count to pocetku
go top

* cyklus pre vpis vetkch riadkov tabuky ale iba pre nMaxRows
****************************************************************
do while !Eof() .and. ++nRows <= nMaxRows

	* tu uloenie sla k.u. a nzvu , potom toto ulom ku radiobuttonu zatia testujem
	pomcisku:=alltrim(str(CISKU))
	pomnazov:=alltrim(NAZOV)
	criadok:=alltrim(str(n))	// prava pre pouitie ako id pri oznaen radio buttonu

	* tu neviem nao vyhodnocuje modulo aj preo je class r0 alebo r1 rozdielny, neskr zisti
	* ak prny poet riadkov
	if (++n % 2) == 0

		* u tu musm doplni ak chcem nieo ohadom ovldania riadku lebo kon s > a alej ide td
        	*cHtml += "<tr class=r0>"	// neviem o to rob, class r0 r1 asi iba pomenujem 
		* tu sksim doplni ale u tu pre table row onclick pre zistenie i som ten riadok klikol vola myFunction javascript
*qqq 8.11.2018 tu zablokujem aby mi nevyhadzovalo oznamy ke chcem spravi aby reagoval na spodn tlatko
*		cHtml +="<tr class=r0 onclick='myFunction(this)'>"	

*qqq 9.11.2018 skam funkciu handleRadioRow, i ozna radio ak kliknem na riadok  - FUNGUJE !!!
*        	cHtml += "<tr class=r0>"
        	cHtml += "<tr class=r0 onclick='handleRadioRow()' style='cursor': pointer;>"

	* ak neprny poet riadkov
	else

		* u tu musm doplni ak chcem nieo ohadom ovldania riadku lebo kon s > a alej ide td
        	*cHtml += "<tr class=r1>"
		* tu sksim doplni ale u tu pre table row onclick pre zistenie i som ten riadok klikol vola myFunction javascript
		* pridvam funkciu pre onclic myFunction a odovzdvam jej this hodnotu, to je selected riadok
*qqq 8.11.2018 tu zablokujem aby mi nevyhadzovalo oznamy ke chcem spravi aby reagoval na spodn tlatko 
*		cHtml +="<tr class=r1 onclick='myFunction(this)'>"	

*qqq 9.11.2018 skam funkciu handleRadioRow, i ozna radio ak kliknem na riadok  - FUNGUJE !!!
*        	cHtml += "<tr class=r1>"
        	cHtml += "<tr class=r1 onclick='handleRadioRow()' style='cursor': pointer;>"
	endif

	* cyklus pre zobrazenie polok vetkch stpcov, i je slo stpca		
	********************************************************************
	for i := 1 to nFields

		* tu sksim upravi, doplni radio button, preto vyhodnocujem, i je to prv radio stpec alebo dtov stpce
		* pvodn
		* cHtml += "<td>" + Var2Char(FieldGet( aFields[i] )) + "</td>"

		* iba pre prv pole zobraz button radio, a ostatn nie, ale zatia neviem, ako sa d poui vybran hodnota, 	
		*****************************************
		* onclick zabezpe e ke kliknem , tak sa aktivuje
		* pozor, ak vynechm name=rctrl, tak potom neodznauje radioboxy, take mus by
		if i=1

			*qqq 8.11.2018 tu sksim prida value= teraz u po kliknut dole tlatka Vybran kat.zemie zobraz jeho slo a nzov
			* toto funguje dobre na radio button, ale iba ak kliknem na pole kde je radio a len na gulicku radio
			* tu ete s onclick, spsobovalo zobrazenie setrow a zobrazenie alert okienka, ale toto vyradzujem
			* cHtml += "<td><input type='radio' name='rctrl' value= "+pomcisku+"-"+pomnazov+" onclick='return setrow('r1')'></td>"

			* toto reaguje na kliknutie buttonu radion
		 	cHtml += "<td><input type='radio' name='rctrl' value= "+pomcisku+"-"+pomnazov+"></td>"

			* toto som skobne pridal hidden tj. ako al prvok, ktor nevidno, ale mal by reagova na kliknutie len treba
			* nejak upravi funkciu v js, aby nebolo radio ale prvok tabulky typu text, events ?
			*cHtml += "<td><input type='radio' name='rctrl' value= "+pomcisku+"-"+pomnazov+"><input type='hidden' name='rctrl2' value= "+pomcisku+"-"+pomnazov+"></td>"

			* a tu prava doplnen slo riadku a malo by to reagova na kliknutie aj na text pri radio buttone u funguje dobre
		 	*cHtml += "<td><input type='radio' id="+"rbox"+criadok+" name='rctrl' value="+pomcisku+"-"+pomnazov+"><label for="+"rbox"+criadok+">Vber"+criadok+"</label></td>"
			* a ete zrum label resp. mem necha label poda poradovho sla v databze napr. poda recno() !!! teraz dm przdny bodky
			* lebo ak nedm ni tak nereaguje, mus tam by nejak text alebo by som musel da ete hidden
		 	*cHtml += "<td><input type='radio' id="+"rbox"+criadok+" name='rctrl' value="+pomcisku+"-"+pomnazov+"><label for="+"rbox"+criadok+">"+".........."+"</label></td>"
			* plus sksim da hidden 	
		 	*cHtml += "<td><input type='radio' id="+"rbox"+criadok+" name='rctrl' value="+pomcisku+"-"+pomnazov+">"+;
			*	      "<input type='hidden' id="+"rbox"+criadok+" name='rctrl' value= "+pomcisku+"-"+pomnazov+"><label for="+"rbox"+criadok+">"+".........."+"</label></td>"
			* tu v spojen s spanclass ktor je funkcia vyie nezobraz radio ale povol kliknutie na polko, ktor ostane vyfarben a funguje ako button radio ale zase iba s textom ? neviem preo
		 	*cHtml += "<td><input type='radio' id="+"rbox"+criadok+" name='rctrl' value="+pomcisku+"-"+pomnazov+">"+;
			*	      "<input type='hidden' id="+"rbox"+criadok+" name='rctrl' value= "+pomcisku+"-"+pomnazov+"><label for="+"rbox"+criadok+"><span class='sc-tooltip'>Tabbable</span></label></td>"

		endif

		* tu u zobraz iba samotn obsah poa
		**************************************

		* povodn zpis  zobraz obsah poa databzy
		* tu by som mohol sksi da aj value pre jednotliv polia uloi, aby som nemusel uklada, napr. doplni zatia neviem ako
		 cHtml += "<td>" + Var2Char(FieldGet( aFields[i] )) + "</td>"

		* tu iba pokusy, ale nedorieil som aby reagovalo na cel riadok
		* sksim rovnako ako pri radio doplni pre riadok hodnoty pomcisku a pomnazov lebo pri kliknuti na riadok neaktualizuje v okienku alert
		* ale nefunguje to , skam hidden tj. schovva value 
		* cHtml += "<td>"+"<input type='radio' name='rctrl2' value= "+pomcisku+"-"+pomnazov+" onclick='return 'myFunction(this)'>" + Var2Char(FieldGet( aFields[i] )) + "</td>"

		* najprv iba takto
		* cHtml += "<td>" +"<value="+"999"+">"+ Var2Char(FieldGet( aFields[i] )) + "</td>"
		* potom tu u ulom hodnotu poa 2 o je CISKU ale toto je zloit, lebo musm browser urobi univerzlne
		* tu sksim fixne da poradie 2 CISKU
		*  cHtml += "<td>" +"<value="+Var2Char(FieldGet( aFields[2] ))+">"+ Var2Char(FieldGet( aFields[i] )) + "</td>"

      next

      cHtml += "</tr>"	// znaka konca riadku tabuky html
      dbSkip()		// skok na al zznam tabuky v dbf
enddo

* toto neviem presne o rob ale asi kontroluje EOF, a ak vychdza na strnku koniec sboru tak asi inak zobraz
   if !Eof()
      // add page down link only if we're not at EOF
      oURL:SetVar("DIR", "1")
      oURL:SetVar("POS", NTrim(RecNo()))
      cNext := oURL:AsString()
   endif

* toto je pridanie tlatok Prev page, Next page na spodok tabuky
   // add hyperlinks for page up/down

/* sksim tam prida svoje tlatko
   cHtml += "<tr><th colspan=" + NTrim(nFields) +" style='text-align:right;'><br>" +;
            iif(cPrev==Nil, "<font style=color:gray;>Prev Page</font>", "<a href=" + cPrev + ">Prev Page</a>") + ;
            " | " +;
            iif(cNext==Nil, "<font style=color:gray;>Next Page</font>", "<a href=" + cNext + ">Next Page</a>") +;
            "</th></tr>" +;
            "</table>"
*/
* tu iba zadefinujem, pozri ako cPrev,cNext je ako URL vytvoren a pridvam svoje tlatka Vber a Storno
cVyber=""	// pri 1!=1 sa zobraz Vber2, ale toto je iba pokusne
cStorno=""
   cHtml += "<tr><th colspan=" + NTrim(nFields) +" style='text-align:left;'><br>" +;
            iif(1!=1, "<font style=color:gray;>Vber1</font>", "<a href=" + cVyber + ">Vber2</a>") + ;
            " | " +;
            iif(1!=1, "<font style=color:gray;>Storno</font>", "<a href=" + cStorno + ">Storno</a>") +;
            "</th>" +;
	    "<th colspan=" + NTrim(nFields) +" style='text-align:right;'>" +;
            iif(cPrev==Nil, "<font style=color:gray;>Prev Page</font>", "<a href=" + cPrev + ">Prev Page</a>") + ;
            " | " +;
            iif(cNext==Nil, "<font style=color:gray;>Next Page</font>", "<a href=" + cNext + ">Next Page</a>") +;
            "</th></tr>" +;
		"<br><tr><th>" + "Poet k.. : " + alltrim(str(pocetku)) + "<tr></th>" +;
            "</table>" +;
	    "<p>" +;
	    "    <button onclick="+"alert(radioValue(document.getElementsByName('rctrl')))"+">" +;
	    "      Vybran kat.zemie" +;
	    "    </button>" +;
	    "    <button onclick="+"alert(radioValue(document.getElementsByName('rctrl2')))"+">" +;
	    "      Vybran kat.zemie-hidden" +;
	    "    </button>" +;
	    "    <button onclick="+"zistirctrl(radioValue(document.getElementsByName('rctrl')))"+">" +;
	    "      Funkcia Zistirctrl" +;
	    "    </button>" +;
	    "</p>"+;
	    "<tr><td class='footer' colspan=2 ><input type='submit' value='Logout_Raukn_ku'></td></tr>"+;
	    "<script language='javascript'>document.Logout_Raukn_ku.RCTRL.focus()></script>" 

* tu toto nejak nefunguje, neviem prs na to ako zska hodnotu pre alie funkcie
*	    "<script language='javascript'>document.Logout_Raukn_ku.OKRES.focus();></script>" 

	dbCLoseAll()	// zatvorenie databz

*qqq 8.11.2018 tu skam i nebude u vedie hodnotu vybranho poa ale toto zobraz oznam ete pred zobrazenm browsera !!!
* lebo toto rob iba cHtml, preto musm poui in systm, sksim public cCISKU
*Msgbox(cCISKU)

Return cHtml

//----------------------------------------------------------------------------------------------------
STATIC FUNCTION LockRecord()
   Local i := 0

   for i := 1 to 10   // try for a max 5 sec
      if dbRLock()
         return .t.
      endif
      sleep( 50 )
   next

   dbCLoseAll()
   Break( "Unable to obtain record lock!" )  // unable to lock record -- this will cause an error to be returned to the client
   Return .f.

//----------------------------------------------------------------------------------------------------
STATIC FUNCTION OpenDBF( cFile, cAlias, cIndex )
   Local i := 0

   DEFAULT cAlias TO cFile

   for i := 1 to 10   // try for a max 5 sec
      if cIndex == NIL
         use (cFile) alias (cAlias) shared new
      else
         use (cFile) alias (cAlias) index (cIndex) shared new
      endif
      if ! NetErr()
         Return .t.
      endif
      sleep( 50 )
   next

   dbCLoseAll()
   Break( "Unable to open database file " + cFile)       // this will cause an error to be returned to the client
   Return .f.

//-----------------------------------------------------------------------------
STATIC FUNCTION isLoggedIn()
   if Empty(ThreadObject():GetCargo("UID"))
      ThreadObject():CloseSession()
      FlashError("You need to log in first...", "Login.htm")
      Return .F.
   endif
   Return .T.

//-----------------------------------------------------------------------------
STATIC FUNCTION HaveSecurityPrivilege( nLevel )
   DEFAULT nLevel TO SEC_ADMIN
   if ThreadObject():GetCargo("SECURITY") >= nLevel
      Return .T.
   endif
   FlashError("Sorry, you do not have access to this function!", "TimeClock?_SID=" + ThreadObject():GetSessionHandle())
   Return .F.

