///////////////////////////////////////////////////////////////////////////////
//
//  WSCHAT.PRG
//
//  Xb2.NET sample showing how to create a simple WebSocket server
//  The client-side is a web browser running .\www_root\WSChat.htm
//
//  Assuming your Xb2.NET HTTP server is running on the local machine, to
//  connect to this WebSocket endpoint navigate to: http://localhost/wschat.htm
//
//  Copyright (c) 2015, Xb2.NET inc.
//  All rights reserved.
//
//  Author:   Boris Borzic
//  Email:    support@xb2.net
//  Web:      http://www.xb2.net
//
///////////////////////////////////////////////////////////////////////////////

#include "xb2net.ch"
#pragma  library ("xb2net.lib")

//-----------------------------------------------------------------------------
PROCEDURE WS_Chat( oSock )
   Local i, imax, cError, cRecv, cToList, aSendTo, aTo, cSend, cMsg, cMyID, aMyContacts[0]
   Static oClients, cHelp

   SET EXACT ON

   if oClients == NIL
      oClients := SafeArray():new()

      TEXT INTO cHelp TRIMMED
      <p><table class=chatTable>
      <tr><th>Command</th><th>Action</th></tr>
      <tr><td><b>?</b></td><td>Help</td></tr>
      <tr><td><b>Q</b></td><td>Disconnect</td></tr>
      <tr><td><b>.</b></td><td>List all connected users</td></tr>
      <tr><td><b>&gt; <i>msg</i></b></td><td>Broadcast <b><i>msg</i></b> to all users</td></tr>
      <tr><td><b><i>user</i> : <i>msg</i></b></td><td>Send <b><i>msg</i></b> to <b><i>user</i></b></td></tr>
      <tr><td><b><i>usr1,usr2,usr3</i> : <i>msg</i></b></td><td>Send <b><i>msg</i></b> to <b><i>usr1</i></b>, <b><i>usr2</i></b> + <b><i>usr3</i></b></td></tr>
      </table></p>
      ENDTEXT
   endif

   oSock:RecvTimeout(30000)

   cMsg := "Welcome to the Xb2.NET chatroom<br>"
   if oSock:isSecure()
      cMsg += "cipher:" + oSock:GetCurrentCipher(.t.)
   endif
   cMsg += "<hr>Enter a user ID:"

   oSock:Send(cMsg)

   while .t.
      cRecv := oSock:recv()

      if !oSock:Connected
         // endpoint has disconnected
         // oSock:StatusCode will contain reason for closing connection
         exit
      elseif cRecv == NIL .or. oSock:OpCode == NIL
         // Recv() timed out - send ping to see if client is still there
         // and wait up to 3 seconds for client to reply.
         if !oSock:Ping("", 3000, @cError)
            ? cError
            exit
         endif
         loop
      endif

      // received some data from client
      cRecv := alltrim(cRecv)

      do case
      case empty(cRecv)
         // do nothing

      case empty(cMyID)
         imax   := len(cRecv)
         cError := NIL

         for i := 1 to imax
            if !isAlpha(cRecv[i]) .and. !isDigit(cRecv[i])
               cError := "User ID can only have alpha-numeric characters"
               exit
            endif
         next
         cRecv := Lower(cRecv)

         if cError != NIL
            oSock:send(cError)
         elseif oClients:Get(cRecv) != NIL
            oSock:Send("User ID is taken; choose something else.")
         else
            cMyID := cRecv
            oClients:Add({cMyID, oSock})
            oSock:Send("Welcome <b>" + upper(cMyID) + "</b><br>" + cHelp )
         endif

      case cRecv == "?"
         oSock:send(cHelp)

      case cRecv[1] == ">"
         cMsg := "<div class=chatBC><b>broadcast " + cMyID + ":</b> " + SafeHtml(alltrim(substr(cRecv,2))) + "</div>"
         oClients:Eval({|a|a[2]:send(cMsg)})

      case cRecv[1] == "."
         cMsg := ""
         oClients:Eval({|a| cMsg += a[1] + ", "})
         oSock:send("Connected clients: " + substr(cMsg,1,len(cMsg)-2))

      case upper(cRecv) == "Q"
         // user wants to quit; initiate socket shutdown from server-side
         oSock:send("Good bye...")
         oSock:close()
         exit

      case (i := At(":", cRecv)) > 0
         // send a private message to one or more users

         if empty(cMsg := substr(cRecv, i+1))
            loop
         endif

         cMsg  := SafeHtml(cMsg)
         cSend := "<div class=chatPM><b>" + cMyID +":</b> " + cMsg + "</div>"

         // parse the recipient list (separated by space, comma or semicolon )
         cToList := Lower(AllTrim(SubStr(cRecv, 1, i-1)))
         cToList := StrTran(cToList, ";", ",")
         cToList := StrTran(cToList, " ", ",")
         aSendTo := String2Array(cToList, ",")
         cToList := ""

         for i := 1 to len(aSendTo)
            if (aTo := oClients:FindAndEval(aSendTo[i], {|a|a[2]:send(cSend)})) == NIL
               oSock:Send(aSendTo[i] + ": User not found")
            else
               cToList += aTo[1] + ","
               // This list keeps track of who you message so when you disconnect from
               // the server we will send them a message that you are offline.
               if !(aTo[1] $ aMyContacts)
                  AAdd(aMyContacts, aTo[1])
               endif
            endif
         next

         if !empty(cToList)
            // send message to yourself indicating that message has been sent
            cToList[-1] := ":"
            oSock:Send("<div class=chatSent><b>" + cToList +"</b> " + cMsg + "</div>")
         endif

      otherwise
         oSock:Send("Invalid command")

      endcase
   end

   oClients:Remove(cMyID)

   // send a message to all of your contacts that you have disconnected
   for i := 1 to len(aMyContacts)
      cMsg := "<div class=chatDC>[<b>" + cMyID + "</b>] has DISCONNECTED</div>"
      oClients:FindAndEval(aMyContacts[i], {|a|a[2]:send(cMsg)})
   next

   oSock:destroy()
   xbSSLFreeErrorQueue()
   Return

//-----------------------------------------------------------------------------
STATIC FUNCTION SafeHtml(cString)
   Return StrTran(StrTran(cString,"&","&amp;"),"<","&lt;")

//-----------------------------------------------------------------------------
STATIC FUNCTION String2Array( cString, cDelim )
   Local i, cData, aData[0], nLen := Len(cDelim), nPos := 1

   while (i := At(cDelim, cString, nPos)) > 0
      // don't allow duplicates or empty values
      if !empty(cData := SubStr(cString, nPos, i-nPos)) .and. ! cData $ aData
         AAdd( aData, cData )
      endif
      nPos := i+nLen
   end

   cData := SubStr(cString, nPos)
   if !empty(cData) .and. ! cData $ aData
      AAdd( aData, cData )
   endif
   Return aData

//-----------------------------------------------------------------------------
// This class allows thread-safe operations on a collection of data
CLASS SafeArray
EXPORTED:
   VAR aData

   INLINE METHOD Init()
      ::aData := {}
      Return self

   INLINE SYNC METHOD Add(x)
      AAdd(::aData, x)
      Return self

   INLINE SYNC METHOD Remove(x)
      Local i := AScan(::aData, x)
      if i > 0
         ARemove(::aData, i)
      endif
      Return self

   INLINE SYNC METHOD Get(x)
      Local i := AScan(::aData, x)
      if i > 0
         Return ::aData[i]
      endif
      Return NIL

   INLINE SYNC METHOD FindAndEval(x, bBlock)
      Local i := AScan(::aData, x)
      if i > 0
         Eval(bBlock, ::aData[i])
         Return ::aData[i]
      endif
      Return NIL

   INLINE SYNC METHOD Eval( bBlock )
      AEval(::aData, bBlock)
      Return self
ENDCLASS


