/******************************************************************************
 *
 *  Xb2NET.CH
 *
 *  Copyright (c) 2001-2016, Xb2.NET inc.
 *  All rights reserved.
 *
 *  Portions Copyright (c) 1993-1998 Microsoft Corp.
 *  Portions Copyright (c) 1982-1986 Regents of the University of California.
 *  Portions Copyright (c) 1995-1998 Eric Young (eay@cryptsoft.com)
 *  Portions Copyright (c) 1998-2001 The OpenSSL Project.
 *
 *****************************************************************************/

#ifndef __XB2NET_CH
#define __XB2NET_CH

#include "appevent.ch"

/*
 * Constants and structures defined by the internet system,
 * Per RFC 790, September 1981, taken from the BSD file netinet/in.h.
 */

/*
 * Protocols
 */
#define IPPROTO_IP              0               /* Internet protocol */
#define IPPROTO_ICMP            1               /* Internet control message protocol */
#define IPPROTO_IGMP            2               /* Group management protocol */
#define IPPROTO_GGP             3               /* Gateway-gateway protocol (deprecated) */
#define IPPROTO_TCP             6               /* Transmission control protocol */
#define IPPROTO_EGP             8               /* Exterior gateway protocol */
#define IPPROTO_PUP             12              /* PARC universal packet protocol */
#define IPPROTO_UDP             17              /* User datagram protocol */
#define IPPROTO_HMP             20              /* Host monitoring protocol */
#define IPPROTO_IDP             22              /* Xerox NS IDP */
#define IPPROTO_RDP             27              /* "reliable datagram" protocol */
#define IPPROTO_RVD             66              /* MIT remote virtual disk */
#define IPPROTO_ND              77              /* UNOFFICIAL net disk protocol */

#define IPPROTO_RAW             255             /* raw IP packet */
#define IPPROTO_MAX             256


/*
 * Port/socket numbers: network standard functions
 */
#define IPPORT_ECHO             7
#define IPPORT_DISCARD          9
#define IPPORT_SYSTAT           11
#define IPPORT_DAYTIME          13
#define IPPORT_NETSTAT          15
#define IPPORT_FTP              21
#define IPPORT_TELNET           23
#define IPPORT_SMTP             25
#define IPPORT_TIMESERVER       37
#define IPPORT_NAMESERVER       42
#define IPPORT_WHOIS            43
#define IPPORT_MTP              57

/*
 * Port/socket numbers: host specific functions
 */
#define IPPORT_TFTP             69
#define IPPORT_RJE              77
#define IPPORT_FINGER           79
#define IPPORT_TTYLINK          87
#define IPPORT_SUPDUP           95

/*
 * UNIX TCP sockets
 */
#define IPPORT_EXECSERVER       512
#define IPPORT_LOGINSERVER      513
#define IPPORT_CMDSERVER        514
#define IPPORT_EFSSERVER        520

/*
 * UNIX UDP sockets
 */
#define IPPORT_BIFFUDP          512
#define IPPORT_WHOSERVER        513
#define IPPORT_ROUTESERVER      520
                                        /* 520+1 also used */

/*
 * Ports < IPPORT_RESERVED are reserved for
 * privileged processes (e.g. root).
 */
#define IPPORT_RESERVED         1024

/*
 * Link numbers
 */
#define IMPLINK_IP              155
#define IMPLINK_LOWEXPER        156
#define IMPLINK_HIGHEXPER       158

/*
 * Definitions of bits in internet address integers.
 * On subnets, the decomposition of addresses to host and net parts
 * is done according to subnet mask, not the masks here.
 */
#define IN_CLASSA_NET           0xff000000
#define IN_CLASSA_NSHIFT        24
#define IN_CLASSA_HOST          0x00ffffff
#define IN_CLASSA_MAX           128

#define IN_CLASSB_NET           0xffff0000
#define IN_CLASSB_NSHIFT        16
#define IN_CLASSB_HOST          0x0000ffff
#define IN_CLASSB_MAX           65536

#define IN_CLASSC_NET           0xffffff00
#define IN_CLASSC_NSHIFT        8
#define IN_CLASSC_HOST          0x000000ff

#define INADDR_ANY              0x00000000
#define INADDR_LOOPBACK         0x7f000001
#define INADDR_BROADCAST        0xffffffff
#define INADDR_NONE             0xffffffff

#define WSADESCRIPTION_LEN      256
#define WSASYS_STATUS_LEN       128


/*
 * Options for use with [gs]etsockopt at the IP level.
 */
#define IP_OPTIONS          1           /* set/get IP per-packet options    */
#define IP_MULTICAST_IF     2           /* set/get IP multicast interface   */
#define IP_MULTICAST_TTL    3           /* set/get IP multicast timetolive  */
#define IP_MULTICAST_LOOP   4           /* set/get IP multicast loopback    */
#define IP_ADD_MEMBERSHIP   5           /* add  an IP group membership      */
#define IP_DROP_MEMBERSHIP  6           /* drop an IP group membership      */
#define IP_TTL              7           /* set/get IP Time To Live          */
#define IP_TOS              8           /* set/get IP Type Of Service       */
#define IP_DONTFRAGMENT     9           /* set/get IP Don't Fragment flag   */


#define IP_DEFAULT_MULTICAST_TTL   1    /* normally limit m'casts to 1 hop  */
#define IP_DEFAULT_MULTICAST_LOOP  1    /* normally hear sends if a member  */
#define IP_MAX_MEMBERSHIPS         20   /* per socket; must fit in one mbuf */


/*
 * Definitions related to sockets: types, address families, options,
 * taken from the BSD file sys/socket.h.
 */

#define INVALID_SOCKET  -1
#define SOCKET_ERROR    -1

/*
 * Types
 */
#define SOCK_STREAM     1               /* stream socket */
#define SOCK_DGRAM      2               /* datagram socket */
#define SOCK_RAW        3               /* raw-protocol interface */
#define SOCK_RDM        4               /* reliably-delivered message */
#define SOCK_SEQPACKET  5               /* sequenced packet stream */

/*
 * Option flags per-socket.
 */
#define SO_DEBUG        0x0001          /* turn on debugging info recording */
#define SO_ACCEPTCONN   0x0002          /* socket has had listen() */
#define SO_REUSEADDR    0x0004          /* allow local address reuse */
#define SO_KEEPALIVE    0x0008          /* keep connections alive */
#define SO_DONTROUTE    0x0010          /* just use interface addresses */
#define SO_BROADCAST    0x0020          /* permit sending of broadcast msgs */
#define SO_USELOOPBACK  0x0040          /* bypass hardware when possible */
#define SO_LINGER       0x0080          /* linger on close if data present */
#define SO_OOBINLINE    0x0100          /* leave received OOB data in line */

#define SO_DONTLINGER   0xff7f          /* ~SO_LINGER */
#define SO_EXCLUSIVEADDRUSE 0xfffb      /* ~SO_REUSEADDR  disallow local address reuse */

/*
 * Additional options.
 */
#define SO_SNDBUF       0x1001          /* send buffer size */
#define SO_RCVBUF       0x1002          /* receive buffer size */
#define SO_SNDLOWAT     0x1003          /* send low-water mark */
#define SO_RCVLOWAT     0x1004          /* receive low-water mark */
#define SO_SNDTIMEO     0x1005          /* send timeout */
#define SO_RCVTIMEO     0x1006          /* receive timeout */
#define SO_ERROR        0x1007          /* get error status and clear */
#define SO_TYPE         0x1008          /* get socket type */

/*
 * WinSock 2 extension -- new options
 */
#define SO_GROUP_ID       0x2001      /* ID of a socket group */
#define SO_GROUP_PRIORITY 0x2002      /* the relative priority within a group*/
#define SO_MAX_MSG_SIZE   0x2003      /* maximum message size */
#define SO_PROTOCOL_INFOA 0x2004      /* WSAPROTOCOL_INFOA structure */
#define SO_PROTOCOL_INFOW 0x2005      /* WSAPROTOCOL_INFOW structure */
#ifdef UNICODE
#define SO_PROTOCOL_INFO  SO_PROTOCOL_INFOW
#else
#define SO_PROTOCOL_INFO  SO_PROTOCOL_INFOA
#endif /* UNICODE */
#define PVD_CONFIG        0x3001       /* configuration info for service provider */
#define SO_CONDITIONAL_ACCEPT 0x3002   /* enable true conditional accept: */
                                       /*  connection is not ack-ed to the */
                                       /*  other side until conditional */
                                       /*  function returns CF_ACCEPT */
/*
 * Options for connect and disconnect data and options.  Used only by
 * non-TCP/IP transports such as DECNet, OSI TP4, etc.
 */
#define SO_CONNDATA     0x7000
#define SO_CONNOPT      0x7001
#define SO_DISCDATA     0x7002
#define SO_DISCOPT      0x7003
#define SO_CONNDATALEN  0x7004
#define SO_CONNOPTLEN   0x7005
#define SO_DISCDATALEN  0x7006
#define SO_DISCOPTLEN   0x7007

/*
 * Option for opening sockets for synchronous access.
 */
#define SO_OPENTYPE     0x7008

#define SO_SYNCHRONOUS_ALERT    0x10
#define SO_SYNCHRONOUS_NONALERT 0x20

/*
 * Other NT-specific options.
 */
#define SO_MAXDG        0x7009
#define SO_MAXPATHDG    0x700A
#define SO_UPDATE_ACCEPT_CONTEXT 0x700B
#define SO_CONNECT_TIME 0x700C

/*
 * TCP options.
 */
#define TCP_NODELAY     0x0001
#define TCP_BSDURGENT   0x7000
//#define TCP_MAXSEG      0x0002

/*
 * Address families.
 */
#define AF_UNSPEC       0               /* unspecified (NOT RECOMMENDED) */
#define AF_UNIX         1               /* local to host (pipes, portals) */
#define AF_INET         2               /* internetwork: UDP, TCP, etc. */
#define AF_IMPLINK      3               /* arpanet imp addresses */
#define AF_PUP          4               /* pup protocols: e.g. BSP */
#define AF_CHAOS        5               /* mit CHAOS protocols */
#define AF_NS           6               /* XEROX NS protocols */
#define AF_IPX          AF_NS           /* IPX protocols: IPX, SPX, etc. */
#define AF_ISO          7               /* ISO protocols */
#define AF_OSI          AF_ISO          /* OSI is ISO */
#define AF_ECMA         8               /* european computer manufacturers */
#define AF_DATAKIT      9               /* datakit protocols */
#define AF_CCITT        10              /* CCITT protocols, X.25 etc */
#define AF_SNA          11              /* IBM SNA */
#define AF_DECnet       12              /* DECnet */
#define AF_DLI          13              /* Direct data link interface */
#define AF_LAT          14              /* LAT */
#define AF_HYLINK       15              /* NSC Hyperchannel */
#define AF_APPLETALK    16              /* AppleTalk */
#define AF_NETBIOS      17              /* NetBios-style addresses */
#define AF_VOICEVIEW    18              /* VoiceView */
#define AF_FIREFOX      19              /* Protocols from Firefox */
#define AF_UNKNOWN1     20              /* Somebody is using this! */
#define AF_BAN          21              /* Banyan */
#define AF_ATM          22              /* Native ATM Services */
#define AF_INET6        23              /* Internetwork Version 6 */
#define AF_CLUSTER      24              /* Microsoft Wolfpack */
#define AF_12844        25              /* IEEE 1284.4 WG AF */
#define AF_IRDA         26              /* IrDA */
#define AF_NETDES       28              /* Network Designers OSI & gateway enabled protocols */

#define AF_MAX          29


/*
 * Protocol families, same as address families for now.
 */
#define PF_UNSPEC       AF_UNSPEC
#define PF_UNIX         AF_UNIX
#define PF_INET         AF_INET
#define PF_IMPLINK      AF_IMPLINK
#define PF_PUP          AF_PUP
#define PF_CHAOS        AF_CHAOS
#define PF_NS           AF_NS
#define PF_IPX          AF_IPX
#define PF_ISO          AF_ISO
#define PF_OSI          AF_OSI
#define PF_ECMA         AF_ECMA
#define PF_DATAKIT      AF_DATAKIT
#define PF_CCITT        AF_CCITT
#define PF_SNA          AF_SNA
#define PF_DECnet       AF_DECnet
#define PF_DLI          AF_DLI
#define PF_LAT          AF_LAT
#define PF_HYLINK       AF_HYLINK
#define PF_APPLETALK    AF_APPLETALK
#define PF_VOICEVIEW    AF_VOICEVIEW
#define PF_FIREFOX      AF_FIREFOX
#define PF_UNKNOWN1     AF_UNKNOWN1
#define PF_BAN          AF_BAN
#define PF_ATM          AF_ATM
#define PF_INET6        AF_INET6

#define PF_MAX          AF_MAX


/*
 * Level number for (get/set)sockopt() to apply to socket itself.
 */
#define SOL_SOCKET      0xffff          /* options for socket level */

/*
 * Maximum queue length specifiable by listen.
 */
#define SOMAXCONN       5

/*
 * Send/Recv flags
 */
#define MSG_NORMAL      0x0             /* send/recv normal messages (default) */
#define MSG_OOB         0x1             /* process out-of-band data */
#define MSG_PEEK        0x2             /* peek at incoming message */
#define MSG_DONTROUTE   0x4             /* send without using routing tables */
#define MSG_MAXIOVLEN   16
#define MSG_PARTIAL     0x8000          /* partial send or recv for message xport */
#define MSG_INTERRUPT   0x10            /* send/recv in the interrupt context */

/*
 * Define constant based on rfc883, used by gethostbyxxxx() calls.
 */
#define MAXGETHOSTSTRUCT        1024

/*
 * Define flags to be used with the WSAAsyncSelect() call.
 */
#define FD_READ         0x01
#define FD_WRITE        0x02
#define FD_OOB          0x04
#define FD_ACCEPT       0x08
#define FD_CONNECT      0x10
#define FD_CLOSE        0x20



#ifndef WSABASEERR
/*
 * All Windows Sockets error constants are biased by WSABASEERR from
 * the "normal"
 */
#define WSABASEERR              10000
/*
 * Windows Sockets definitions of regular Microsoft C error constants
 */
#define WSAEINTR                (WSABASEERR+4)                       // Interrupted function call.
#define WSAEBADF                (WSABASEERR+9)
#define WSAEACCES               (WSABASEERR+13)                      // Permission denied.
#define WSAEFAULT               (WSABASEERR+14)                      // Bad address.
#define WSAEINVAL               (WSABASEERR+22)                      // Invalid argument.
#define WSAEMFILE               (WSABASEERR+24)                      // Too many open files.

/*
 * Windows Sockets definitions of regular Berkeley error constants
 */
#define WSAEWOULDBLOCK          (WSABASEERR+35)                      // Resource temporarily unavailable.
#define WSAEINPROGRESS          (WSABASEERR+36)                      // Operation now in progress.
#define WSAEALREADY             (WSABASEERR+37)                      // Operation already in progress.
#define WSAENOTSOCK             (WSABASEERR+38)                      // Socket operation on non-socket.
#define WSAEDESTADDRREQ         (WSABASEERR+39)                      // Destination address required.
#define WSAEMSGSIZE             (WSABASEERR+40)                      // Message too long.
#define WSAEPROTOTYPE           (WSABASEERR+41)                      // Protocol wrong type for socket.
#define WSAENOPROTOOPT          (WSABASEERR+42)                      // Bad protocol option.
#define WSAEPROTONOSUPPORT      (WSABASEERR+43)                      // Protocol not supported.
#define WSAESOCKTNOSUPPORT      (WSABASEERR+44)                      // Socket type not supported.
#define WSAEOPNOTSUPP           (WSABASEERR+45)                      // Operation not supported.
#define WSAEPFNOSUPPORT         (WSABASEERR+46)                      // Protocol family not supported.
#define WSAEAFNOSUPPORT         (WSABASEERR+47)                      // Address family not supported by protocol family.
#define WSAEADDRINUSE           (WSABASEERR+48)                      // Address already in use.
#define WSAEADDRNOTAVAIL        (WSABASEERR+49)                      // Cannot assign requested address.
#define WSAENETDOWN             (WSABASEERR+50)                      // Network is down.
#define WSAENETUNREACH          (WSABASEERR+51)                      // Network is unreachable.
#define WSAENETRESET            (WSABASEERR+52)                      // Network dropped connection on reset.
#define WSAECONNABORTED         (WSABASEERR+53)                      // Software caused connection abort.
#define WSAECONNRESET           (WSABASEERR+54)                      // Connection reset by peer.
#define WSAENOBUFS              (WSABASEERR+55)                      // No buffer space available.
#define WSAEISCONN              (WSABASEERR+56)                      // Socket is already connected.
#define WSAENOTCONN             (WSABASEERR+57)                      // Socket is not connected.
#define WSAESHUTDOWN            (WSABASEERR+58)                      // Cannot send after socket shutdown.
#define WSAETOOMANYREFS         (WSABASEERR+59)
#define WSAETIMEDOUT            (WSABASEERR+60)                      // Connection timed out.
#define WSAECONNREFUSED         (WSABASEERR+61)                      // Connection refused.
#define WSAELOOP                (WSABASEERR+62)
#define WSAENAMETOOLONG         (WSABASEERR+63)
#define WSAEHOSTDOWN            (WSABASEERR+64)                      // Host is down.
#define WSAEHOSTUNREACH         (WSABASEERR+65)                      // No route to host.
#define WSAENOTEMPTY            (WSABASEERR+66)
#define WSAEPROCLIM             (WSABASEERR+67)                      // Too many processes.
#define WSAEUSERS               (WSABASEERR+68)
#define WSAEDQUOT               (WSABASEERR+69)
#define WSAESTALE               (WSABASEERR+70)
#define WSAEREMOTE              (WSABASEERR+71)


/*
 * Extended Windows Sockets error constant definitions
 */
#define WSASYSNOTREADY          (WSABASEERR+91)                      // Network subsystem is unavailable.
#define WSAVERNOTSUPPORTED      (WSABASEERR+92)                      // WINSOCK.DLL version out of range.
#define WSANOTINITIALISED       (WSABASEERR+93)                      // Successful WSAStartup not yet performed.
#define WSAEDISCON              (WSABASEERR+101)                     // Graceful shutdown in progress.
#define WSAENOMORE              (WSABASEERR+102)
#define WSAECANCELLED           (WSABASEERR+103)
#define WSAEINVALIDPROCTABLE    (WSABASEERR+104)
#define WSAEINVALIDPROVIDER     (WSABASEERR+105)
#define WSAEPROVIDERFAILEDINIT  (WSABASEERR+106)
#define WSASYSCALLFAILURE       (WSABASEERR+107)
#define WSASERVICE_NOT_FOUND    (WSABASEERR+108)
#define WSATYPE_NOT_FOUND       (WSABASEERR+109)                     // Class type not found.
#define WSA_E_NO_MORE           (WSABASEERR+110)
#define WSA_E_CANCELLED         (WSABASEERR+111)
#define WSAEREFUSED             (WSABASEERR+112)

/*
 * Error return codes from :GetHostByName() and :GetHostByAddr()
 * when using the resolver.
 */


/* Authoritative Answer: Host not found */
#define WSAHOST_NOT_FOUND       (WSABASEERR+1001)                    // Host not found.
#define HOST_NOT_FOUND          WSAHOST_NOT_FOUND

/* Non-Authoritative: Host not found, or SERVERFAIL */
#define WSATRY_AGAIN            (WSABASEERR+1002)                    // Non-authoritative host not found.
#define TRY_AGAIN               WSATRY_AGAIN

/* Non recoverable errors, FORMERR, REFUSED, NOTIMP */
#define WSANO_RECOVERY          (WSABASEERR+1003)                    // This is a non-recoverable error.
#define NO_RECOVERY             WSANO_RECOVERY

/* Valid name, no data record of requested type */
#define WSANO_DATA              (WSABASEERR+1004)                    // Valid name, no data record of requested type.
#define NO_DATA                 WSANO_DATA

/* no address, look for MX record */
#define WSANO_ADDRESS           WSANO_DATA
#define NO_ADDRESS              WSANO_ADDRESS

/*
 * Define QOS related error return codes
 */

#define WSA_QOS_RECEIVERS             (WSABASEERR + 1005)            // at least one Reserve has arrived
#define WSA_QOS_SENDERS               (WSABASEERR + 1006)            // at least one Path has arrived
#define WSA_QOS_NO_SENDERS            (WSABASEERR + 1007)            // there are no senders
#define WSA_QOS_NO_RECEIVERS          (WSABASEERR + 1008)            // there are no receivers
#define WSA_QOS_REQUEST_CONFIRMED     (WSABASEERR + 1009)            // Reserve has been confirmed
#define WSA_QOS_ADMISSION_FAILURE     (WSABASEERR + 1010)            // error due to lack of resources
#define WSA_QOS_POLICY_FAILURE        (WSABASEERR + 1011)            // rejected for administrative reasons - bad credentials
#define WSA_QOS_BAD_STYLE             (WSABASEERR + 1012)            // unknown or conflicting style
#define WSA_QOS_BAD_OBJECT            (WSABASEERR + 1013)            // problem with some part of the filterspec or providerspecific buffer in general
#define WSA_QOS_TRAFFIC_CTRL_ERROR    (WSABASEERR + 1014)            // problem with some part of the flowspec
#define WSA_QOS_GENERIC_ERROR         (WSABASEERR + 1015)            // general error
#define WSA_QOS_ESERVICETYPE          (WSABASEERR + 1016)
#define WSA_QOS_EFLOWSPEC             (WSABASEERR + 1017)
#define WSA_QOS_EPROVSPECBUF          (WSABASEERR + 1018)
#define WSA_QOS_EFILTERSTYLE          (WSABASEERR + 1019)
#define WSA_QOS_EFILTERTYPE           (WSABASEERR + 1020)
#define WSA_QOS_EFILTERCOUNT          (WSABASEERR + 1021)
#define WSA_QOS_EOBJLENGTH            (WSABASEERR + 1022)
#define WSA_QOS_EFLOWCOUNT            (WSABASEERR + 1023)
#define WSA_QOS_EUNKOWNPSOBJ          (WSABASEERR + 1024)
#define WSA_QOS_EPOLICYOBJ            (WSABASEERR + 1025)
#define WSA_QOS_EFLOWDESC             (WSABASEERR + 1026)
#define WSA_QOS_EPSFLOWSPEC           (WSABASEERR + 1027)
#define WSA_QOS_EPSFILTERSPEC         (WSABASEERR + 1028)
#define WSA_QOS_ESDMODEOBJ            (WSABASEERR + 1029)
#define WSA_QOS_ESHAPERATEOBJ         (WSABASEERR + 1030)
#define WSA_QOS_RESERVED_PETYPE       (WSABASEERR + 1031)
#define WSA_SECURE_HOST_NOT_FOUND     (WSABASEERR + 1032)
#define WSA_IPSEC_NAME_POLICY_ERROR   (WSABASEERR + 1033)

#endif /* ifdef WSABASEERR */

/*
 * xb2.net specific socket error codes
 */

#define WSA_ERR_SLOW_RECV             (WSABASEERR + 89001)           // data transmission terminated because data transmission is < :MinRecvRate
#define WSA_ERR_SLOW_SEND             (WSABASEERR + 89002)           // data transmission terminated because data transmission is < :MinSendRate
#define WSA_ERR_QUOTA_EXCEEDED        (WSABASEERR + 89003)           // data transmission quota exceeded
#define WSA_ERR_WS_UPGRADE            (WSABASEERR + 89004)           // WebSocket upgrade request failed


#define FIONREAD                1074030207
#define FIONBIO                 -2147195266
#define FIOASYNC                -2147195267
#define SIOCATMARK              1074033415


/*
 * WinSock 2 extension -- manifest constants for :ShutDown()
 */
#define SD_RECEIVE      0x00    // Disables receives
#define SD_SEND         0x01    // Disables sends
#define SD_BOTH         0x02    // Disables any data transfer


/*
 * Microsoft specific extension to WinSock
 */
#define TF_DISCONNECT         0x01
#define TF_REUSE_SOCKET       0x02
#define TF_WRITE_BEHIND       0x04
#define TF_USE_DEFAULT_WORKER 0x00
#define TF_USE_SYSTEM_THREAD  0x10
#define TF_USE_KERNEL_APC     0x20


/*
 * HOSTENT array - returned by :GetHostByAddr() and :GetHostByName()
 */
#define HOSTENT_SIZE   5
#define HOSTENT_CNAME  1
#define HOSTENT_ALIAS  2
#define HOSTENT_NTYPE  3
#define HOSTENT_NLEN   4
#define HOSTENT_ADDR   5

/*
 * SERVENT array - returned by :GetServByPort() and :GetServByName()
 */
#define SERVENT_SIZE   4
#define SERVENT_CNAME  1
#define SERVENT_ALIAS  2
#define SERVENT_NPORT  3
#define SERVENT_CPROT  4

/*
 * PROTOENT array - returned by :GetProtoByNumber() and :GetProtoByName()
 */
#define PROTOENT_SIZE   3
#define PROTOENT_CNAME  1
#define PROTOENT_ALIAS  2
#define PROTOENT_NUMBER 3


/*
 * UPTIME array - returned by :UpTime()
 */
#define UPTIME_SIZE   4
#define UPTIME_DAY    1
#define UPTIME_HRS    2
#define UPTIME_MIN    3
#define UPTIME_SEC    4


/*
 * :GetVar( cName, nSource )  nSource values:
 */
#define VAR_ANY       0    // default - returns first value found with name matching <cName>
#define VAR_QUERY     1    // from URI query string
#define VAR_CONTENT   2    // from posted content (eg. form or soap request envelope)


/*
 * name/value array pairs
 */
#define VAR_NAME      1
#define VAR_VALUE     2
#define VAR_ATTRIB    3
#define VAR_OPTIONS   4


/*
 * xml node elements
 */
#define XTAG_SIZE           4  // size of xml node array
#define XTAG_NAME           1  // xml tag element name as string
#define XTAG_ATTRIB         2  // element attributes as string
#define XTAG_CONTENT        3  // tag contents; a string (tag value) or array containing subnodes (XTAG_TYPE = XTYPE_NODE)
#define XTAG_TYPE           4  // numeric value indicating type of XTAG_CONTENT


/*
 * XTAG_TYPE values:
 */
#define XTYPE_NODE          0  // a subnode
#define XTYPE_TEXT          1  // a data value - no subnodes
#define XTYPE_INSTRUCTION   2  // <?  **** ?>
#define XTYPE_COMMENT       3  // <!--  ****  -->
#define XTYPE_DOCTYPE       4  // <! **** >
#define XTYPE_HREF          5  // <somenode href="#id1" />
#define XTYPE_XOP           6  // <xop:Include href="cid:xxxxxxx" xmlns:xop="http://www.w3.org/2004/08/xop/include" />
#define XTYPE_CDATA         7  // <![CDATA[ **** ]]>


/*
 * xbPayload:List array elements
 */
#define PL_SIZE             6  // array structure size
#define PL_ID               1  // uuid
#define PL_DATA             2  // the payload data - string, object or file name
#define PL_DATA_T           3  // PL_DATA type
#define PL_TYPE             4  // media-type or absolute-URI of payload
#define PL_TYPE_T           5  // PL_TYPE type
#define PL_OPTIONS          6  // optional information used by parser


/*
 * PL_TYPE_T values:
 */
#define PL_TYPE_UNCHANGED   0  // same as previous chunk
#define PL_TYPE_MIME        1  // PL_TYPE is a media MIME type
#define PL_TYPE_URI         2  // PL_TYPE is an absolute URI
#define PL_TYPE_UNKNOWN     3  // PL_TYPE is not known
#define PL_TYPE_NONE        4  // there is no data
#define PL_TYPE_XBASE      12  // data contains an Xbase++ value of any data type (object, codeblock, array, number, string, date, etc...)

/*
 * PL_DATA types:
 */
#define PL_DATA_NONE        0  // no data or unknown format
#define PL_DATA_STRING      1  // a char string
#define PL_DATA_FILE        2  // a disk file
#define PL_DATA_OBJECT      3  // an object having a :AsString() method and :MimeType property
#define PL_DATA_CB          4  // an Xbase++ codeblock
#define PL_DATA_XBASE       5  // data contains an Xbase++ value of any data type (object, codeblock, array, number, string, date, etc...)


/*
 * xbPayload:Format
 */
#define PAYLOAD_DIME        0  // content-type: application/dime
#define PAYLOAD_MP_MIXED    1  // content-type: multipart/mixed
#define PAYLOAD_MP_FORMDATA 2  // content-type: multipart/form-data
#define PAYLOAD_MP_RELATED  3  // content-type: multipart/related
#define PAYLOAD_XOP         4  // content-type: multipart/related; type="application/xop+xml"; startinfo="text/xml"
#define PAYLOAD_MTOM        5  // content-type: multipart/related; type="application/xop+xml"; startinfo="application/soap+xml"


/*
 * xbXMLDocument:SerializationFormat
 */
#define XML_BINDING_ENCODED   0  // XML type attributes are included in serialized document
#define XML_BINDING_LITERAL   1  // XML type attributes are NOT included in serialized document (data is serialized according to a schema)


/*
 * xbSOAPEnvelope message styles:
 */
#define SOAP_STYLE_RPC        0  // this is the default for Xb2.NET
#define SOAP_STYLE_DOCUMENT   1  // this is the default used by Microsoft .NET development tools.


/*
 * xbSOAPEnvelope serialization formats:
 */
#define SOAP_BINDING_ENCODED  0  // this is the default for Xb2.NET (parameter type information is included with message)
#define SOAP_BINDING_LITERAL  1  // this is the default used by Microsoft .NET development tools (data is serialized according to a schema)


/*
 * xbSOAPEnvelope:Transport types:
 */
#define VIA_SOCKETS           0  // use raw sockets, this is the default for xbSOAPEnvelope:Transport
#define VIA_WININET           1  // use WinInet API
#define VIA_WINHTTP           2  // use WinHttp API


/*
 * xbSOAPEnvelope:SecurityFlags:
 */

#ifndef _WININET_
#define SECURITY_FLAG_IGNORE_REVOCATION         0x00000080
#define SECURITY_FLAG_IGNORE_UNKNOWN_CA         0x00000100
#define SECURITY_FLAG_IGNORE_WRONG_USAGE        0x00000200
#define SECURITY_FLAG_IGNORE_CERT_CN_INVALID    0x00001000 // bad common name in X509 Cert.
#define SECURITY_FLAG_IGNORE_CERT_DATE_INVALID  0x00002000 // expired X509 Cert.
#define SECURITY_FLAG_IGNORE_REDIRECT_TO_HTTPS  0x00004000 // ex: http:// to https://
#define SECURITY_FLAG_IGNORE_REDIRECT_TO_HTTP   0x00008000 // ex: https:// to http://
#endif


/*
 * xbTLog:TimePeriod values:
 */

#define TLOG_UNLIMITED     0  // data is always appended to the same log file
#define TLOG_HOURLY        1  // log files are created hourly
#define TLOG_DAILY         2  // log files are created daily starting with the first entry that occurs after midnight
#define TLOG_WEEKLY        3  // log files are created weekly starting with the first entry that occurs after midnight Saturday
#define TLOG_MONTHLY       4  // log files are created monthly starting with the first entry that occurs after midnight of the last day of the month



/*
 * xbTLog:Fields
 *
 * Compatible with Microsoft IIS W3C Extended Log File fields (W3C Working Draft WD-logfile-960323)
 */

#define EXLOG_DATE             0x000001   // date of activity
#define EXLOG_TIME             0x000002   // time of activity
#define EXLOG_C_IP             0x000004   // IP address of client who accessed the server
#define EXLOG_CS_USERNAME      0x000008   // name of authenticated user who accessed the server
#define EXLOG_S_SITENAME       0x000010   // HTTP server appname
#define EXLOG_S_COMPUTERNAME   0x000020   // name of server on which the log entry was generated
#define EXLOG_S_IP             0x000040   // IP address of server on which the log entry was generated
#define EXLOG_S_PORT           0x000080   // server port number the client connected to
#define EXLOG_CS_METHOD        0x000100   // the action client was trying to perform (eg. GET, POST, etc...)
#define EXLOG_CS_URI_STEM      0x000200   // the resource accessed (eg. index.htm)
#define EXLOG_CS_URI_QUERY     0x000400   // the query, if any, the client was trying to perform
#define EXLOG_SC_STATUS        0x000800   // the response status of the action
#define EXLOG_SC_WIN32_STATUS  0x001000   // the response status of the action as a win32 error code
#define EXLOG_SC_BYTES         0x002000   // number of bytes sent by the server
#define EXLOG_CS_BYTES         0x004000   // number of bytes received by the server
#define EXLOG_TIME_TAKEN       0x008000   // duration of time, in milliseconds, that the action consumed
#define EXLOG_CS_VERSION       0x010000   // the protocol version that the client used (eg. HTTP/1.1)
#define EXLOG_CS_HOST          0x020000   // the HTTP 'host' header field
#define EXLOG_CS_USER_AGENT    0x040000   // the HTTP browser used by the client
#define EXLOG_CS_COOKIE        0x080000   // the content of a HTTP cookie received, if any.
#define EXLOG_CS_REFERRER      0x100000   // the previous HTTP site visited by the user, if it provided a link to the current site.


/*
 * xbSocket:
 */


/* callback appevent code */
#define xbeP_xbSocket                xbeP_User + 13001

/* xbSocket status codes */
#define SOCK_ST_CONNECT              13001
#define SOCK_ST_SSLCONNECT           13002
#define SOCK_ST_ACCEPT               13003
#define SOCK_ST_CLOSE                13004
#define SOCK_ST_RECV                 13005
#define SOCK_ST_SEND                 13006
#define SOCK_ST_SSLACCEPT            13007

/*
 * xbWebSocket:
 */

/* WebSocket opcodes */
#define WS_OP_CONT   0x0  // continuation frame
#define WS_OP_TEXT   0x1  // text frame (UTF-8 encoded)
#define WS_OP_BIN    0x2  // binary frame
#define WS_OP_CLOSE  0x8  // connection close
#define WS_OP_PING   0x9  // ping
#define WS_OP_PONG   0xA  // pong


/* WebSocket status codes for closing a connection */

#define WS_STATUS_1000  1000
   /* 1000 indicates a normal closure, meaning that the purpose for which the connection
      was established has been fulfilled.*/

#define WS_STATUS_1001  1001
  /*  1001 indicates that an endpoint is "going away", such as a server going down
      or a browser having navigated away from a page. */

#define WS_STATUS_1002  1002
   /* 1002 indicates that an endpoint is terminating the connection due to a
      protocol error. */

#define WS_STATUS_1003  1003
   /* 1003 indicates that an endpoint is terminating the connection
      because it has received a type of data it cannot accept (e.g., an
      endpoint that understands only text data MAY send this if it
      receives a binary message). */

#define WS_STATUS_1004  1004
   /*   Reserved. The specific meaning might be defined in the future. */

#define WS_STATUS_1005  1005
   /* 1005 is a reserved value and MUST NOT be set as a status code in a
      Close control frame by an endpoint.  It is designated for use in
      applications expecting a status code to indicate that no status
      code was actually present. */

#define WS_STATUS_1006  1006
   /* 1006 is a reserved value and MUST NOT be set as a status code in a
      Close control frame by an endpoint.  It is designated for use in
      applications expecting a status code to indicate that the
      connection was closed abnormally, e.g., without sending or
      receiving a Close control frame. */

#define WS_STATUS_1007  1007
   /* 1007 indicates that an endpoint is terminating the connection
      because it has received data within a message that was not
      consistent with the type of the message (e.g., non-UTF-8 [RFC3629]
      data within a text message). */

#define WS_STATUS_1008  1008
   /* 1008 indicates that an endpoint is terminating the connection
      because it has received a message that violates its policy.  This
      is a generic status code that can be returned when there is no
      other more suitable status code (e.g., 1003 or 1009) or if there
      is a need to hide specific details about the policy. */

#define WS_STATUS_1009  1009
   /* 1009 indicates that an endpoint is terminating the connection
      because it has received a message that is too big for it to process. */

#define WS_STATUS_1010  1010
   /* 1010 indicates that an endpoint (client) is terminating the
      connection because it has expected the server to negotiate one or
      more extension, but the server didn't return them in the response
      message of the WebSocket handshake.  The list of extensions that
      are needed SHOULD appear in the /reason/ part of the Close frame.
      Note that this status code is not used by the server, because it
      can fail the WebSocket handshake instead. */

#define WS_STATUS_1011  1011
   /* 1011 indicates that a server is terminating the connection because
      it encountered an unexpected condition that prevented it from
      fulfilling the request. */

#define WS_STATUS_1015  1015
   /* 1015 is a reserved value and MUST NOT be set as a status code in a
      Close control frame by an endpoint.  It is designated for use in
      applications expecting a status code to indicate that the
      connection was closed due to a failure to perform a TLS handshake
      (e.g., the server certificate can't be verified). */


/*
 * xbFTPClient stuff:
 */

/* callback appevent code */
#define xbeP_xbFTPClient             xbeP_User + 12001

#xtranslate isFTP_ERR(<n>)           =>  <n> \>= 12001 .and. <n> \<= 12006
#xtranslate isFTP_ST(<n>)            =>  <n> \>= 12050 .and. <n> \<= 12072


/* xbFTPClient error codes */
#define FTP_ERR_CONNECT              12001
#define FTP_ERR_READ                 12002
#define FTP_ERR_SEND                 12003
#define FTP_ERR_SOCKET               12004
#define FTP_ERR_SENDING              12005
#define FTP_ERR_RECEIVING            12006


/* xbFTPClient status codes */
#define FTP_ST_CONNECT               12050
#define FTP_ST_SSLCONNECT            12051
#define FTP_ST_PROTECTION            12052
#define FTP_ST_SENDCOMMAND           12053
#define FTP_ST_CLOSING               12054
#define FTP_ST_LOGIN                 12055
#define FTP_ST_SETCURRENTDIRECTORY   12056
#define FTP_ST_GETCURRENTDIRECTORY   12057
#define FTP_ST_CREATEDIRECTORY       12058
#define FTP_ST_DELETEDIRECTORY       12059
#define FTP_ST_DIRECTORY             12060
#define FTP_ST_GETFILE               12061
#define FTP_ST_PUTFILE               12062
#define FTP_ST_SETTYPE               12063
#define FTP_ST_PORT                  12064
#define FTP_ST_PASV                  12065
#define FTP_ST_DELETEFILE            12066
#define FTP_ST_RENAMEFILE            12067
#define FTP_ST_WAITFORCONNECT        12068
#define FTP_ST_RECEIVING             12069
#define FTP_ST_RECEIVECOMPLETE       12070
#define FTP_ST_SENDING               12071
#define FTP_ST_SENDCOMPLETE          12072


/* ZLIB defines */

#define Z_ERRNO                 -1
#define Z_STREAM_ERROR          -2
#define Z_DATA_ERROR            -3
#define Z_MEM_ERROR             -4
#define Z_BUF_ERROR             -5
#define Z_VERSION_ERROR         -6
#define Z_OK                     0
#define Z_STREAM_END             1
#define Z_NEED_DICT              2

#define Z_DEFAULT_COMPRESSION	  -1  // default compression, equivalent to about 6
#define Z_NO_COMPRESSION	      0  // no compression
#define Z_BEST_SPEED	            1  // best speed, lowest compression
#define Z_BEST_COMPRESSION	      9  // best compression, slowest speed

/*
 * xbCPConvert function
 */

/*  Code Page Default Values */

#define CP_ACP                    0           // default to ANSI code page
#define CP_OEMCP                  1           // default to OEM  code page
#define CP_MACCP                  2           // default to MAC  code page
#define CP_THREAD_ACP             3           // current thread's ANSI code page
#define CP_SYMBOL                 42          // SYMBOL translations
#define CP_UTF7                   65000       // UTF-7 translation
#define CP_UTF8                   65001       // UTF-8 translation

/* MBCS and Unicode Translation Flags */

#define MB_PRECOMPOSED            0x00000001  // use precomposed chars
#define MB_COMPOSITE              0x00000002  // use composite chars
#define MB_USEGLYPHCHARS          0x00000004  // use glyph chars, not ctrl chars
#define MB_ERR_INVALID_CHARS      0x00000008  // error for invalid chars

#define WC_COMPOSITECHECK         0x00000200  // convert composite to precomposed
#define WC_DISCARDNS              0x00000010  // discard non-spacing chars
#define WC_SEPCHARS               0x00000020  // generate separate chars
#define WC_DEFAULTCHAR            0x00000040  // replace w/ default char
#define WC_ERR_INVALID_CHARS      0x00000080  // error for invalid chars
#define WC_NO_BEST_FIT_CHARS      0x00000400  // do not use best fit chars


/*
 * Win32 error codes
 */


#define ERROR_PIPE_LOCAL                 229
#define ERROR_BAD_PIPE                   230
#define ERROR_PIPE_BUSY                  231
#define ERROR_NO_DATA                    232
#define ERROR_PIPE_NOT_CONNECTED         233
#define ERROR_MORE_DATA                  234
#define ERROR_VC_DISCONNECTED            240
#define ERROR_INVALID_EA_NAME            254
#define ERROR_EA_LIST_INCONSISTENT       255
#define WAIT_TIMEOUT                     258
#define ERROR_PIPE_CONNECTED             535
#define ERROR_PIPE_LISTENING             536
#define ERROR_IO_INCOMPLETE              996
#define ERROR_IO_PENDING                 997


/*
 *  Win32 NamedPipe definitions
 */

/* dwOpenMode values for CreateNamedPipe  */
#define PIPE_ACCESS_INBOUND       0x00000001  // The flow of data in the pipe goes from client to server only.
#define PIPE_ACCESS_OUTBOUND      0x00000002  // The flow of data in the pipe goes from server to client only.
#define PIPE_ACCESS_DUPLEX        0x00000003  // Pipe is bidirectional; both server and client processes can read from and write to the pipe.

/* dwPipeMode values for CreateNamedPipe  */
#define PIPE_WAIT                 0x00000000  // Blocking mode is enabled. Read, Write and Connect functions will wait until there is data to read, all data is written, or a client is connected.
#define PIPE_NOWAIT               0x00000001  // Nonblocking mode - supported for compatibility with Microsoft LAN Manager and should not be used to achieve asynchronous I/O with named pipes.
#define PIPE_READMODE_BYTE        0x00000000  // Data is read from the pipe as a stream of bytes. This mode can be used with either PIPE_TYPE_MESSAGE or PIPE_TYPE_BYTE.
#define PIPE_READMODE_MESSAGE     0x00000002  // Data is read from the pipe as a stream of messages. This mode can be only used if PIPE_TYPE_MESSAGE is also specified.
#define PIPE_TYPE_BYTE            0x00000000  // Data is written to the pipe as a stream of bytes. This mode cannot be used with PIPE_READMODE_MESSAGE.
#define PIPE_TYPE_MESSAGE         0x00000004  // Data is written to the pipe as a stream of messages. This mode can be used with either PIPE_READMODE_MESSAGE or PIPE_READMODE_BYTE.

/* CreateNamedPipe nMaxInstances          */
#define PIPE_UNLIMITED_INSTANCES  255

/* NamedPipe End flags for GetNamedPipeInfo */
#define PIPE_CLIENT_END           0x00000000  // The handle refers to the client end of a named pipe instance (this is the default).
#define PIPE_SERVER_END           0x00000001  // The handle refers to the server end of a named pipe instance.

/* WaitNamedPipe : nTimeOut  */
#define NMPWAIT_WAIT_FOREVER      0xffffffff
#define NMPWAIT_USE_DEFAULT_WAIT  0x00000000

#define GENERIC_READ              0x80000000
#define GENERIC_WRITE             0x40000000
#define GENERIC_EXECUTE           0x20000000
#define GENERIC_ALL               0x10000000


/*
 *  Win32 CreateFile
 */

/* dwDesiredAccess */
#define FILE_READ_DATA                       0x0001
#define FILE_WRITE_DATA                      0x0002
#define FILE_CREATE_PIPE_INSTANCE            0x0004
#define FILE_READ_ATTRIBUTES                 0x0080
#define FILE_WRITE_ATTRIBUTES                0x0100
#define FILE_GENERIC_READ                    0x00120089 // (STANDARD_RIGHTS_READ|FILE_READ_DATA|FILE_READ_ATTRIBUTES|FILE_READ_EA|SYNCHRONIZE)
#define FILE_GENERIC_WRITE                   0x00120116 // (STANDARD_RIGHTS_WRITE|FILE_WRITE_DATA|FILE_WRITE_ATTRIBUTES|FILE_WRITE_EA|FILE_APPEND_DATA|SYNCHRONIZE)
#define FILE_GENERIC_EXECUTE                 0x001200A0 // (STANDARD_RIGHTS_EXECUTE|FILE_READ_ATTRIBUTES|FILE_EXECUTE|SYNCHRONIZE)

/* dwShareMode */
#define FILE_SHARE_READ                      0x00000001
#define FILE_SHARE_WRITE                     0x00000002
#define FILE_SHARE_DELETE                    0x00000004

/* dwCreationDisposition */
#define CREATE_NEW                           1
#define CREATE_ALWAYS                        2
#define OPEN_EXISTING                        3
#define OPEN_ALWAYS                          4
#define TRUNCATE_EXISTING                    5

/* dwFlagsAndAttributes */
#define FILE_FLAG_WRITE_THROUGH              0x80000000
#define FILE_FLAG_OVERLAPPED                 0x40000000
#define FILE_FLAG_NO_BUFFERING               0x20000000
#define FILE_FLAG_RANDOM_ACCESS              0x10000000
#define FILE_FLAG_SEQUENTIAL_SCAN            0x08000000
#define FILE_FLAG_DELETE_ON_CLOSE            0x04000000
#define FILE_FLAG_BACKUP_SEMANTICS           0x02000000
#define FILE_FLAG_POSIX_SEMANTICS            0x01000000
#define FILE_FLAG_OPEN_REPARSE_POINT         0x00200000
#define FILE_FLAG_OPEN_NO_RECALL             0x00100000

#define FILE_ATTRIBUTE_READONLY              0x00000001
#define FILE_ATTRIBUTE_HIDDEN                0x00000002
#define FILE_ATTRIBUTE_SYSTEM                0x00000004
#define FILE_ATTRIBUTE_DIRECTORY             0x00000010
#define FILE_ATTRIBUTE_ARCHIVE               0x00000020
#define FILE_ATTRIBUTE_DEVICE                0x00000040
#define FILE_ATTRIBUTE_NORMAL                0x00000080
#define FILE_ATTRIBUTE_TEMPORARY             0x00000100
#define FILE_ATTRIBUTE_SPARSE_FILE           0x00000200
#define FILE_ATTRIBUTE_REPARSE_POINT         0x00000400
#define FILE_ATTRIBUTE_COMPRESSED            0x00000800
#define FILE_ATTRIBUTE_OFFLINE               0x00001000
#define FILE_ATTRIBUTE_NOT_CONTENT_INDEXED   0x00002000
#define FILE_ATTRIBUTE_ENCRYPTED             0x00004000

#define INVALID_HANDLE_VALUE -1
#define FILE_BEGIN            0
#define FILE_CURRENT          1
#define FILE_END              2


/*
 * xbHash() - OpenSSL Message Digest (Hash) Functions:
 * ------------------------+------------+-------------+------------------------
 *                         | Security   | Digest Size | Relative Speed (lower is better)
 *      Algorithm          | Confidence | (bits)      | 64 byte msg   8k msg
 * ------------------------+------------+-------------+-------------+----------
 */
#define MD4           1  //  insecure     128           10            10
#define MD5           2  //  very low     128           10            10
#define RIPEMD160     3  //  high         160           13            29
#define SHA1          4  //  high         160           10            10
#define SHA224        5  //  very high    224           12            27
#define SHA256        6  //  very high    256           12            27
#define SHA384        7  //  very high    384           12            27
#define SHA512        8  //  very high    512           12            27
#define WHIRLPOOL     9  //  very high    512           15            50
#define MDC2         10  //  very high    128           31           340

/*
 * xbSSLContext():init methods
 */

// SSLv2 is no longer supported
//#define SSL_v2              1
//#define SSL_v2_client       2
//#define SSL_v2_server       3

#define DTLS_method           1 // DTLS 1.0 and 1.2
#define DTLS_client_method    2 // DTLS 1.0 and 1.2
#define DTLS_server_method    3 // DTLS 1.0 and 1.2
#define TLS_method            4 // Negotiate highest available SSL/TLS version
#define TLS_client_method     5 // Negotiate highest available SSL/TLS version
#define TLS_server_method     6 // Negotiate highest available SSL/TLS version
#define SSL_v3                7 // SSL v3 should only be used for backward compatibility
#define SSL_v3_client         8 // SSL v3 should only be used for backward compatibility
#define SSL_v3_server         9 // SSL v3 should only be used for backward compatibility
#define TLS_v1               10
#define TLS_v1_client        11
#define TLS_v1_server        12
#define TLSv1_1              13
#define TLSv1_1_client       14
#define TLSv1_1_server       15
#define TLSv1_2              16
#define TLSv1_2_client       17
#define TLSv1_2_server       18
#define DTLSv1               19
#define DTLSv1_client        20
#define DTLSv1_server        21
#define DTLSv1_2             22
#define DTLSv1_2_client      23
#define DTLSv1_2_server      24
#define SSL_METHOD_LEN       24

// for backward compatibility
#define SSL_v23           TLS_method
#define SSL_v23_client    TLS_client_method
#define SSL_v23_server    TLS_server_method


/*
 * xbSSLContext:Version array elements
 */

#define SSLEAY_VERSION		   1
#define SSLEAY_OPTIONS        2  // no longer supported
#define SSLEAY_CFLAGS		   3
#define SSLEAY_BUILT_ON		   4
#define SSLEAY_PLATFORM		   5
#define SSLEAY_DIR		      6
#define SSLEAY_VER_LEN		   6

/*
 * xbSocket:GetCurrentCipher array elements
 */

#define CIPHER_NAME     1   // name of cipher
#define CIPHER_VER      2   // the SSL/TLS protocol version that first defined the cipher
#define CIPHER_BITS     3   // number of secret bits used for cipher
#define CIPHER_ABITS    4   // number of algorithm bits used for cipher

/*
 * xbSSLContext file types
 */

#define X509_FILETYPE_PEM	   1
#define X509_FILETYPE_ASN1	   2
#define X509_FILETYPE_DEFAULT	3
#define SSL_FILETYPE_PEM      X509_FILETYPE_PEM
#define SSL_FILETYPE_ASN1     X509_FILETYPE_ASN1


/* Used in SSL_set_shutdown() / SSL_get_shutdown() */
#define SSL_SENT_SHUTDOWN	         1
#define SSL_RECEIVED_SHUTDOWN	      2


/*
 * OpenSSL: SSL_get_error() result codes for TLS/SSL I/O operations
 */

#define SSL_ERROR_NONE			      0
#define SSL_ERROR_SSL               1
#define SSL_ERROR_WANT_READ		   2
#define SSL_ERROR_WANT_WRITE		   3
#define SSL_ERROR_WANT_X509_LOOKUP	4
#define SSL_ERROR_SYSCALL		      5
#define SSL_ERROR_ZERO_RETURN		   6
#define SSL_ERROR_WANT_CONNECT		7
#define SSL_ERROR_WANT_ACCEPT		   8


/* The following are the possible values for xbSocket:GetSSLState()
 * Used to indicate where we are up to in the SSL connection establishment.
 */

#define SSL_ST_CONNECT           0x1000
#define SSL_ST_ACCEPT            0x2000
#define SSL_ST_MASK              0x0FFF
#define SSL_ST_INIT              0x3000   //(SSL_ST_CONNECT|SSL_ST_ACCEPT)
#define SSL_ST_BEFORE            0x4000
#define SSL_ST_OK                0x03
#define SSL_ST_RENEGOTIATE       0x3004   //(0x04|SSL_ST_INIT)

#define SSL_CB_LOOP              0x01
#define SSL_CB_EXIT              0x02
#define SSL_CB_READ              0x04
#define SSL_CB_WRITE             0x08
#define SSL_CB_ALERT             0x4000 /* used in callback */
#define SSL_CB_READ_ALERT        0x4004 //(SSL_CB_ALERT|SSL_CB_READ)
#define SSL_CB_WRITE_ALERT       0x4008 //(SSL_CB_ALERT|SSL_CB_WRITE)
#define SSL_CB_ACCEPT_LOOP       0x2001 //(SSL_ST_ACCEPT|SSL_CB_LOOP)
#define SSL_CB_ACCEPT_EXIT       0x2002 //(SSL_ST_ACCEPT|SSL_CB_EXIT)
#define SSL_CB_CONNECT_LOOP      0x1001 //(SSL_ST_CONNECT|SSL_CB_LOOP)
#define SSL_CB_CONNECT_EXIT      0x1002 //(SSL_ST_CONNECT|SSL_CB_EXIT)
#define SSL_CB_HANDSHAKE_START   0x10
#define SSL_CB_HANDSHAKE_DONE		0x20

/* The following 2 states are kept in ssl->rstate when reads fail,
 * you should not need these */
#define SSL_ST_READ_HEADER       0xF0
#define SSL_ST_READ_BODY         0xF1
#define SSL_ST_READ_DONE         0xF2

/*
 * SSL_CTX_ctrl / SSL_ctrl
 */
#define SSL_CTRL_NEED_TMP_RSA                   1
#define SSL_CTRL_SET_TMP_RSA                    2
#define SSL_CTRL_SET_TMP_DH                     3
#define SSL_CTRL_SET_TMP_ECDH                   4
#define SSL_CTRL_SET_TMP_RSA_CB                 5
#define SSL_CTRL_SET_TMP_DH_CB                  6
#define SSL_CTRL_SET_TMP_ECDH_CB                7
#define SSL_CTRL_GET_SESSION_REUSED             8
#define SSL_CTRL_GET_CLIENT_CERT_REQUEST        9
#define SSL_CTRL_GET_NUM_RENEGOTIATIONS         10
#define SSL_CTRL_CLEAR_NUM_RENEGOTIATIONS       11
#define SSL_CTRL_GET_TOTAL_RENEGOTIATIONS       12
#define SSL_CTRL_GET_FLAGS                      13
#define SSL_CTRL_EXTRA_CHAIN_CERT               14
#define SSL_CTRL_SET_MSG_CALLBACK               15
#define SSL_CTRL_SET_MSG_CALLBACK_ARG           16
/* only applies to datagram connections */
#define SSL_CTRL_SET_MTU                        17
/* Stats */
#define SSL_CTRL_SESS_NUMBER                    20
#define SSL_CTRL_SESS_CONNECT                   21
#define SSL_CTRL_SESS_CONNECT_GOOD              22
#define SSL_CTRL_SESS_CONNECT_RENEGOTIATE       23
#define SSL_CTRL_SESS_ACCEPT                    24
#define SSL_CTRL_SESS_ACCEPT_GOOD               25
#define SSL_CTRL_SESS_ACCEPT_RENEGOTIATE        26
#define SSL_CTRL_SESS_HIT                       27
#define SSL_CTRL_SESS_CB_HIT                    28
#define SSL_CTRL_SESS_MISSES                    29
#define SSL_CTRL_SESS_TIMEOUTS                  30
#define SSL_CTRL_SESS_CACHE_FULL                31
#define SSL_CTRL_OPTIONS                        32
#define SSL_CTRL_MODE                           33
#define SSL_CTRL_GET_READ_AHEAD                 40
#define SSL_CTRL_SET_READ_AHEAD                 41
#define SSL_CTRL_SET_SESS_CACHE_SIZE            42
#define SSL_CTRL_GET_SESS_CACHE_SIZE            43
#define SSL_CTRL_SET_SESS_CACHE_MODE            44
#define SSL_CTRL_GET_SESS_CACHE_MODE            45
#define SSL_CTRL_GET_MAX_CERT_LIST              50
#define SSL_CTRL_SET_MAX_CERT_LIST              51
#define SSL_CTRL_SET_MAX_SEND_FRAGMENT          52

#define SSL_CTRL_SET_TLSEXT_SERVERNAME_CB       53
#define SSL_CTRL_SET_TLSEXT_SERVERNAME_ARG      54
#define SSL_CTRL_SET_TLSEXT_HOSTNAME            55
#define SSL_CTRL_SET_TLSEXT_DEBUG_CB            56
#define SSL_CTRL_SET_TLSEXT_DEBUG_ARG           57
#define SSL_CTRL_GET_TLSEXT_TICKET_KEYS         58
#define SSL_CTRL_SET_TLSEXT_TICKET_KEYS         59
#define SSL_CTRL_SET_TLSEXT_OPAQUE_PRF_INPUT    60
#define SSL_CTRL_SET_TLSEXT_OPAQUE_PRF_INPUT_CB 61
#define SSL_CTRL_SET_TLSEXT_OPAQUE_PRF_INPUT_CB_ARG 62
#define SSL_CTRL_SET_TLSEXT_STATUS_REQ_CB       63
#define SSL_CTRL_SET_TLSEXT_STATUS_REQ_CB_ARG   64
#define SSL_CTRL_SET_TLSEXT_STATUS_REQ_TYPE     65
#define SSL_CTRL_GET_TLSEXT_STATUS_REQ_EXTS     66
#define SSL_CTRL_SET_TLSEXT_STATUS_REQ_EXTS     67
#define SSL_CTRL_GET_TLSEXT_STATUS_REQ_IDS      68
#define SSL_CTRL_SET_TLSEXT_STATUS_REQ_IDS      69
#define SSL_CTRL_GET_TLSEXT_STATUS_REQ_OCSP_RESP        70
#define SSL_CTRL_SET_TLSEXT_STATUS_REQ_OCSP_RESP        71
#define SSL_CTRL_SET_TLSEXT_TICKET_KEY_CB       72
#define SSL_CTRL_SET_TLS_EXT_SRP_USERNAME_CB    75
#define SSL_CTRL_SET_SRP_VERIFY_PARAM_CB                76
#define SSL_CTRL_SET_SRP_GIVE_CLIENT_PWD_CB             77
#define SSL_CTRL_SET_SRP_ARG            78
#define SSL_CTRL_SET_TLS_EXT_SRP_USERNAME               79
#define SSL_CTRL_SET_TLS_EXT_SRP_STRENGTH               80
#define SSL_CTRL_SET_TLS_EXT_SRP_PASSWORD               81
#define SSL_CTRL_TLS_EXT_SEND_HEARTBEAT                 85
#define SSL_CTRL_GET_TLS_EXT_HEARTBEAT_PENDING          86
#define SSL_CTRL_SET_TLS_EXT_HEARTBEAT_NO_REQUESTS      87
#define DTLS_CTRL_GET_TIMEOUT           73
#define DTLS_CTRL_HANDLE_TIMEOUT        74
#define DTLS_CTRL_LISTEN                        75
#define SSL_CTRL_GET_RI_SUPPORT                 76
#define SSL_CTRL_CLEAR_OPTIONS                  77
#define SSL_CTRL_CLEAR_MODE                     78
#define SSL_CTRL_GET_EXTRA_CHAIN_CERTS          82
#define SSL_CTRL_CLEAR_EXTRA_CHAIN_CERTS        83
#define SSL_CTRL_CHAIN                          88
#define SSL_CTRL_CHAIN_CERT                     89
#define SSL_CTRL_GET_CURVES                     90
#define SSL_CTRL_SET_CURVES                     91
#define SSL_CTRL_SET_CURVES_LIST                92
#define SSL_CTRL_GET_SHARED_CURVE               93
#define SSL_CTRL_SET_ECDH_AUTO                  94
#define SSL_CTRL_SET_SIGALGS                    97
#define SSL_CTRL_SET_SIGALGS_LIST               98
#define SSL_CTRL_CERT_FLAGS                     99
#define SSL_CTRL_CLEAR_CERT_FLAGS               100
#define SSL_CTRL_SET_CLIENT_SIGALGS             101
#define SSL_CTRL_SET_CLIENT_SIGALGS_LIST        102
#define SSL_CTRL_GET_CLIENT_CERT_TYPES          103
#define SSL_CTRL_SET_CLIENT_CERT_TYPES          104
#define SSL_CTRL_BUILD_CERT_CHAIN               105
#define SSL_CTRL_SET_VERIFY_CERT_STORE          106
#define SSL_CTRL_SET_CHAIN_CERT_STORE           107
#define SSL_CTRL_GET_PEER_SIGNATURE_NID         108
#define SSL_CTRL_GET_SERVER_TMP_KEY             109
#define SSL_CTRL_GET_RAW_CIPHERLIST             110
#define SSL_CTRL_GET_EC_POINT_FORMATS           111
#define SSL_CTRL_GET_CHAIN_CERTS                115
#define SSL_CTRL_SELECT_CURRENT_CERT            116
#define SSL_CTRL_SET_CURRENT_CERT               117
#define SSL_CTRL_CHECK_PROTO_VERSION            119
#define DTLS_CTRL_SET_LINK_MTU                  120
#define DTLS_CTRL_GET_LINK_MIN_MTU              121


/*
 * flags for xbSSLContext:SetOptions() / xbSSLContext:GetOptions()
 */

#define SSL_OP_MICROSOFT_SESS_ID_BUG                  0x00000001
#define SSL_OP_NETSCAPE_CHALLENGE_BUG                 0x00000002
/* Allow initial connection to servers that don't support RI */
#define SSL_OP_LEGACY_SERVER_CONNECT                  0x00000004
#define SSL_OP_NETSCAPE_REUSE_CIPHER_CHANGE_BUG       0x00000008
#define SSL_OP_SSLREF2_REUSE_CERT_TYPE_BUG            0x00000010
#define SSL_OP_MICROSOFT_BIG_SSLV3_BUFFER             0x00000020
#define SSL_OP_SAFARI_ECDHE_ECDSA_BUG                 0x00000040
#define SSL_OP_SSLEAY_080_CLIENT_DH_BUG               0x00000080
#define SSL_OP_TLS_D5_BUG                             0x00000100
#define SSL_OP_TLS_BLOCK_PADDING_BUG                  0x00000200

/* Disable SSL 3.0/TLS 1.0 CBC vulnerability workaround that was added
 * in OpenSSL 0.9.6d.  Usually (depending on the application protocol)
 * the workaround is not needed.  Unfortunately some broken SSL/TLS
 * implementations cannot handle it at all, which is why we include
 * it in SSL_OP_ALL. */
#define SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS            0x00000800

/* SSL_OP_ALL: various bug workarounds that should be rather harmless.
 * This used to be 0x000FFFFF before 0.9.7. */
#define SSL_OP_ALL                                    0x80000BFF

/* DTLS options */
#define SSL_OP_NO_QUERY_MTU                           0x00001000
/* Turn on Cookie Exchange (on relevant for servers) */
#define SSL_OP_COOKIE_EXCHANGE                        0x00002000
/* Don't use RFC4507 ticket extension */
#define SSL_OP_NO_TICKET                              0x00004000
/* Use Cisco's "speshul" version of DTLS_BAD_VER (as client)  */
#define SSL_OP_CISCO_ANYCONNECT                       0x00008000

/* As server, disallow session resumption on renegotiation */
#define SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION 0x00010000
/* Don't use compression even if supported */
#define SSL_OP_NO_COMPRESSION                         0x00020000
/* Permit unsafe legacy renegotiation */
#define SSL_OP_ALLOW_UNSAFE_LEGACY_RENEGOTIATION      0x00040000
/* If set, always create a new key when using tmp_ecdh parameters */
#define SSL_OP_SINGLE_ECDH_USE                        0x00080000
/* If set, always create a new key when using tmp_dh parameters */
#define SSL_OP_SINGLE_DH_USE                          0x00100000
/* Does nothing: retained for compatibiity */
#define SSL_OP_EPHEMERAL_RSA                          0x0
/* Set on servers to choose the cipher according to the server's
 * preferences */
#define SSL_OP_CIPHER_SERVER_PREFERENCE               0x00400000
/* If set, a server will allow a client to issue a SSLv3.0 version number
 * as latest version supported in the premaster secret, even when TLSv1.0
 * (version 3.1) was announced in the client hello. Normally this is
 * forbidden to prevent version rollback attacks. */
#define SSL_OP_TLS_ROLLBACK_BUG                       0x00800000

#define SSL_OP_NO_SSLv2                               0x01000000
#define SSL_OP_NO_SSLv3                               0x02000000
#define SSL_OP_NO_TLSv1                               0x04000000
#define SSL_OP_NO_TLSv1_2                             0x08000000
#define SSL_OP_NO_TLSv1_1                             0x10000000

#define SSL_OP_NO_DTLSv1                              0x04000000
#define SSL_OP_NO_DTLSv1_2                            0x08000000

#define SSL_OP_NETSCAPE_CA_DN_BUG			            0x20000000
#define SSL_OP_NETSCAPE_DEMO_CIPHER_CHANGE_BUG        0x40000000
/* Make server add server-hello extension from early version of
 * cryptopro draft, when GOST ciphersuite is negotiated.
 * Required for interoperability with CryptoPro CSP 3.x
 */
#define SSL_OP_CRYPTOPRO_TLSEXT_BUG                   0x80000000


/*
 * flags for xbSSLContext:SetMode() / xbSSLContext:GetMode()
 */

/* Allow SSL_write(..., n) to return r with 0 < r < n (i.e. report success
 * when just a single record has been written): */
#define SSL_MODE_ENABLE_PARTIAL_WRITE                 0x00000001
/* Make it possible to retry SSL_write() with changed buffer location
 * (buffer contents must stay the same!); this is not the default to avoid
 * the misconception that non-blocking SSL_write() behaves like
 * non-blocking write(): */
#define SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER           0x00000002
/* Never bother the application with retries if the transport
 * is blocking: */
#define SSL_MODE_AUTO_RETRY                           0x00000004
/* Don't attempt to automatically build certificate chain */
#define SSL_MODE_NO_AUTO_CHAIN                        0x00000008
/* Save RAM by releasing read and write buffers when they're empty. (SSL3 and
 * TLS only.)  "Released" buffers are put onto a free-list in the context
 * or just freed (depending on the context's setting for freelist_max_len). */
#define SSL_MODE_RELEASE_BUFFERS                      0x00000010
/* Send the current time in the Random fields of the ClientHello and
 * ServerHello records for compatibility with hypothetical implementations
 * that require it.
 */
#define SSL_MODE_SEND_CLIENTHELLO_TIME                0x00000020
#define SSL_MODE_SEND_SERVERHELLO_TIME                0x00000040


/*
 * flags for xbSSLContext:SetVerify()
 *
 * use either SSL_VERIFY_NONE or SSL_VERIFY_PEER, the last 2 options
 * are 'ored' with SSL_VERIFY_PEER if they are desired
 */
#define SSL_VERIFY_NONE			            0x00
#define SSL_VERIFY_PEER			            0x01
#define SSL_VERIFY_FAIL_IF_NO_PEER_CERT	0x02
#define SSL_VERIFY_CLIENT_ONCE		      0x04



/* BIO stuff */

/* BIO_FILENAME_READ|BIO_CLOSE to open or close on free.
 * BIO_set_fp(in,stdin,BIO_NOCLOSE); */
#define BIO_NOCLOSE		0x00
#define BIO_CLOSE		0x01

/* These are used in the following macros and are passed to
 * BIO_ctrl() */
#define BIO_CTRL_RESET		1  /* opt - rewind/zero etc */
#define BIO_CTRL_EOF		2  /* opt - are we at the eof */
#define BIO_CTRL_INFO		3  /* opt - extra tit-bits */
#define BIO_CTRL_SET		4  /* man - set the 'IO' type */
#define BIO_CTRL_GET		5  /* man - get the 'IO' type */
#define BIO_CTRL_PUSH		6  /* opt - internal, used to signify change */
#define BIO_CTRL_POP		7  /* opt - internal, used to signify change */
#define BIO_CTRL_GET_CLOSE	8  /* man - set the 'close' on free */
#define BIO_CTRL_SET_CLOSE	9  /* man - set the 'close' on free */
#define BIO_CTRL_PENDING	10  /* opt - is their more data buffered */
#define BIO_CTRL_FLUSH		11  /* opt - 'flush' buffered output */
#define BIO_CTRL_DUP		12  /* man - extra stuff for 'duped' BIO */
#define BIO_CTRL_WPENDING	13  /* opt - number of bytes still to write */
/* callback is int cb(BIO *bio,state,ret); */
#define BIO_CTRL_SET_CALLBACK	14  /* opt - set callback function */
#define BIO_CTRL_GET_CALLBACK	15  /* opt - set callback function */

#define BIO_CTRL_SET_FILENAME	30	/* BIO_s_file special */

/* modifiers */
#define BIO_FP_READ		0x02
#define BIO_FP_WRITE		0x04
#define BIO_FP_APPEND		0x08
#define BIO_FP_TEXT		0x10

#define BIO_FLAGS_READ		0x01
#define BIO_FLAGS_WRITE		0x02
#define BIO_FLAGS_IO_SPECIAL	0x04
#define BIO_FLAGS_RWS (BIO_FLAGS_READ + BIO_FLAGS_WRITE + BIO_FLAGS_IO_SPECIAL)
#define BIO_FLAGS_SHOULD_RETRY	0x08

/* Used in BIO_gethostbyname() */
#define BIO_GHBN_CTRL_HITS		1
#define BIO_GHBN_CTRL_MISSES		2
#define BIO_GHBN_CTRL_CACHE_SIZE	3
#define BIO_GHBN_CTRL_GET_ENTRY		4
#define BIO_GHBN_CTRL_FLUSH		5

/* Mostly used in the SSL BIO */
/* Not used anymore
 * #define BIO_FLAGS_PROTOCOL_DELAYED_READ 0x10
 * #define BIO_FLAGS_PROTOCOL_DELAYED_WRITE 0x20
 * #define BIO_FLAGS_PROTOCOL_STARTUP	0x40
 */

#define BIO_FLAGS_BASE64_NO_NL	0x100

/* This is used with memory BIOs: it means we shouldn't free up or change the
 * data in any way.
 */
#define BIO_FLAGS_MEM_RDONLY	0x200


#define BIO_CONN_S_BEFORE		1
#define BIO_CONN_S_GET_IP		2
#define BIO_CONN_S_GET_PORT		3
#define BIO_CONN_S_CREATE_SOCKET	4
#define BIO_CONN_S_CONNECT		5
#define BIO_CONN_S_OK			6
#define BIO_CONN_S_BLOCKED_CONNECT	7
#define BIO_CONN_S_NBIO			8

#define BIO_C_SET_CONNECT			100
#define BIO_C_DO_STATE_MACHINE			101
#define BIO_C_SET_NBIO				102
#define BIO_C_SET_PROXY_PARAM			103
#define BIO_C_SET_FD				104
#define BIO_C_GET_FD				105
#define BIO_C_SET_FILE_PTR			106
#define BIO_C_GET_FILE_PTR			107
#define BIO_C_SET_FILENAME			108
#define BIO_C_SET_SSL				109
#define BIO_C_GET_SSL				110
#define BIO_C_SET_MD				111
#define BIO_C_GET_MD				112
#define BIO_C_GET_CIPHER_STATUS			113
#define BIO_C_SET_BUF_MEM			114
#define BIO_C_GET_BUF_MEM_PTR			115
#define BIO_C_GET_BUFF_NUM_LINES		116
#define BIO_C_SET_BUFF_SIZE			117
#define BIO_C_SET_ACCEPT			118
#define BIO_C_SSL_MODE				119
#define BIO_C_GET_MD_CTX			120
#define BIO_C_GET_PROXY_PARAM			121
#define BIO_C_SET_BUFF_READ_DATA		122 /* data to read first */
#define BIO_C_GET_CONNECT			123
#define BIO_C_GET_ACCEPT			124
#define BIO_C_SET_SSL_RENEGOTIATE_BYTES		125
#define BIO_C_GET_SSL_NUM_RENEGOTIATES		126
#define BIO_C_SET_SSL_RENEGOTIATE_TIMEOUT	127
#define BIO_C_FILE_SEEK				128
#define BIO_C_GET_CIPHER_CTX			129
#define BIO_C_SET_BUF_MEM_EOF_RETURN		130/*return end of input value*/
#define BIO_C_SET_BIND_MODE			131
#define BIO_C_GET_BIND_MODE			132
#define BIO_C_FILE_TELL				133
#define BIO_C_GET_SOCKS				134
#define BIO_C_SET_SOCKS				135

#define BIO_C_SET_WRITE_BUF_SIZE		136/* for BIO_s_bio */
#define BIO_C_GET_WRITE_BUF_SIZE		137
#define BIO_C_MAKE_BIO_PAIR			138
#define BIO_C_DESTROY_BIO_PAIR			139
#define BIO_C_GET_WRITE_GUARANTEE		140
#define BIO_C_GET_READ_REQUEST			141
#define BIO_C_SHUTDOWN_WR			142
#define BIO_C_NREAD0				143
#define BIO_C_NREAD				144
#define BIO_C_NWRITE0				145
#define BIO_C_NWRITE				146
#define BIO_C_RESET_READ_REQUEST		147


/* OpenSSL NID list */

/* list of pre-defined eliptic curves that may be used by xbSSLContext:SetECDHCurve method */

/* start of eliptic curve list */
#define NID_X9_62_c2pnb163v1         684 // c2pnb163v1: X9.62 curve over a 163 bit binary field
#define NID_X9_62_c2pnb163v2         685 // c2pnb163v2: X9.62 curve over a 163 bit binary field
#define NID_X9_62_c2pnb163v3         686 // c2pnb163v3: X9.62 curve over a 163 bit binary field
#define NID_X9_62_c2pnb176v1         687 // c2pnb176v1: X9.62 curve over a 176 bit binary field
#define NID_X9_62_c2tnb191v1         688 // c2tnb191v1: X9.62 curve over a 191 bit binary field
#define NID_X9_62_c2tnb191v2         689 // c2tnb191v2: X9.62 curve over a 191 bit binary field
#define NID_X9_62_c2tnb191v3         690 // c2tnb191v3: X9.62 curve over a 191 bit binary field
#define NID_X9_62_c2onb191v4         691
#define NID_X9_62_c2onb191v5         692
#define NID_X9_62_c2pnb208w1         693 // c2pnb208w1: X9.62 curve over a 208 bit binary field
#define NID_X9_62_c2tnb239v1         694 // c2tnb239v1: X9.62 curve over a 239 bit binary field
#define NID_X9_62_c2tnb239v2         695 // c2tnb239v2: X9.62 curve over a 239 bit binary field
#define NID_X9_62_c2tnb239v3         696 // c2tnb239v3: X9.62 curve over a 239 bit binary field
#define NID_X9_62_c2onb239v4         697
#define NID_X9_62_c2onb239v5         698
#define NID_X9_62_c2pnb272w1         699 // c2pnb272w1: X9.62 curve over a 272 bit binary field
#define NID_X9_62_c2pnb304w1         700 // c2pnb304w1: X9.62 curve over a 304 bit binary field
#define NID_X9_62_c2tnb359v1         701 // c2tnb359v1: X9.62 curve over a 359 bit binary field
#define NID_X9_62_c2pnb368w1         702 // c2pnb368w1: X9.62 curve over a 368 bit binary field
#define NID_X9_62_c2tnb431r1         703 // c2tnb431r1: X9.62 curve over a 431 bit binary field
#define NID_X9_62_prime192v1         409 // prime192v1: NIST/X9.62/SECG curve over a 192 bit prime field
#define NID_X9_62_prime192v2         410 // prime192v2: X9.62 curve over a 192 bit prime field
#define NID_X9_62_prime192v3         411 // prime192v3: X9.62 curve over a 192 bit prime field
#define NID_X9_62_prime239v1         412 // prime239v1: X9.62 curve over a 239 bit prime field
#define NID_X9_62_prime239v2         413 // prime239v2: X9.62 curve over a 239 bit prime field
#define NID_X9_62_prime239v3         414 // prime239v3: X9.62 curve over a 239 bit prime field
#define NID_X9_62_prime256v1         415 // prime256v1: X9.62/SECG curve over a 256 bit prime field
#define NID_secp112r1                704 // secp112r1 : SECG/WTLS curve over a 112 bit prime field
#define NID_secp112r2                705 // secp112r2 : SECG curve over a 112 bit prime field
#define NID_secp128r1                706 // secp128r1 : SECG curve over a 128 bit prime field
#define NID_secp128r2                707 // secp128r2 : SECG curve over a 128 bit prime field
#define NID_secp160k1                708 // secp160k1 : SECG curve over a 160 bit prime field
#define NID_secp160r1                709 // secp160r1 : SECG curve over a 160 bit prime field
#define NID_secp160r2                710 // secp160r2 : SECG/WTLS curve over a 160 bit prime field
#define NID_secp192k1                711 // secp192k1 : SECG curve over a 192 bit prime field
#define NID_secp224k1                712 // secp224k1 : SECG curve over a 224 bit prime field
#define NID_secp224r1                713 // secp224r1 : NIST/SECG curve over a 224 bit prime field
#define NID_secp256k1                714 // secp256k1 : SECG curve over a 256 bit prime field
#define NID_secp384r1                715 // secp384r1 : NIST/SECG curve over a 384 bit prime field
#define NID_secp521r1                716 // secp521r1 : NIST/SECG curve over a 521 bit prime field
#define NID_sect113r1                717 // sect113r1 : SECG curve over a 113 bit binary field
#define NID_sect113r2                718 // sect113r2 : SECG curve over a 113 bit binary field
#define NID_sect131r1                719 // sect131r1 : SECG/WTLS curve over a 131 bit binary field
#define NID_sect131r2                720 // sect131r2 : SECG curve over a 131 bit binary field
#define NID_sect163k1                721 // sect163k1 : NIST/SECG/WTLS curve over a 163 bit binary field
#define NID_sect163r1                722 // sect163r1 : SECG curve over a 163 bit binary field
#define NID_sect163r2                723 // sect163r2 : NIST/SECG curve over a 163 bit binary field
#define NID_sect193r1                724 // sect193r1 : SECG curve over a 193 bit binary field
#define NID_sect193r2                725 // sect193r2 : SECG curve over a 193 bit binary field
#define NID_sect233k1                726 // sect233k1 : NIST/SECG/WTLS curve over a 233 bit binary field
#define NID_sect233r1                727 // sect233r1 : NIST/SECG/WTLS curve over a 233 bit binary field
#define NID_sect239k1                728 // sect239k1 : SECG curve over a 239 bit binary field
#define NID_sect283k1                729 // sect283k1 : NIST/SECG curve over a 283 bit binary field
#define NID_sect283r1                730 // sect283r1 : NIST/SECG curve over a 283 bit binary field
#define NID_sect409k1                731 // sect409k1 : NIST/SECG curve over a 409 bit binary field
#define NID_sect409r1                732 // sect409r1 : NIST/SECG curve over a 409 bit binary field
#define NID_sect571k1                733 // sect571k1 : NIST/SECG curve over a 571 bit binary field
#define NID_sect571r1                734 // sect571r1 : NIST/SECG curve over a 571 bit binary field
#define NID_wap_wsg_idm_ecid_wtls1   735 // wap-wsg-idm-ecid-wtls1: WTLS curve over a 113 bit binary field
#define NID_wap_wsg_idm_ecid_wtls3   736 // wap-wsg-idm-ecid-wtls3: NIST/SECG/WTLS curve over a 163 bit binary field
#define NID_wap_wsg_idm_ecid_wtls4   737 // wap-wsg-idm-ecid-wtls4: SECG curve over a 113 bit binary field
#define NID_wap_wsg_idm_ecid_wtls5   738 // wap-wsg-idm-ecid-wtls5: X9.62 curve over a 163 bit binary field
#define NID_wap_wsg_idm_ecid_wtls6   739 // wap-wsg-idm-ecid-wtls6: SECG/WTLS curve over a 112 bit prime field
#define NID_wap_wsg_idm_ecid_wtls7   740 // wap-wsg-idm-ecid-wtls7: SECG/WTLS curve over a 160 bit prime field
#define NID_wap_wsg_idm_ecid_wtls8   741 // wap-wsg-idm-ecid-wtls8: WTLS curve over a 112 bit prime field
#define NID_wap_wsg_idm_ecid_wtls9   742 // wap-wsg-idm-ecid-wtls9: WTLS curve over a 160 bit prime field
#define NID_wap_wsg_idm_ecid_wtls10  743 // wap-wsg-idm-ecid-wtls10: NIST/SECG/WTLS curve over a 233 bit binary field
#define NID_wap_wsg_idm_ecid_wtls11  744 // wap-wsg-idm-ecid-wtls11: NIST/SECG/WTLS curve over a 233 bit binary field
#define NID_wap_wsg_idm_ecid_wtls12  745 // wap-wsg-idm-ecid-wtls12: WTLS curvs over a 224 bit prime field
/* end of eliptic curve list */


#define NID_undef			0
#define NID_itu_t		645
#define NID_ccitt		404
#define NID_iso		181
#define NID_joint_iso_itu_t		646
#define NID_joint_iso_ccitt		393
#define NID_member_body		182
#define NID_identified_organization		676
#define NID_hmac_md5		780
#define NID_hmac_sha1		781
#define NID_certicom_arc		677
#define NID_international_organizations		647
#define NID_wap		678
#define NID_wap_wsg		679
#define NID_selected_attribute_types		394
#define NID_clearance		395
#define NID_ISO_US		183
#define NID_X9_57		184
#define NID_X9cm		185
#define NID_dsa		116
#define NID_dsaWithSHA1		113
#define NID_ansi_X9_62		405
#define NID_X9_62_prime_field		406
#define NID_X9_62_characteristic_two_field		407
#define NID_X9_62_id_characteristic_two_basis		680
#define NID_X9_62_onBasis		681
#define NID_X9_62_tpBasis		682
#define NID_X9_62_ppBasis		683
#define NID_X9_62_id_ecPublicKey		408
#define NID_ecdsa_with_SHA1		416
#define NID_ecdsa_with_Recommended		791
#define NID_ecdsa_with_Specified		792
#define NID_ecdsa_with_SHA224		793
#define NID_ecdsa_with_SHA256		794
#define NID_ecdsa_with_SHA384		795
#define NID_ecdsa_with_SHA512		796
#define NID_cast5_cbc		108
#define NID_cast5_ecb		109
#define NID_cast5_cfb64		110
#define NID_cast5_ofb64		111
#define NID_pbeWithMD5AndCast5_CBC		112
#define NID_id_PasswordBasedMAC		782
#define NID_id_DHBasedMac		783
#define NID_rsadsi		1
#define NID_pkcs		2
#define NID_pkcs1		186
#define NID_rsaEncryption		6
#define NID_md2WithRSAEncryption		7
#define NID_md4WithRSAEncryption		396
#define NID_md5WithRSAEncryption		8
#define NID_sha1WithRSAEncryption		65
#define NID_rsaesOaep		919
#define NID_mgf1		911
#define NID_rsassaPss		912
#define NID_sha256WithRSAEncryption		668
#define NID_sha384WithRSAEncryption		669
#define NID_sha512WithRSAEncryption		670
#define NID_sha224WithRSAEncryption		671
#define NID_pkcs3		27
#define NID_dhKeyAgreement		28
#define NID_pkcs5		187
#define NID_pbeWithMD2AndDES_CBC		9
#define NID_pbeWithMD5AndDES_CBC		10
#define NID_pbeWithMD2AndRC2_CBC		168
#define NID_pbeWithMD5AndRC2_CBC		169
#define NID_pbeWithSHA1AndDES_CBC		170
#define NID_pbeWithSHA1AndRC2_CBC		68
#define NID_id_pbkdf2		69
#define NID_pbes2		161
#define NID_pbmac1		162
#define NID_pkcs7		20
#define NID_pkcs7_data		21
#define NID_pkcs7_signed		22
#define NID_pkcs7_enveloped		23
#define NID_pkcs7_signedAndEnveloped		24
#define NID_pkcs7_digest		25
#define NID_pkcs7_encrypted		26
#define NID_pkcs9		47
#define NID_pkcs9_emailAddress		48
#define NID_pkcs9_unstructuredName		49
#define NID_pkcs9_contentType		50
#define NID_pkcs9_messageDigest		51
#define NID_pkcs9_signingTime		52
#define NID_pkcs9_countersignature		53
#define NID_pkcs9_challengePassword		54
#define NID_pkcs9_unstructuredAddress		55
#define NID_pkcs9_extCertAttributes		56
#define NID_ext_req		172
#define NID_SMIMECapabilities		167
#define NID_SMIME		188
#define NID_id_smime_mod		189
#define NID_id_smime_ct		190
#define NID_id_smime_aa		191
#define NID_id_smime_alg		192
#define NID_id_smime_cd		193
#define NID_id_smime_spq		194
#define NID_id_smime_cti		195
#define NID_id_smime_mod_cms		196
#define NID_id_smime_mod_ess		197
#define NID_id_smime_mod_oid		198
#define NID_id_smime_mod_msg_v3		199
#define NID_id_smime_mod_ets_eSignature_88		200
#define NID_id_smime_mod_ets_eSignature_97		201
#define NID_id_smime_mod_ets_eSigPolicy_88		202
#define NID_id_smime_mod_ets_eSigPolicy_97		203
#define NID_id_smime_ct_receipt		204
#define NID_id_smime_ct_authData		205
#define NID_id_smime_ct_publishCert		206
#define NID_id_smime_ct_TSTInfo		207
#define NID_id_smime_ct_TDTInfo		208
#define NID_id_smime_ct_contentInfo		209
#define NID_id_smime_ct_DVCSRequestData		210
#define NID_id_smime_ct_DVCSResponseData		211
#define NID_id_smime_ct_compressedData		786
#define NID_id_ct_asciiTextWithCRLF		787
#define NID_id_smime_aa_receiptRequest		212
#define NID_id_smime_aa_securityLabel		213
#define NID_id_smime_aa_mlExpandHistory		214
#define NID_id_smime_aa_contentHint		215
#define NID_id_smime_aa_msgSigDigest		216
#define NID_id_smime_aa_encapContentType		217
#define NID_id_smime_aa_contentIdentifier		218
#define NID_id_smime_aa_macValue		219
#define NID_id_smime_aa_equivalentLabels		220
#define NID_id_smime_aa_contentReference		221
#define NID_id_smime_aa_encrypKeyPref		222
#define NID_id_smime_aa_signingCertificate		223
#define NID_id_smime_aa_smimeEncryptCerts		224
#define NID_id_smime_aa_timeStampToken		225
#define NID_id_smime_aa_ets_sigPolicyId		226
#define NID_id_smime_aa_ets_commitmentType		227
#define NID_id_smime_aa_ets_signerLocation		228
#define NID_id_smime_aa_ets_signerAttr		229
#define NID_id_smime_aa_ets_otherSigCert		230
#define NID_id_smime_aa_ets_contentTimestamp		231
#define NID_id_smime_aa_ets_CertificateRefs		232
#define NID_id_smime_aa_ets_RevocationRefs		233
#define NID_id_smime_aa_ets_certValues		234
#define NID_id_smime_aa_ets_revocationValues		235
#define NID_id_smime_aa_ets_escTimeStamp		236
#define NID_id_smime_aa_ets_certCRLTimestamp		237
#define NID_id_smime_aa_ets_archiveTimeStamp		238
#define NID_id_smime_aa_signatureType		239
#define NID_id_smime_aa_dvcs_dvc		240
#define NID_id_smime_alg_ESDHwith3DES		241
#define NID_id_smime_alg_ESDHwithRC2		242
#define NID_id_smime_alg_3DESwrap		243
#define NID_id_smime_alg_RC2wrap		244
#define NID_id_smime_alg_ESDH		245
#define NID_id_smime_alg_CMS3DESwrap		246
#define NID_id_smime_alg_CMSRC2wrap		247
#define NID_id_alg_PWRI_KEK		893
#define NID_id_smime_cd_ldap		248
#define NID_id_smime_spq_ets_sqt_uri		249
#define NID_id_smime_spq_ets_sqt_unotice		250
#define NID_id_smime_cti_ets_proofOfOrigin		251
#define NID_id_smime_cti_ets_proofOfReceipt		252
#define NID_id_smime_cti_ets_proofOfDelivery		253
#define NID_id_smime_cti_ets_proofOfSender		254
#define NID_id_smime_cti_ets_proofOfApproval		255
#define NID_id_smime_cti_ets_proofOfCreation		256
#define NID_friendlyName		156
#define NID_localKeyID		157
#define NID_ms_csp_name		417
#define NID_LocalKeySet		856
#define NID_x509Certificate		158
#define NID_sdsiCertificate		159
#define NID_x509Crl		160
#define NID_pbe_WithSHA1And128BitRC4		144
#define NID_pbe_WithSHA1And40BitRC4		145
#define NID_pbe_WithSHA1And3_Key_TripleDES_CBC		146
#define NID_pbe_WithSHA1And2_Key_TripleDES_CBC		147
#define NID_pbe_WithSHA1And128BitRC2_CBC		148
#define NID_pbe_WithSHA1And40BitRC2_CBC		149
#define NID_keyBag		150
#define NID_pkcs8ShroudedKeyBag		151
#define NID_certBag		152
#define NID_crlBag		153
#define NID_secretBag		154
#define NID_safeContentsBag		155
#define NID_md2		3
#define NID_md4		257
#define NID_md5		4
#define NID_md5_sha1		114
#define NID_hmacWithMD5		797
#define NID_hmacWithSHA1		163
#define NID_hmacWithSHA224		798
#define NID_hmacWithSHA256		799
#define NID_hmacWithSHA384		800
#define NID_hmacWithSHA512		801
#define NID_rc2_cbc		37
#define NID_rc2_ecb		38
#define NID_rc2_cfb64		39
#define NID_rc2_ofb64		40
#define NID_rc2_40_cbc		98
#define NID_rc2_64_cbc		166
#define NID_rc4		5
#define NID_rc4_40		97
#define NID_des_ede3_cbc		44
#define NID_rc5_cbc		120
#define NID_rc5_ecb		121
#define NID_rc5_cfb64		122
#define NID_rc5_ofb64		123
#define NID_ms_ext_req		171
#define NID_ms_code_ind		134
#define NID_ms_code_com		135
#define NID_ms_ctl_sign		136
#define NID_ms_sgc		137
#define NID_ms_efs		138
#define NID_ms_smartcard_login		648
#define NID_ms_upn		649
#define NID_idea_cbc		34
#define NID_idea_ecb		36
#define NID_idea_cfb64		35
#define NID_idea_ofb64		46
#define NID_bf_cbc		91
#define NID_bf_ecb		92
#define NID_bf_cfb64		93
#define NID_bf_ofb64		94
#define NID_id_pkix		127
#define NID_id_pkix_mod		258
#define NID_id_pe		175
#define NID_id_qt		259
#define NID_id_kp		128
#define NID_id_it		260
#define NID_id_pkip		261
#define NID_id_alg		262
#define NID_id_cmc		263
#define NID_id_on		264
#define NID_id_pda		265
#define NID_id_aca		266
#define NID_id_qcs		267
#define NID_id_cct		268
#define NID_id_ppl		662
#define NID_id_ad		176
#define NID_id_pkix1_explicit_88		269
#define NID_id_pkix1_implicit_88		270
#define NID_id_pkix1_explicit_93		271
#define NID_id_pkix1_implicit_93		272
#define NID_id_mod_crmf		273
#define NID_id_mod_cmc		274
#define NID_id_mod_kea_profile_88		275
#define NID_id_mod_kea_profile_93		276
#define NID_id_mod_cmp		277
#define NID_id_mod_qualified_cert_88		278
#define NID_id_mod_qualified_cert_93		279
#define NID_id_mod_attribute_cert		280
#define NID_id_mod_timestamp_protocol		281
#define NID_id_mod_ocsp		282
#define NID_id_mod_dvcs		283
#define NID_id_mod_cmp2000		284
#define NID_info_access		177
#define NID_biometricInfo		285
#define NID_qcStatements		286
#define NID_ac_auditEntity		287
#define NID_ac_targeting		288
#define NID_aaControls		289
#define NID_sbgp_ipAddrBlock		290
#define NID_sbgp_autonomousSysNum		291
#define NID_sbgp_routerIdentifier		292
#define NID_ac_proxying		397
#define NID_sinfo_access		398
#define NID_proxyCertInfo		663
#define NID_id_qt_cps		164
#define NID_id_qt_unotice		165
#define NID_textNotice		293
#define NID_server_auth		129
#define NID_client_auth		130
#define NID_code_sign		131
#define NID_email_protect		132
#define NID_ipsecEndSystem		294
#define NID_ipsecTunnel		295
#define NID_ipsecUser		296
#define NID_time_stamp		133
#define NID_OCSP_sign		180
#define NID_dvcs		297
#define NID_id_it_caProtEncCert		298
#define NID_id_it_signKeyPairTypes		299
#define NID_id_it_encKeyPairTypes		300
#define NID_id_it_preferredSymmAlg		301
#define NID_id_it_caKeyUpdateInfo		302
#define NID_id_it_currentCRL		303
#define NID_id_it_unsupportedOIDs		304
#define NID_id_it_subscriptionRequest		305
#define NID_id_it_subscriptionResponse		306
#define NID_id_it_keyPairParamReq		307
#define NID_id_it_keyPairParamRep		308
#define NID_id_it_revPassphrase		309
#define NID_id_it_implicitConfirm		310
#define NID_id_it_confirmWaitTime		311
#define NID_id_it_origPKIMessage		312
#define NID_id_it_suppLangTags		784
#define NID_id_regCtrl		313
#define NID_id_regInfo		314
#define NID_id_regCtrl_regToken		315
#define NID_id_regCtrl_authenticator		316
#define NID_id_regCtrl_pkiPublicationInfo		317
#define NID_id_regCtrl_pkiArchiveOptions		318
#define NID_id_regCtrl_oldCertID		319
#define NID_id_regCtrl_protocolEncrKey		320
#define NID_id_regInfo_utf8Pairs		321
#define NID_id_regInfo_certReq		322
#define NID_id_alg_des40		323
#define NID_id_alg_noSignature		324
#define NID_id_alg_dh_sig_hmac_sha1		325
#define NID_id_alg_dh_pop		326
#define NID_id_cmc_statusInfo		327
#define NID_id_cmc_identification		328
#define NID_id_cmc_identityProof		329
#define NID_id_cmc_dataReturn		330
#define NID_id_cmc_transactionId		331
#define NID_id_cmc_senderNonce		332
#define NID_id_cmc_recipientNonce		333
#define NID_id_cmc_addExtensions		334
#define NID_id_cmc_encryptedPOP		335
#define NID_id_cmc_decryptedPOP		336
#define NID_id_cmc_lraPOPWitness		337
#define NID_id_cmc_getCert		338
#define NID_id_cmc_getCRL		339
#define NID_id_cmc_revokeRequest		340
#define NID_id_cmc_regInfo		341
#define NID_id_cmc_responseInfo		342
#define NID_id_cmc_queryPending		343
#define NID_id_cmc_popLinkRandom		344
#define NID_id_cmc_popLinkWitness		345
#define NID_id_cmc_confirmCertAcceptance		346
#define NID_id_on_personalData		347
#define NID_id_on_permanentIdentifier		858
#define NID_id_pda_dateOfBirth		348
#define NID_id_pda_placeOfBirth		349
#define NID_id_pda_gender		351
#define NID_id_pda_countryOfCitizenship		352
#define NID_id_pda_countryOfResidence		353
#define NID_id_aca_authenticationInfo		354
#define NID_id_aca_accessIdentity		355
#define NID_id_aca_chargingIdentity		356
#define NID_id_aca_group		357
#define NID_id_aca_role		358
#define NID_id_aca_encAttrs		399
#define NID_id_qcs_pkixQCSyntax_v1		359
#define NID_id_cct_crs		360
#define NID_id_cct_PKIData		361
#define NID_id_cct_PKIResponse		362
#define NID_id_ppl_anyLanguage		664
#define NID_id_ppl_inheritAll		665
#define NID_Independent		667
#define NID_ad_OCSP		178
#define NID_ad_ca_issuers		179
#define NID_ad_timeStamping		363
#define NID_ad_dvcs		364
#define NID_caRepository		785
#define NID_id_pkix_OCSP_basic		365
#define NID_id_pkix_OCSP_Nonce		366
#define NID_id_pkix_OCSP_CrlID		367
#define NID_id_pkix_OCSP_acceptableResponses		368
#define NID_id_pkix_OCSP_noCheck		369
#define NID_id_pkix_OCSP_archiveCutoff		370
#define NID_id_pkix_OCSP_serviceLocator		371
#define NID_id_pkix_OCSP_extendedStatus		372
#define NID_id_pkix_OCSP_valid		373
#define NID_id_pkix_OCSP_path		374
#define NID_id_pkix_OCSP_trustRoot		375
#define NID_algorithm		376
#define NID_md5WithRSA		104
#define NID_des_ecb		29
#define NID_des_cbc		31
#define NID_des_ofb64		45
#define NID_des_cfb64		30
#define NID_rsaSignature		377
#define NID_dsa_2		67
#define NID_dsaWithSHA		66
#define NID_shaWithRSAEncryption		42
#define NID_des_ede_ecb		32
#define NID_des_ede3_ecb		33
#define NID_des_ede_cbc		43
#define NID_des_ede_cfb64		60
#define NID_des_ede3_cfb64		61
#define NID_des_ede_ofb64		62
#define NID_des_ede3_ofb64		63
#define NID_desx_cbc		80
#define NID_sha		41
#define NID_sha1		64
#define NID_dsaWithSHA1_2		70
#define NID_sha1WithRSA		115
#define NID_ripemd160		117
#define NID_ripemd160WithRSA		119
#define NID_sxnet		143
#define NID_X500		11
#define NID_X509		12
#define NID_commonName		13
#define NID_surname		100
#define NID_serialNumber		105
#define NID_countryName		14
#define NID_localityName		15
#define NID_stateOrProvinceName		16
#define NID_streetAddress		660
#define NID_organizationName		17
#define NID_organizationalUnitName		18
#define NID_title		106
#define NID_description		107
#define NID_searchGuide		859
#define NID_businessCategory		860
#define NID_postalAddress		861
#define NID_postalCode		661
#define NID_postOfficeBox		862
#define NID_physicalDeliveryOfficeName		863
#define NID_telephoneNumber		864
#define NID_telexNumber		865
#define NID_teletexTerminalIdentifier		866
#define NID_facsimileTelephoneNumber		867
#define NID_x121Address		868
#define NID_internationaliSDNNumber		869
#define NID_registeredAddress		870
#define NID_destinationIndicator		871
#define NID_preferredDeliveryMethod		872
#define NID_presentationAddress		873
#define NID_supportedApplicationContext		874
#define NID_member		875
#define NID_owner		876
#define NID_roleOccupant		877
#define NID_seeAlso		878
#define NID_userPassword		879
#define NID_userCertificate		880
#define NID_cACertificate		881
#define NID_authorityRevocationList		882
#define NID_certificateRevocationList		883
#define NID_crossCertificatePair		884
#define NID_name		173
#define NID_givenName		99
#define NID_initials		101
#define NID_generationQualifier		509
#define NID_x500UniqueIdentifier		503
#define NID_dnQualifier		174
#define NID_enhancedSearchGuide		885
#define NID_protocolInformation		886
#define NID_distinguishedName		887
#define NID_uniqueMember		888
#define NID_houseIdentifier		889
#define NID_supportedAlgorithms		890
#define NID_deltaRevocationList		891
#define NID_dmdName		892
#define NID_pseudonym		510
#define NID_role		400
#define NID_X500algorithms		378
#define NID_rsa		19
#define NID_mdc2WithRSA		96
#define NID_mdc2		95
#define NID_id_ce		81
#define NID_subject_directory_attributes		769
#define NID_subject_key_identifier		82
#define NID_key_usage		83
#define NID_private_key_usage_period		84
#define NID_subject_alt_name		85
#define NID_issuer_alt_name		86
#define NID_basic_constraints		87
#define NID_crl_number		88
#define NID_crl_reason		141
#define NID_invalidity_date		142
#define NID_delta_crl		140
#define NID_issuing_distribution_point		770
#define NID_certificate_issuer		771
#define NID_name_constraints		666
#define NID_crl_distribution_points		103
#define NID_certificate_policies		89
#define NID_any_policy		746
#define NID_policy_mappings		747
#define NID_authority_key_identifier		90
#define NID_policy_constraints		401
#define NID_ext_key_usage		126
#define NID_freshest_crl		857
#define NID_inhibit_any_policy		748
#define NID_target_information		402
#define NID_no_rev_avail		403
#define NID_anyExtendedKeyUsage		910
#define NID_netscape		57
#define NID_netscape_cert_extension		58
#define NID_netscape_data_type		59
#define NID_netscape_cert_type		71
#define NID_netscape_base_url		72
#define NID_netscape_revocation_url		73
#define NID_netscape_ca_revocation_url		74
#define NID_netscape_renewal_url		75
#define NID_netscape_ca_policy_url		76
#define NID_netscape_ssl_server_name		77
#define NID_netscape_comment		78
#define NID_netscape_cert_sequence		79
#define NID_ns_sgc		139
#define NID_org		379
#define NID_dod		380
#define NID_iana		381
#define NID_Directory		382
#define NID_Management		383
#define NID_Experimental		384
#define NID_Private		385
#define NID_Security		386
#define NID_SNMPv2		387
#define NID_Mail		388
#define NID_Enterprises		389
#define NID_dcObject		390
#define NID_mime_mhs		504
#define NID_mime_mhs_headings		505
#define NID_mime_mhs_bodies		506
#define NID_id_hex_partial_message		507
#define NID_id_hex_multipart_message		508
#define NID_rle_compression		124
#define NID_zlib_compression		125
#define NID_aes_128_ecb		418
#define NID_aes_128_cbc		419
#define NID_aes_128_ofb128		420
#define NID_aes_128_cfb128		421
#define NID_id_aes128_wrap		788
#define NID_aes_128_gcm		895
#define NID_aes_128_ccm		896
#define NID_id_aes128_wrap_pad		897
#define NID_aes_192_ecb		422
#define NID_aes_192_cbc		423
#define NID_aes_192_ofb128		424
#define NID_aes_192_cfb128		425
#define NID_id_aes192_wrap		789
#define NID_aes_192_gcm		898
#define NID_aes_192_ccm		899
#define NID_id_aes192_wrap_pad		900
#define NID_aes_256_ecb		426
#define NID_aes_256_cbc		427
#define NID_aes_256_ofb128		428
#define NID_aes_256_cfb128		429
#define NID_id_aes256_wrap		790
#define NID_aes_256_gcm		901
#define NID_aes_256_ccm		902
#define NID_id_aes256_wrap_pad		903
#define NID_aes_128_cfb1		650
#define NID_aes_192_cfb1		651
#define NID_aes_256_cfb1		652
#define NID_aes_128_cfb8		653
#define NID_aes_192_cfb8		654
#define NID_aes_256_cfb8		655
#define NID_aes_128_ctr		904
#define NID_aes_192_ctr		905
#define NID_aes_256_ctr		906
#define NID_aes_128_xts		913
#define NID_aes_256_xts		914
#define NID_des_cfb1		656
#define NID_des_cfb8		657
#define NID_des_ede3_cfb1		658
#define NID_des_ede3_cfb8		659
#define NID_sha256		672
#define NID_sha384		673
#define NID_sha512		674
#define NID_sha224		675
#define NID_dsa_with_SHA224		802
#define NID_dsa_with_SHA256		803
#define NID_hold_instruction_code		430
#define NID_hold_instruction_none		431
#define NID_hold_instruction_call_issuer		432
#define NID_hold_instruction_reject		433
#define NID_data		434
#define NID_pss		435
#define NID_ucl		436
#define NID_pilot		437
#define NID_pilotAttributeType		438
#define NID_pilotAttributeSyntax		439
#define NID_pilotObjectClass		440
#define NID_pilotGroups		441
#define NID_iA5StringSyntax		442
#define NID_caseIgnoreIA5StringSyntax		443
#define NID_pilotObject		444
#define NID_pilotPerson		445
#define NID_account		446
#define NID_document		447
#define NID_room		448
#define NID_documentSeries		449
#define NID_Domain		392
#define NID_rFC822localPart		450
#define NID_dNSDomain		451
#define NID_domainRelatedObject		452
#define NID_friendlyCountry		453
#define NID_simpleSecurityObject		454
#define NID_pilotOrganization		455
#define NID_pilotDSA		456
#define NID_qualityLabelledData		457
#define NID_userId		458
#define NID_textEncodedORAddress		459
#define NID_rfc822Mailbox		460
#define NID_info		461
#define NID_favouriteDrink		462
#define NID_roomNumber		463
#define NID_photo		464
#define NID_userClass		465
#define NID_host		466
#define NID_manager		467
#define NID_documentIdentifier		468
#define NID_documentTitle		469
#define NID_documentVersion		470
#define NID_documentAuthor		471
#define NID_documentLocation		472
#define NID_homeTelephoneNumber		473
#define NID_secretary		474
#define NID_otherMailbox		475
#define NID_lastModifiedTime		476
#define NID_lastModifiedBy		477
#define NID_domainComponent		391
#define NID_aRecord		478
#define NID_pilotAttributeType27		479
#define NID_mXRecord		480
#define NID_nSRecord		481
#define NID_sOARecord		482
#define NID_cNAMERecord		483
#define NID_associatedDomain		484
#define NID_associatedName		485
#define NID_homePostalAddress		486
#define NID_personalTitle		487
#define NID_mobileTelephoneNumber		488
#define NID_pagerTelephoneNumber		489
#define NID_friendlyCountryName		490
#define NID_organizationalStatus		491
#define NID_janetMailbox		492
#define NID_mailPreferenceOption		493
#define NID_buildingName		494
#define NID_dSAQuality		495
#define NID_singleLevelQuality		496
#define NID_subtreeMinimumQuality		497
#define NID_subtreeMaximumQuality		498
#define NID_personalSignature		499
#define NID_dITRedirect		500
#define NID_audio		501
#define NID_documentPublisher		502
#define NID_id_set		512
#define NID_set_ctype		513
#define NID_set_msgExt		514
#define NID_set_attr		515
#define NID_set_policy		516
#define NID_set_certExt		517
#define NID_set_brand		518
#define NID_setct_PANData		519
#define NID_setct_PANToken		520
#define NID_setct_PANOnly		521
#define NID_setct_OIData		522
#define NID_setct_PI		523
#define NID_setct_PIData		524
#define NID_setct_PIDataUnsigned		525
#define NID_setct_HODInput		526
#define NID_setct_AuthResBaggage		527
#define NID_setct_AuthRevReqBaggage		528
#define NID_setct_AuthRevResBaggage		529
#define NID_setct_CapTokenSeq		530
#define NID_setct_PInitResData		531
#define NID_setct_PI_TBS		532
#define NID_setct_PResData		533
#define NID_setct_AuthReqTBS		534
#define NID_setct_AuthResTBS		535
#define NID_setct_AuthResTBSX		536
#define NID_setct_AuthTokenTBS		537
#define NID_setct_CapTokenData		538
#define NID_setct_CapTokenTBS		539
#define NID_setct_AcqCardCodeMsg		540
#define NID_setct_AuthRevReqTBS		541
#define NID_setct_AuthRevResData		542
#define NID_setct_AuthRevResTBS		543
#define NID_setct_CapReqTBS		544
#define NID_setct_CapReqTBSX		545
#define NID_setct_CapResData		546
#define NID_setct_CapRevReqTBS		547
#define NID_setct_CapRevReqTBSX		548
#define NID_setct_CapRevResData		549
#define NID_setct_CredReqTBS		550
#define NID_setct_CredReqTBSX		551
#define NID_setct_CredResData		552
#define NID_setct_CredRevReqTBS		553
#define NID_setct_CredRevReqTBSX		554
#define NID_setct_CredRevResData		555
#define NID_setct_PCertReqData		556
#define NID_setct_PCertResTBS		557
#define NID_setct_BatchAdminReqData		558
#define NID_setct_BatchAdminResData		559
#define NID_setct_CardCInitResTBS		560
#define NID_setct_MeAqCInitResTBS		561
#define NID_setct_RegFormResTBS		562
#define NID_setct_CertReqData		563
#define NID_setct_CertReqTBS		564
#define NID_setct_CertResData		565
#define NID_setct_CertInqReqTBS		566
#define NID_setct_ErrorTBS		567
#define NID_setct_PIDualSignedTBE		568
#define NID_setct_PIUnsignedTBE		569
#define NID_setct_AuthReqTBE		570
#define NID_setct_AuthResTBE		571
#define NID_setct_AuthResTBEX		572
#define NID_setct_AuthTokenTBE		573
#define NID_setct_CapTokenTBE		574
#define NID_setct_CapTokenTBEX		575
#define NID_setct_AcqCardCodeMsgTBE		576
#define NID_setct_AuthRevReqTBE		577
#define NID_setct_AuthRevResTBE		578
#define NID_setct_AuthRevResTBEB		579
#define NID_setct_CapReqTBE		580
#define NID_setct_CapReqTBEX		581
#define NID_setct_CapResTBE		582
#define NID_setct_CapRevReqTBE		583
#define NID_setct_CapRevReqTBEX		584
#define NID_setct_CapRevResTBE		585
#define NID_setct_CredReqTBE		586
#define NID_setct_CredReqTBEX		587
#define NID_setct_CredResTBE		588
#define NID_setct_CredRevReqTBE		589
#define NID_setct_CredRevReqTBEX		590
#define NID_setct_CredRevResTBE		591
#define NID_setct_BatchAdminReqTBE		592
#define NID_setct_BatchAdminResTBE		593
#define NID_setct_RegFormReqTBE		594
#define NID_setct_CertReqTBE		595
#define NID_setct_CertReqTBEX		596
#define NID_setct_CertResTBE		597
#define NID_setct_CRLNotificationTBS		598
#define NID_setct_CRLNotificationResTBS		599
#define NID_setct_BCIDistributionTBS		600
#define NID_setext_genCrypt		601
#define NID_setext_miAuth		602
#define NID_setext_pinSecure		603
#define NID_setext_pinAny		604
#define NID_setext_track2		605
#define NID_setext_cv		606
#define NID_set_policy_root		607
#define NID_setCext_hashedRoot		608
#define NID_setCext_certType		609
#define NID_setCext_merchData		610
#define NID_setCext_cCertRequired		611
#define NID_setCext_tunneling		612
#define NID_setCext_setExt		613
#define NID_setCext_setQualf		614
#define NID_setCext_PGWYcapabilities		615
#define NID_setCext_TokenIdentifier		616
#define NID_setCext_Track2Data		617
#define NID_setCext_TokenType		618
#define NID_setCext_IssuerCapabilities		619
#define NID_setAttr_Cert		620
#define NID_setAttr_PGWYcap		621
#define NID_setAttr_TokenType		622
#define NID_setAttr_IssCap		623
#define NID_set_rootKeyThumb		624
#define NID_set_addPolicy		625
#define NID_setAttr_Token_EMV		626
#define NID_setAttr_Token_B0Prime		627
#define NID_setAttr_IssCap_CVM		628
#define NID_setAttr_IssCap_T2		629
#define NID_setAttr_IssCap_Sig		630
#define NID_setAttr_GenCryptgrm		631
#define NID_setAttr_T2Enc		632
#define NID_setAttr_T2cleartxt		633
#define NID_setAttr_TokICCsig		634
#define NID_setAttr_SecDevSig		635
#define NID_set_brand_IATA_ATA		636
#define NID_set_brand_Diners		637
#define NID_set_brand_AmericanExpress		638
#define NID_set_brand_JCB		639
#define NID_set_brand_Visa		640
#define NID_set_brand_MasterCard		641
#define NID_set_brand_Novus		642
#define NID_des_cdmf		643
#define NID_rsaOAEPEncryptionSET		644
#define NID_ipsec3		749
#define NID_ipsec4		750
#define NID_whirlpool		804
#define NID_cryptopro		805
#define NID_cryptocom		806
#define NID_id_GostR3411_94_with_GostR3410_2001		807
#define NID_id_GostR3411_94_with_GostR3410_94		808
#define NID_id_GostR3411_94		809
#define NID_id_HMACGostR3411_94		810
#define NID_id_GostR3410_2001		811
#define NID_id_GostR3410_94		812
#define NID_id_Gost28147_89		813
#define NID_gost89_cnt		814
#define NID_id_Gost28147_89_MAC		815
#define NID_id_GostR3411_94_prf		816
#define NID_id_GostR3410_2001DH		817
#define NID_id_GostR3410_94DH		818
#define NID_id_Gost28147_89_CryptoPro_KeyMeshing		819
#define NID_id_Gost28147_89_None_KeyMeshing		820
#define NID_id_GostR3411_94_TestParamSet		821
#define NID_id_GostR3411_94_CryptoProParamSet		822
#define NID_id_Gost28147_89_TestParamSet		823
#define NID_id_Gost28147_89_CryptoPro_A_ParamSet		824
#define NID_id_Gost28147_89_CryptoPro_B_ParamSet		825
#define NID_id_Gost28147_89_CryptoPro_C_ParamSet		826
#define NID_id_Gost28147_89_CryptoPro_D_ParamSet		827
#define NID_id_Gost28147_89_CryptoPro_Oscar_1_1_ParamSet		828
#define NID_id_Gost28147_89_CryptoPro_Oscar_1_0_ParamSet		829
#define NID_id_Gost28147_89_CryptoPro_RIC_1_ParamSet		830
#define NID_id_GostR3410_94_TestParamSet		831
#define NID_id_GostR3410_94_CryptoPro_A_ParamSet		832
#define NID_id_GostR3410_94_CryptoPro_B_ParamSet		833
#define NID_id_GostR3410_94_CryptoPro_C_ParamSet		834
#define NID_id_GostR3410_94_CryptoPro_D_ParamSet		835
#define NID_id_GostR3410_94_CryptoPro_XchA_ParamSet		836
#define NID_id_GostR3410_94_CryptoPro_XchB_ParamSet		837
#define NID_id_GostR3410_94_CryptoPro_XchC_ParamSet		838
#define NID_id_GostR3410_2001_TestParamSet		839
#define NID_id_GostR3410_2001_CryptoPro_A_ParamSet		840
#define NID_id_GostR3410_2001_CryptoPro_B_ParamSet		841
#define NID_id_GostR3410_2001_CryptoPro_C_ParamSet		842
#define NID_id_GostR3410_2001_CryptoPro_XchA_ParamSet		843
#define NID_id_GostR3410_2001_CryptoPro_XchB_ParamSet		844
#define NID_id_GostR3410_94_a		845
#define NID_id_GostR3410_94_aBis		846
#define NID_id_GostR3410_94_b		847
#define NID_id_GostR3410_94_bBis		848
#define NID_id_Gost28147_89_cc		849
#define NID_id_GostR3410_94_cc		850
#define NID_id_GostR3410_2001_cc		851
#define NID_id_GostR3411_94_with_GostR3410_94_cc		852
#define NID_id_GostR3411_94_with_GostR3410_2001_cc		853
#define NID_id_GostR3410_2001_ParamSet_cc		854
#define NID_camellia_128_cbc		751
#define NID_camellia_192_cbc		752
#define NID_camellia_256_cbc		753
#define NID_id_camellia128_wrap		907
#define NID_id_camellia192_wrap		908
#define NID_id_camellia256_wrap		909
#define NID_camellia_128_ecb		754
#define NID_camellia_128_ofb128		766
#define NID_camellia_128_cfb128		757
#define NID_camellia_192_ecb		755
#define NID_camellia_192_ofb128		767
#define NID_camellia_192_cfb128		758
#define NID_camellia_256_ecb		756
#define NID_camellia_256_ofb128		768
#define NID_camellia_256_cfb128		759
#define NID_camellia_128_cfb1		760
#define NID_camellia_192_cfb1		761
#define NID_camellia_256_cfb1		762
#define NID_camellia_128_cfb8		763
#define NID_camellia_192_cfb8		764
#define NID_camellia_256_cfb8		765
#define NID_kisa		773
#define NID_seed_ecb		776
#define NID_seed_cbc		777
#define NID_seed_cfb128		779
#define NID_seed_ofb128		778
#define NID_hmac		855
#define NID_cmac		894
#define NID_rc4_hmac_md5		915
#define NID_aes_128_cbc_hmac_sha1		916
#define NID_aes_192_cbc_hmac_sha1		917
#define NID_aes_256_cbc_hmac_sha1		918

#endif  /* __XB2NET_CH */


