// ---------------------------
// Xb2.NET sample webshop checkout using PayUnity service
// https://payunity.docs.oppwa.com/
// ---------------------------

#define _BASEURL        "https://test.oppwa.com/"
#define _AUTH_USERID    "8a8294174e0078ad014e1a2781e035ab"
#define _AUTH_PASSWORD  "8Dy7awrPX3"
#define _AUTH_ENTITYID  "8a8294174e0078ad014e1a2781d035a7"

#pragma library("xb2net.lib")
#include "xb2net.ch"
#include "common.ch"
#include "error.ch"

STATIC oSSL

// ---------------------------
// Create an SSL context that will later be used when communicating with PayUnity
// This procedure is automatically called when the script/DLL is loaded

INIT PROCEDURE CreateSSLContext()
   oSSL := xbSSLContext():New(TLS_client_method)
   oSSL:SetOptions(SSL_OP_NO_SSLv3)
   Return

// ---------------------------
// Don't forget to free memory associated with the xbSSLContext instance
// This procedure is automatically called when the DLL is unloaded or the application terminates

EXIT PROCEDURE DestroySSLContext()
   if oSSL != NIL
      oSSL:Destroy()
   endif
   Return

// ---------------------------
// This is the script main procedure

PROCEDURE Checkout()
   Local cID
   Local oClient := ThreadObject()
   Local cAmt := oClient:GetVar("AMT")
   Local cCur := oClient:GetVar("CUR")

   if empty(cAmt) .or. val(cAmt) <= 0 .or. empty(cCur)
      // display input form to get amount and currency
      oClient:HTTPResponse:Content := PrepareCheckoutHtml()
   else
      cID := PrepareCheckout(val(cAmt), cCur)
      oClient:HTTPResponse:Content := CreatePaymentFormHtml( val(cAmt), cCur, cID )
   endif
   Return

// ---------------------------
// This is the callback function called from PayUnity after the payment is completed.
// Here we simply display the result, but if it was a real transaction we would save
// the result in a database and initiate the order fulfillment process.

PROCEDURE CheckoutResult()
   Static cHtml
   Local oURI, oHTTP, oResp
   Local oClient := ThreadObject()
   Local cPath   := oClient:GetVar("resourcePath")

   if cHtml == NIL
      TEXT INTO cHtml TRIMMED
      <!doctype html>
      <html>
      <head>
        <link rel=stylesheet type=text/css href=/style.css>
        <meta name=viewport content="width=device-width, initial-scale=1, user-scalable=no">
      </head>
      <body style="padding-left:8px;"}>
      <h1 style="color:#900; margin:0">Payment result: <!--e--></h1><br>
      <pre class=green>%result%</pre>
      </body>
      </html>
      ENDTEXT
   endif

   if empty(cPath)
      oClient:HTTPResponse:Content := strtran(strtran(cHtml, "<!--e-->", "(ERROR)"), "%result%", oClient:HTTPRequest:AsString())
      Return
   endif

   oURI := xbURI():new(_BASEURL + cPath)
   AddMyCredentials(oURI)

   oHttp := xbHTTPClient():new()
   oHttp:SetSSLContext(oSSL)

   oResp := oHttp:Execute(oURI)

   if oResp == NIL
      RaiseError("PayUnity", oHttp:ErrorCode, oHttp:ErrorSource)
   elseif oResp:StatusCode != 200
      RaiseError("PayUnity", 0, if(oResp:ContentType == "application/json", oResp:Content, oResp:StartLine))
   elseif empty(oResp:Content)
      RaiseError("PayUnity", 0, "No content received")
   elseif ! "successfully processed" $ oResp:Content
      RaiseError("PayUnity", 0, oResp:Content)
   endif

   oClient:HTTPResponse:Content := strtran(cHtml, "%result%", strtran(oResp:Content,",",","+chr(10)), 1,1)
   Return

// ---------------------------
// Step 1 - the HTML page where user can specify the amount and currency of payment.
// In a typical web shop situation, this step would be bypassed since the payment
// amount would be calculated by adding up all items in the shopping cart.

STATIC FUNCTION PrepareCheckoutHtml()
   Static cHtml
   if cHtml == NIL
      TEXT INTO cHtml TRIMMED
      <!doctype html>
      <html>
      <head>
        <title>checkout_1</title>
        <meta name=viewport content="width=device-width, initial-scale=1, user-scalable=no">
        <link rel=stylesheet type=text/css href=/style.css>
        <style type=text/css>
         .prechk {color:#555; background-color:#f6f6f6; border:dashed 1px #999; margin:25px 10px; padding:25px; width:260px; border-radius:15px; box-shadow:3px 3px 2px #ccc;}
         .prechk label { width:26%; float:left; text-align:right; padding-right:13px; }
         .prechk input, .prechk select { color:#222; width:70px; border-radius:4px; box-shadow:1px 0 #ccc;}
         .prechk input[type=submit] { color:#744; width:70px; font-size:large; float:right; border-radius:8px; box-shadow:3px 3px 2px #ccc;}
          h1 {color:#900; margin:0}
          h3 {color:#666;}
        </style>
      </head>
      <body style="padding-left:8px; max-width:800px;"}>
      <h1>WebShop checkout</h1>
      <div class="prechk">
      <form method="post" name="checkout" id="checkout" onsubmit="prg/checkout.prg">
      <label>Amount</label><input type="text" size=10 name="AMT" onclick="select()" required><br>
      <label>Currency</label>
        <select name="CUR">
        <option value="CAD">CAD</option>
        <option value="CHF">CHF</option>
        <option value="EUR" selected>EUR</option>
        <option value="GBP">GBP</option>
        <option value="USD">USD</option>
        </select>
      <input type="submit" value="Next">
      </form>
      </div>
      <hr><a href=/viewsource?%prg%><hilite>View Source</hilite></a>
      </body>
      </html>
      ENDTEXT
      cHtml := strtran(cHtml, "%prg%", ThreadObject():Parent:GetRelativePath(__FILEPATH__))
   endif
   Return cHtml

// ---------------------------
// Step 2 - the HTML page where user enters credit card info and completes payment

STATIC FUNCTION CreatePaymentFormHtml( nAmt, cCurrency, cCheckoutID )
   Static cHtml
   if cHtml == NIL
      TEXT INTO cHtml TRIMMED
      <!doctype html>
      <html>
      <head>
        <title>checkout_2</title>
        <link rel=stylesheet type=text/css href=/style.css>
        <meta name=viewport content="width=device-width, initial-scale=1, user-scalable=no">
        <script src="%baseurl%v1/paymentWidgets.js?checkoutId=%id%"></script>
        <style type=text/css>
        table {margin:20px auto; background:#f9f9f9; border-radius:10px; box-shadow:3px 4px 2px #ccc;}
        th { color:#383; padding:5px 8px; text-align:left; font-weight:bold; background:#e6e6e6; vertical-align:bottom;}
        td { color:#444; padding:2px 8px; text-align:left; font-weight:normal; background:#f0f0f0;}
        h1 {color:#900; margin:0}
        </style>
      </head>
      <body>
      <center>
      <h1>Checkout Amount: %am% %cu%</h1>
      <form action="https://xb2.net/prg/CheckoutResult" class="paymentWidgets">VISA MASTER AMEX</form>

      <hr>
      <table>
      <tr><th colspan=4>Credit Card Test Accounts</th></tr>
      <tr><th>Card</th><th>Number</th><th>CVV</th><th>Exp Date</th></tr>
      <tr><td>VISA</td><td>4200000000000000</td><td rowspan=2>any 3 digits</td><td rowspan=3>any date after today</td></tr>
      <tr><td>MASTER</td><td>5454545454545454</td></tr>
      <tr><td>AMEX</td><td>377777777777770</td><td>any 4 digits</td></tr>
      </table>
      </center>

      </body>
      </html>
      ENDTEXT
      cHtml := strtran(cHtml, "%baseurl%", _BASEURL)
   endif
   Return strtran(strtran(strtran(cHtml,"%am%",str(nAmt),1,1), "%cu%", cCurrency,1,1), "%id%",cCheckoutID,1,1)

// ---------------------------
// This function will prepare the checkout process by fetching a unique ID from PayUnity server

STATIC FUNCTION PrepareCheckout( nAmt, cCurrency )
   LOCAL cURL := _BASEURL + 'v1/checkouts'
   LOCAL oHTTP, oResp, oSSL, oForm

   DEFAULT cCurrency TO 'EUR'

   oForm := xbForm():new()
   AddMyCredentials(oForm)
   oForm:SetVar('amount'     , LTrim(Str((nAmt))))
   oForm:SetVar('currency'   , cCurrency)
   oForm:SetVar('paymentType', 'PA')

   oHttp := xbHTTPClient():new()
   oHttp:SetSSLContext(oSSL)

   oResp := oHttp:Execute(cURL,,oForm)

   if oResp == NIL
      RaiseError("PayUnity", oHttp:ErrorCode, oHttp:ErrorSource)
      Return NIL
   elseif oResp:StatusCode != 200
      RaiseError("PayUnity", 0, if(oResp:ContentType == "application/json", oResp:Content, oResp:StartLine))
      Return NIL
   elseif empty(oResp:Content)
      RaiseError("PayUnity", 0, "No content received")
      Return NIL
   elseif ! "successfully created" $ oResp:Content
      RaiseError("PayUnity", 0, oResp:Content)
      Return NIL
   endif
   Return GetJSONValue(oResp:Content, "id")

// ---------------------------
STATIC PROCEDURE AddMyCredentials( oForm )
   oForm:SetVar('authentication.userId'  , _AUTH_USERID  )
   oForm:SetVar('authentication.password', _AUTH_PASSWORD)
   oForm:SetVar('authentication.entityId', _AUTH_ENTITYID)
   Return

// ---------------------------
// simple function to retrieve a single JSON variable

STATIC FUNCTION GetJSONValue( cJSON, cName )
   Local i,j

   if ! '"' $ cName
      cName := '"' + cName + '"'
   endif
   if (i := at(cName, cJSON)) == 0
      Return NIL // not found
   endif
   if (i := at(':', cJSON, i+len(cName))) == 0
      Return NIL // should never happen if valid JSON string
   endif
   if (i := at('"', cJSON, i+1)) == 0
      Return NIL // should never happen if valid JSON string
   endif
   if (j := at('"', cJSON, i+1)) == 0
      Return NIL // should never happen if valid JSON string
   endif
   Return substr(cJSON, i+1, j-i-1)

// ---------------------------
STATIC PROCEDURE RaiseError( cSubSystem, nCode, cDescription, cOperation, nSeverity )
   Local oError := Error():new()

   DEFAULT nCode      TO 0
   DEFAULT nSeverity  TO XPP_ES_ERROR
   DEFAULT cOperation TO ProcName(1)

   oError:subSystem   := cSubSystem
   oError:subCode     := nCode
   oError:description := cDescription
   oError:operation   := cOperation
   oError:severity    := nSeverity
   oError:osCode      := DosError()

   Eval(ErrorBlock(), oError)
   Return


