// ---------------------------
// Xb2.NET sample showing how to maintain state persistence in a PRG script
// ---------------------------

#pragma library("xb2net.lib")

// We save the state info in a data file using var2bin / bin2var
#define DATA_FILE  "StatePersistence.bin"

// Global-static variable keeps state information in memory
STATIC saData

// ---------------------------
// This procedure is automatically called when the script/DLL is loaded

INIT PROCEDURE LoadData()
   if FExists(DATA_FILE, "-D-S+H")
      saData := bin2var(MemoRead( DATA_FILE ))
   else
      saData := {0, nil, nil}
   endif
   Return

// ---------------------------
// This procedure is automatically called when the DLL is unloaded or the application terminates

EXIT PROCEDURE SaveData()
   xbSaveToFile( var2bin(saData), DATA_FILE, .f. )
   Return

// ---------------------------
// This is the script main procedure.
// It causes the DLL to be loaded the first time the script is called by a web client.

PROCEDURE StatePersistence()
   Static cHtml
   if cHtml == NIL
      TEXT INTO cHtml WRAP chr(10) TRIMMED
      <!doctype html>
      <html>
      <head>
        <meta name=viewport content="width=device-width, initial-scale=1, user-scalable=no">
        <link rel=stylesheet type=text/css href=/style.css>
        <style type=text/css>
          h1 {color:#900;}
          table {margin:20px 0; background:#eee; border:dashed 1px #999; border-radius:10px; box-shadow:3px 4px 2px #ccc;}
          th { color:#383; padding:5px 8px; text-align:left; font-weight:bold; background:#ddd; vertical-align:bottom;}
          td { color:#444; padding:2px 8px; text-align:left; font-weight:normal; background:#f0f0f0;}
          input { color:#744; background-color:#ddd; float:right; margin:9px 5px; padding:3px 8px; border-radius:8px; box-shadow:3px 4px 2px #ccc;}
        </style>
      </head>

      <body style="padding-left:8px; max-width:800px;"}>
      <h1>State Persistence</h1>
      This sample shows one way of maintaining state persistence when the HTTP server is restarted or when
      a PRG script is modified and it's DLL is reloaded. This script contains a global STATIC variable that
      keeps track how many times the page is loaded plus the date & time of last access.
      <p />
      Reload page to see the counter increment.

      <table>
      <tr><th>Load Count</th><th>Last Access</th></tr>
      <tr><td>%n%</td><td>%d%</td></tr>
      <tr><td colspan=2>
      <input type="button" value="Reload Page" onclick="location.reload()" />
      </td></tr>
      </table>

      <hr><a href=/viewsource?%prg%><hilite>View Source</hilite></a>
      </body>
      </html>
      ENDTEXT
      cHtml := strtran(cHtml, "%prg%", ThreadObject():Parent:GetRelativePath(__FILEPATH__))
   endif

   // increment page load count
   ++saData[1]

   // generate HTML page
   ThreadObject():HTTPResponse:Content := strtran(strtran(cHtml,"%n%", str(saData[1])), "%d%", xbGMTDate(saData[2],saData[3]))

   // save current date/time as last access
   saData[2] := date()
   saData[3] := seconds()
   Return


